* mer02_all.do
* Sampling - all calculations
*
	clear all
	version 12
	set more off
	cd c:\mer\ch02\data		/* this line is computer dependent */
	capture log close mer02_all
	log using mer02_all, name(mer02_all) replace 
	
*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Create the subset of the Brazil data designated for use with this chapter
*************************************************************************
	use brazil.dta, clear
	* keep selected variables
	keep  mun comm fam id cistern fam_n diarr di_cnt di_d di_d_ln age sex water_tx ///
		m_pop_families m_cistern_elig m_cistern_const m_hdi
	* compute proportion in each cistern group sampled
		gen m_cistern_notconst=m_cistern_elig-m_cistern_const
		bysort mun: egen cist=total(cistern)
        bysort mun: egen nocist=total(1-cistern)
		bysort mun: gen psel_cistern=cist/m_cistern_const if cistern==1
		bysort mun: replace psel_cistern=nocist/m_cistern_notconst if cistern==0
			* sort mun
			* br mun comm fam id cistern m_cistern_elig m_cistern_const m_cistern_notconst cist psel_cistern
		replace psel_cistern=1 if psel_cistern>1
			* one municip. had more non-cisterns sampled than we thought were on the "eligible" list
		drop m_cistern_notconst cist nocist 
	* compute proportion of family included in sampling
		bysort fam: gen psel_fam=_N/fam_n
		codebook psel_fam
			* sort mun comm fam
			* br mun comm fam id fam_n psel_fam
		replace psel_fam=1 if psel_fam>1
	* compute sampling weights based on probability of selection of family
		* and probability of individual being included
		bysort fam: gen psel=psel_cistern*psel_fam
		gen wt=1/psel
	* save sampling file
		order  mun comm fam fam_n id cistern psel_cistern psel_fam psel wt age sex ///
			diarr di_cnt di_d di_d_ln water_tx m_hdi m_pop_families
		label var psel_cistern "prob. of being selected list of eligible cisterns"
		label var psel_fam "prob. of being included in the families data"
		label var fam_n "number of people in household"
		label var psel "overall probability of selection"
		label var wt "sampling weight 1/psel "
		drop m_cistern_elig m_cistern_const
		save brazil_smpl.dta, replace

*************************************************************************
* Ex 2.2 - 2.4 Basic calcuations
*************************************************************************
	* compute incidence - simple random sample
		svyset id, clear
		svy: prop diarr
			scalar se_srs=_se[No]
	* computer incidence - stratified sample
		svyset id, strata(mun)
		svy: prop diarr
		svy: prop diarr, over(mun)
		tab mun diarr
	* compute incidence - stratified and sampling weights
		svyset, clear
		svyset id [pweight=wt], strata(mun)
		svy: prop diarr
	* compute incidence - clustering only
		svyset, clear
		svyset fam
		svy: prop diarr
	* compute incidence - stratified, sampling weights and clustering
		svyset, clear
		svyset fam [pweight=wt], strata(mun)
		svy: prop diarr
			scalar se_cl=_se[No]
			di "design effect =  " (se_cl/se_srs)^2
			di "var SRS =  " se_srs^2
			di "var cl =  " se_cl^2
		* alternatively, the design effect can be obtained as the misspecification effects (meff) in ...
			estat effects, deff deft meff meft
			
*************************************************************************
* Ex 2.5 - 2.6 Sample size of people in a water cistern study
*************************************************************************
	sampsi .15 .1, p(.8) nocontinuity 	
	sampsi .15 .1, p(.8) 
	* Sample size with adjustment for clustering
		xtlogit diarr cistern, i(fam)
		sampsi .15 .1, p(.8) nocontinuity 	
		sampclus, rho(0.45) obs(6) 	

*************************************************************************
* Ex 2.7 Power Calculation by Simulation
*************************************************************************
	set seed 123456
	use brazil_smpl.dta, clear
	* create a dichotomous variable for family size (1-6, 6+)
		gen fam_c2=fam_n>=6 & !missing(fam_n)
	* regress -di_d_ln- on -cistern- and -fam-c2-
		reg di_d_ln fam_c2 cistern 
		* compute predicted values
			predict pred, xb
		* generate revised the predicted values to reflect an effect of .07 unit increase in ln(days)
			replace pred=pred - (fam_c2*0.0537) + (fam_c2*0.07) 
	* set up a file to hold the results from the simulation
		tempname memhold
		postfile `memhold' beta z using ss_sim_rslt.dta, replace
	* set up loop to repeat the analyses many times
		local i=0
		while `i'<1000 {
		* generate -di_d_ln_new - normally distributed with mean = pred. val. and SD=0.605
			gen di_d_ln_new=pred+0.605*rnormal()
		* regress di_d_ln_new on the same set of predictors
			 quietly reg di_d_ln_new fam_c2 cistern
  			* post the regression coefficient and z statistic to the results file
			post `memhold' (_b[fam_c2]) (_b[fam_c2]/_se[fam_c2]) 
   	* repeat the loop
		drop di_d_ln_new
		local i = `i'+1
		}
	* stop the process of posting results to a file
		postclose `memhold'
	* open the file that captured the results
		use ss_sim_rslt.dta, clear
	* compute the p-values
		gen pval = 2*(1-normal(abs(z)))
   	* compute and display the power
		count if pval<0.05
		scalar power=r(N)/1000
		display "Power is " %8.3f scalar(power)
  	* compare the computed power to a calculated value
		sampsi 1.14 1.21, sd(.605) n1(337) n2(159)

*************************************************************************
* Ex 2.8 Sample size for freedom from disease
*************************************************************************
di (1-(.05^(1/10)))*(100-((10-1)/2))
