* mer05_all.do
* Sampling - all calculations
*
	clear all
	version 12
	set more off
	cd c:\mer\ch05\data		/* this line is computer dependent */
	* capture log close mer05_all
	* log using mer05_all, name(mer04_all) replace 
	
*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Open the Norovirus testing data designated for use with this chapter
*************************************************************************
	use nv.dta, clear

*************************************************************************
* Ex 5.1 Measuring precision and agreement - quantitative test results
*************************************************************************

* CV
* compute the mean, standard deviations and CV of the 3 sets of EIA results
	egen avg=rowmean(eia1 eia2 eia3)
	egen sd=rowsd(eia1 eia2 eia3)
	gen cv=sd/avg
   	* compute the average CV 
		sum cv 
* Pearson correlation coefficient
	pwcorr eia1 eia2 eia3, obs
* Concordance correlation and plot
	concord eia1 eia2	/* -concord- is a Stata add-on */
	format eia1 eia2 %6.2f
	concord eia1 eia3, ccc(scheme(s1mono) lpattern(solid dash) ///
		ytitle("adjusted OD - sample 1", m(r+2)) ylabel(0.0(1)4.0, angle(horizontal)) ///
		xtitle("adjusted OD - sample 2", m(t+2))) xlabel(0.0(1)4.0, format(%4.2f))
	graph save "figs/fig05_ccc.gph", replace
	graph export "figs/fig05_ccc.emf", replace
* Limits of Agreement plot
	concord eia1 eia3, loa(scheme(s1mono) lopts(lpat(dash solid dash) lcol(gs8 gs0 gs8)) ///
		ytitle("difference between values", m(r+0)) ///
		xtitle("mean value", m(t+2)) legend(off) caption("") ///
		text(.1 3 "observed average difference" -.55 3 "lower limit of agreement" ///
		.55 3 "upper limit of agreement") ylabel(-1(0.2)1, angle(horizontal)))
	graph save "figs/fig05_loa.gph", replace
	graph export "figs/fig05_loa.emf", replace

*************************************************************************
* Ex 5.2 Computing kappa to compare two tests with dichotomous outcome
*************************************************************************

* compute McNemar's chi-square for assessing the positive proportions (PCR and EIA)
	mcc pcr_c2 eiafin_c2
* compute kappa using the dichotomized PCR and EIA results
	kap pcr_c2 eiafin_c2
	* Note SE of kappa which does not assume independence of tests computed in SAS
* compute the confidence interval for kappa
	kapci pcr_c2 eiafin_c2	/* -kapci- is a Stata add-on */
	* compare with bootstrap estimates
	kapci pcr_c2 eiafin_c2, est(bsall) reps(1000) 


*************************************************************************
* Ex 5.3 Agreement among ordinal test results
*************************************************************************

* generate 3 level test results 0=neg, 1=equivocal 2=pos
	recode pcr (.=0 "negative") (1/27=2 "positive") (27/50=1 "equivocal"), gen(pcr_c3)
	egen eiafin_c3=cut(eiafin), at(0 .09 .11 99) icodes
* compute an unweighted kappa
	kap pcr_c3 eiafin_c3, tab
	kapci pcr_c3 eiafin_c3, reps(1000) 
* compute the weighted kappa and confidence interval
	* set up the agreement (weight) matrix
		kapwgt nv_wt 1 \ 0.5 1 \ 0 0.5 1 
	kap pcr_c3 eiafin_c3, wgt(nv_wt) tab
	kapci pcr_c3 eiafin_c3, reps(1000) wgt(nv_wt)

*************************************************************************
* Ex 5.4 Computing Se and Sp 
*************************************************************************

	diagt gs em			/* EM */	/* -diagt- is a Stata add-on */
	diagt gs pcr_c2		/* PCR */
	diagt gs eiafin_c2	/* EIA */


*************************************************************************
* Fig 5.4 Generating a figure for selecting a cutpoint
*************************************************************************

local fnyx "0.04 8"
local fpyx "0.07 43"
twoway ///
  (function normalden(x,32,3), range(20 28) recast(area) fcolor(gs14) lwidth(none)) ///
  (function normalden(x,32,3), range(20 44) xline(28)) ///
  (function normalden(x,20,5), range(28 40) recast(area) fcolor(gs13) lwidth(none)) ///
  (function normalden(x,20,5), range(0 40) yline(0) xline(28)) ///
  (pcarrowi `fnyx' 0.01 26) /// need to split off text to have multiline labels
  (pcarrowi `fpyx' 0.01 29, lcolor(black) mcolor(black)) , ///
  xlabel(0(10)50, ticks) scheme(s1mono) plotregion(lcolor(none)) legend(off) ///
  text(`fnyx' "false" "negatives", place(N))  ///
  text(`fpyx' "false" "positives", place(N))  ///
  text(.1 27 "cutpoint", orientation(vertical)) ///
  text(.145 5 "healthy") text(.145 35 "diseased") 
	graph save "figs/fig05_cutpoint.gph", replace
	graph export "figs/fig05_cutpoint.emf", replace
	
* determining if EIA values overlap in GS+ and GS- population
	bysort gs: sum eiafin

*************************************************************************
* Fig 5.5 Generating a Se - Sp plot (2 graph ROC)
*************************************************************************

/* this section uses the add-on command -roctg2-
	at the time of writing - roctg2 was not widely avaialable
	but can be obtained from the author Michael Reichenheim - michael@ims.uerj.br */
use nv.dta, clear
	keep if eiafin<1
	roctg2 gs eiafin, cont nograph saving("figs/fig05_sesp_plot") replace 
* generate the plot
	use "C:\mer\ch05\data\figs\fig05_sesp_plot.dta", clear
	twoway (line sens score, sort lcolor(black) lpat(solid)) ///
		(rarea sens_l sens_u score, sort lcolor(black) lpat(dash) fcolor(none)) ///
		(line spec score, sort lcolor(black) lpat(solid)) ///
		(rarea spec_l spec_u score, sort lcolor(black) lpat(dash) fcolor(none)), ///
		ytitle("sensitivity / specificity (%)") ytitle(, margin(small)) ylab(0(20)100, angle(horizontal)) ///
		xtitle(cutpoint OD) xtitle(, margin(small)) ///
		legend(off) scheme(s1mono) text(90 0.4 "specificity", place(NE)) ///
		text(45 0.5 "sensitivity", place(NE)) 
use nv.dta, clear

*************************************************************************
* Fig 5.6 ROC curves
*************************************************************************
gen eiafin2=eiafin
replace eiafin2=0.001 if eiafin2<0.001
gen eia_ln=ln(eiafin2)
* fit a parametric ROC curve
	rocfit gs eia_ln, continuous(20) 
* generate a plot of both non-parametric and parametric ROC curves
	rocplot , confband scheme(s1mono) plotopts(msymbol(o) msize(small) mcol(black)) ///
		lineopts(lcolor(black) lpat(dash)) ciopts(fcolor(gs16) lcolor(gs8) ) ///
		ytitle(, m(r+2)) ylab(0(.20)1, angle(horizontal)) xtitle(, m(t+2)) ///
		note("Area under curve = 0.955,    SE(area)=0.015") name(rocfit_ln, replace)
	graph save "figs/fig05_roc.gph", replace
	graph export "figs/fig05_roc.emf", replace
* confirm that non-parametric curves for original data and log transformed are identical
	roctab gs eiafin, graph name(roctab_orig, replace)
	roctab gs eia_ln, graph name(roctab_ln, replace)
*************************************************************************
* Ex 5.7 Likelihood ratios
*************************************************************************

* generate various cutpoints for -eiafin-
	gen e_00=eiafin>=-1 & !missing(eiafin)
	gen e_01=eiafin>=0.01 & !missing(eiafin)
	gen e_05=eiafin>=0.05 & !missing(eiafin)
	gen e_10=eiafin>=0.10 & !missing(eiafin)
	gen e_30=eiafin>=0.30 & !missing(eiafin)
	gen e_70=eiafin>=0.70 & !missing(eiafin)
	gen e_150=eiafin>=1.50 & !missing(eiafin)
	
* Cutpoint specific LR
	* compute Se and Sp at various cutpoints
	* diagt gs e_00   /* no T-, Se=100%, Sp = undefined */
	diagt gs e_01
	diagt gs e_05
	diagt gs e_10
	diagt gs e_30
	diagt gs e_70
	diagt gs e_150

* Category specific LR (categories using cutpoints as left ends of ranges)
* obtain the categories used for cutpoints
	egen eiafin_c7=cut(eiafin), at(-1 .01 .05 .10 .30 .70 1.5 99)
* derive likelihoods
	tab eiafin_c7 gs , col
		* divide col% for PI+ by col% for PI- to get LR


*************************************************************************
* Ex 5.8 Parallel and serial interpretation
*************************************************************************

* cross-tabulate PCR, EIA
	bysort gs:tab pcr_c2 eiafin_c2, row col
* generate results for series interpretation
	gen series = (pcr_c2 & eiafin_c2) if !missing(pcr_c2, eiafin_c2)
		diagt gs series
* generate results for parallel interpretation
	gen parallel = (pcr_c2 | eiafin_c2) if !missing(pcr_c2, eiafin_c2)
		diagt gs parallel
* computing covariance
	di "D+   " 68/82 - (.866*.963)
	di "D-   " 88/106 - (.972*.858)

*************************************************************************
* Ex 5.11 Evaluations based on the composite Reference Standard
*************************************************************************

* evaluating eia with pcr as a reference test and virus isol. (vi) as the resolver
	* compute the CRS psuedo-gold standard (pcr2=reference vi=resolver)
		gen crs = (eiafin_c2 | pcr_c2) if !missing(eiafin_c2, pcr_c2)
	* tabulate EIA and PCR
		tab em eiafin_c2
		tab em pcr_c2 if eiafin_c2==0
	* evaluate em
		diagt crs em

*************************************************************************
* Ex 5.12 Generating the tabular data required for TAGS
*************************************************************************

* split the data into 2 populations
	set seed 87654
	sort sp_id
	gen rand=runiform()
	sort gs rand
	gen pop=1
	replace pop=2 if (gs==0 & rand<.75) | (gs==1 & rand<.25)
	tab gs pop, col

* generate data for 2 tests (pcr_c2, eiafin_c2), 2 populations
	* these data not used in text
	bysort pop: tab pcr_c2 eiafin_c2
	
* generate data for 3 tests (pcr_c2, eiafin_c2, em), 2 populations 
	bysort pop: table pcr_c2 eiafin_c2 em
	
* LCM model run using TAGS at http://www.epi.ucdavis.edu/diagnostictests/QUERY.HTM

*************************************************************************
* Ex 5.13 Evaluation of the effects of population of origin
*************************************************************************

* logistic regression analysis of EIA
	* unconditional estimate of Se and Sp
		diagt gs eiafin_c2
	* (confirming same results as obtained from the 2X2 table)
		logit eiafin_c2 if gs==1
			display invlogit(_b[_cons])
		logit eiafin_c2 if gs==0
			display invlogit(_b[_cons])

	* evaluate the effect of population on Se and Sp of EIA
		* use -gs- to classify individuals as D+ and D-
		logit eiafin_c2 i.pop if gs==1
			* can't do in the GS=0 population as there are no + EIA results in the low prevalence pop.
			
log close mer05_all
