* MER
* ch13_all.do
* Confounding - all calculations
*
	version 12
	clear all
	set more off 
	cd c:\mer\ch13\data		/* this line is computer dependent */
	capture log close 
	log using mer13_all, replace text

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analyses of the dataset provided
*/


*************************************************************************
* Ex 13.7 Mantel-Haenszel - little or no confounding - hypothetical CRD data
*************************************************************************
use "C:\mer\data\crd.dta", clear
	* create crude association table of BRD with MHSC
	* use epitab to create tables and measures of association
	cc crd strep
	* create the tables by level of IBR
	bysort rsv: tab crd strep, row
	* create the adjusted MH OR
	cc crd strep, by(rsv)

*************************************************************************
* Ex 13.8 Mantel-Haenszel - confounding present
*************************************************************************
	* create the tables by level of Province
	bysort city: tab crd strep, row
	* create the adjusted MH OR
	cc crd strep, by(city)

*************************************************************************
* Section 13.6 Multivariable control of confounders
*************************************************************************
	* fit logistic models with and without -rsv- included
	logit crd strep, or
	logit crd strep rsv, or

*************************************************************************
* Ex 13.9 Mantel-Haenszel - interaction
*************************************************************************
use c:\mer\data\mi.dta, clear
	* create a variable that represents death at less then 100 days after admission
		gen d100 = surv_mi <= 100
		replace d100 = . if missing(surv_mi) & died==1
	* remove observations cenesored before 100 days
		drop if surv_mi<100 & died==0
	* save the new dataset (it will be used again in this chapter)
		save mi100, replace
	* compute the stratum specific and adjusted MH OR
	cs d100 card, by(prmi)
	bysort prmi: tab d100 card 

*************************************************************************
* Section 13.7.1 Open the first teaching data set from chapter 1 
*************************************************************************
* create files specifically for indirect standardization
use vacc_factual.dta, clear
	* create and save Table 13.4
	collapse (count) pop=subject (sum) cases=d, by(c e)
	bysort e: egen totcases=total(cases)
	save ind_vacc_summ.dta, replace

*************************************************************************
* Section 13.7.1 Standardize the rates
*************************************************************************
	use vacc_factual.dta, clear
	* create the std pop rates based on non-exposed subjects
	tab c d if e==0, row
	*std pop file called "ind_stdpop"
	* create summary from vacc_factual
	bysort c: tab e d, row
	* summary file called ind_vacc_summ 
	use ind_vacc_summ, clear
	istdize totcases pop c using ind_stdpop, by(e) rate(jth_rate crude_rate)
	* the adjusted (standardized) rates are equal

*************************************************************************
* Section 13.7.2 Create marginal structural model based on total population
*************************************************************************
	use vacc_factual.dta, clear
	collapse (count) count=subject, by(c e d)
	* create probability of exposure by confounder level
	logit e c [fw=count]
	predict p, p
	* create probability of non-exposure in non-exposed
	replace p=1-p if e==0
	* gen total weight and then create total psuedo-population
	gen wtot=1/p
	gen pseudopop_tot=count*wtot
		* perform analysis
		tab e d [fw= pseudopop_tot], col
		cs d e [fw= pseudopop_tot]
		* proporions of disease by exposure are equal
		* RR = 1

*************************************************************************
* Section 13.7.2 Create MS model based on exposed as the standard
*************************************************************************
	use vacc_factual.dta, clear
	collapse (count) count=subject, by(c e d)
	* create probability of exposure by confounder level
	logit e c [fw=count]
	predict p, p
	* gen exposed and unexposed weights and then create exposed psuedo-population
	gen wexp=1 if e==1
	replace wexp=p/(1-p) if e==0
	gen pseudopop_exp=count*wexp
		* perform analysis
		tab e d [fw= pseudopop_exp], col
		cs d e [fw= pseudopop_exp]
		* proporions of disease by exposure are equal
		* RR = 1

*************************************************************************
* Example 13.10 Propensity scores
*************************************************************************
use mi100.dta, clear

	* first compute unconditional association between -brd- and -mhcysc- 
		cs d100 ptca, or
	* compute propensity scores (based on -brsvsc-, -hssc-, prov and wt0
	  	xi:pscore ptca sex age white bmi prchf prmi prcabg, pscore(ps) logit comsup ///
			blockid(block) numblo(20) 
			* use -detail- option to find variable/block not balanced
		tab block, sum(ps)
	* recompute unconditional assoc. using obs. with common support
		cs d100 ptca if comsup==1, or
		* unconditional risk difference is -.175
		* unconditional OR is 0.236
*************************************************************************
* Ex 13.10 cont'd matching using propensity scores
*************************************************************************
	* nearest neighbour matching restricted to area of common support
		attnd d100 ptca ,  comsup pscore(ps) detail
			* appears to reduce effect of -ptca- (att=-0.073)
	* radius matching based on pscore in area of common support
		* note takes a long time to run this command
		attr d100 ptca , comsup pscore(ps) radius(0.02)
			* reduces effect of -ptca- (att=-0.128)
			* but not as much as nearest neighbour matching
*************************************************************************
* Ex 13.11 using pscore in a stratified analysis
*************************************************************************
	atts d100 ptca, pscore(ps) blockid(block) comsup detail
	* reduces effect of -ptca- (att=-0.085) almost as much as n.n. matching
*************************************************************************
* Ex 13.12 using pscore in a logistic regression model
*************************************************************************
	* unconditional OR
		logit d100 ptca, or				/* OR = 0.212 */
			estimates store uncon
	* OR adjusted for ps - either continuous or categorical
		logit d100 ptca ps if comsup, or				/* OR = 0.389 */
			estimates store ps_cont
		logit d100 ptca i.block if comsup, or	/* OR = 0.393 */
			estimates store ps_cat
	* comparison with direct control of confounders
		  logit d100 ptca sex age white bmi prchf prmi prcabg if comsup, or 	/* OR = 0.386, p=0.001 */
			estimates store cond
	estimates table uncon ps_cont ps_cat cond, eform

		  
*************************************************************************
* Sensitivity analysis - unmeasured confounders
*************************************************************************
use mi100.dta, clear
	cs d100 white
* sensitivity analysis ... assuming:
	* above avg. socio-economic status cuts risk of death by approximately 0.5
		* actual value sampled from a triangular distribution low=0.3, expected=0.5, high=0.7
	* 60% of whites and 40% of non-whites are high SES
	episens d100 white, drrcd(tri(.3 .5 .7)) dpexp(c(.6)) dpunexp(c(.4)) study(cs) reps(100)
	episens d100 white, drrcd(c(.5)) dpexp(c(.6)) dpunexp(c(.4)) study(cs) reps(100)
