* MER
* ch14_all.do
* Linear regression - all calculations
*
	version 12
	clear all
	set more off 
	cd c:\mer\ch14\data		/* this line is computer dependent */
	capture log close 
	log using mer14_all, replace text

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Open the data 
*************************************************************************

use c:\mer\data\bw5k.dta, clear

*************************************************************************
* Ex 14.1 - 14.2 Simple linear regression models - birthweight as outcome
*			- gestation length as continuous predictor (limited to <=40 weeks)
*			- multiple births as dichotomous predictor
*************************************************************************
	* continuous predictor
		summ bwt, d
		regress bwt gest 
		* generate prediction SE (of mean and individual)
		predict SE_mean, stdp
		predict SE_ind, stdf
		* br gest SE_mean SE_ind
	* prediction intervals
		twoway (scatter bwt gest, msize(small)) ///
			(lfitci bwt gest, ciplot(rline) blcolor(black) blpattern(dash)) ///
			(lfitci bwt gest, stdf ciplot(rline) blcolor(black) blwidth(medthick) ///
				blpattern(shortdash_dot_dot)), ///
			legend(off) scheme(s1mono) ytitle("birth weight (gm)", m(r+2)) /// 
			ylabel(, angle(horizontal) format(%4.0f)) xtitle("length of gestation (weeks)", m(t+2)) ///
			plotregion(style(none)) 
	graph save "figs/fig14_predict.gph", replace
	graph export "figs/fig14_predict.emf", replace
	* dichotomous predictor
		regress bwt multbrth

*************************************************************************
* Ex 14.3 Assessing the overall significance of multiple predictors
*************************************************************************
	* regression adding 3 variables describing maternal smoking
	reg bwt gest cig_1 cig_2 cig_3
		test cig_1 cig_2 cig_3
		
*************************************************************************
* Ex 14.4 Modifying (scaling) the predictors
*************************************************************************
	* scaling gestation length
		codebook gest
		gen gest39=gest-39
		reg bwt gest39
	* scaling cigarette consumption (convert to packs/day) - not in text
		gen cig_2p=cig_2/20
		reg bwt cig_2p

*************************************************************************
* Ex 14.6 - 14.7 Coding indicator variables
*************************************************************************
	* coding indicator variables
		codebook meduc_c4
		reg bwt gest i.meduc_c4
		testparm  i.meduc_c4
	* hierarchical coding
		reg bwt gest i.meduc_c4	/* indicator coding */
			estimates store indic
		xi3:reg bwt gest b.meduc_c4	/* hierarchical coding */
			estimates store hier
			* note xi3 is a a Stata add on which needs to be downloaded
		estimates table indic hier
	* note, with Stata 12 an alternative to obtain heirarchical coded estimates
		* is to use indicator coding and a contrasts command
		reg bwt gest i.meduc_c4	/* indicator coding */
			contrasts ar.meduc_c4, nowald
		
*************************************************************************
* Ex 14.8 Modelling correlated predictors - VIF
*************************************************************************
	* correlations among the cigarette variables
	corr cig_1 cig_2 cig_3
	reg bwt cig_1 cig_2 cig_3
	* quadratic regression without centring
		reg bwt gest
		gen gest_sq=gest^2 /* creating a squared scaled term */
		corr gest_sq gest
		reg bwt gest gest_sq
		estat vif
	* quadratic regression with centring
		gen gest39_sq = gest39^2
		corr gest39 gest39_sq
		reg bwt gest39 gest39_sq
		estat vif
	
*************************************************************************
* Ex 14.9 - 14.11 Interactions
*************************************************************************
	* two dihcotomous predictors
		* create dichotomous variables for weight gain and tbo
		gen byte wtgain_c2 = wtgain >=30 if !missing(wtgain)
		gen byte tbo_c2 = tbo >=2 if !missing(tbo)
		reg bwt wtgain_c2 tbo_c2
		reg bwt i.wtgain_c2##tbo_c2
		estat vif
		* a dichotomous and a continuous predictor
		gen wtgain_ct=wtgain-31
		reg bwt tbo_c2 wtgain_ct
		reg bwt i.tbo_c2##c.wtgain_ct
		* graph the predicted values
			predict pv, xb
			sort wtgain
			twoway (line pv wtgain if tbo_c2==0, lcol(black) lpat(dash)) ///
				(line pv wtgain if tbo_c2==1, lcol(black) lpat(solid)) ///
				(scatter bwt wtgain if bwt>=2000 & bwt<=5000, ///
					ms(o) msize(vsmall)), ///
				xtitle("weight gain (lb)", m(t+2)) ///
				ytitle("birth weight (gm)", m(r+2)) ylabel(, angle(horizontal)) ///
				legend(off) scheme(s1mono) plotregion(style(none)) ///
				text(3800 70 "parity=1", place(NE)) text(3100 70 "parity=2+", place(NE))
			graph save "figs/fig14_interact.gph", replace
			graph export "figs/fig14_interact.emf", replace
		drop pv
	* two continuous predictors
		reg bwt c.wtgain##c.tbo
			margins, at(wtgain=(0 100) tbo==(1(1)4))
			marginsplot, noci title("") recast(line) ///
				xtitle("weight gain (lb)", m(t+2)) xlab(0(20)120) ///
				ytitle("birth weight (gm)", m(r+2)) ylabel(, angle(horizontal)) ///
				legend(off) scheme(s1mono) plotregion(style(none)) ///
				text(3775 102 "parity=1", place(NE)) text(3725 102 "parity=2", place(NE)) ///
				text(3675 102 "parity=3", place(NE)) text(3625 102 "parity=4", place(NE))
			graph save "figs/fig14_interact2.gph", replace
			graph export "figs/fig14_interact2.emf", replace


*************************************************************************
* Ex 14.12 Causal model and multivariable model for bwt
*************************************************************************
	* fit a multivariable model
		* to simplify things we will limit analyses to single births
		* -wtgain- and -gest- are intervening variables for the effects of other factors so 
		* they are being excluded
		gen white=(mrace_c3==2) if !missing(mrace_c3)
			label define white_lbl 0 "non-white" 1 "white" 
			label val white white_lbl
		gen byte college = meduc_c4 >= 3 if !missing(meduc_c4)
			label define college_lbl 0 "<college" 1 "college" 
			label val college college_lbl
		
		reg bwt white college tbo cig_2 if multbrth==0
			reg bwt i.white##c.cig_2 college tbo if multbrth==0	/* intn. not sig. P=0.117 
				but retain it for interpretation purposes */
			reg bwt i.tbo##c.cig_2 college white if multbrth==0
				testparm tbo#c.cig_2	/* intn. not sig. P=0.240*/
			reg bwt i.college##c.cig_2 white tbo if multbrth==0	/* interaction not sig. P=0.127 */
			sum bwt if multbrth==0
		
*************************************************************************
* Ex 14.13 - 14.14 Evaluation of model
*************************************************************************
	* base all evaluations on the following model model
		reg bwt white college cig_2 gest
	* compute residuals and diagnostic statistics
		capture drop pv rsta rstu
		predict pv, xb
		predict rsta, rsta
		predict rstu, rstu
	* evaluation of predictive ability
		* r2 = 28%
	* evaluating homscedasticity
		twoway scatter rsta pv, xtitle(, m(t+2)) ytitle("standardised residuals", m(r+2)) ///
		ylabel(, angle(horizontal)) xtitle("linear prediction", m(t+2)) ///
			scheme(s1mono) plotregion(style(none)) msize(small)
		graph save "figs/fig14_het.gph", replace
		graph export "figs/fig14_het.emf", replace
		sum pv, detail
		estat hettest
		egen pv_c5=cut(pv), at(0 2500 3000 3500 4000 9999)
		tab pv_c5,sum(rsta)
		drop pv_c5
		* significant heteroscedasticity - but different than what appears in plot
			* greatest variance at lowest pv (but few observations in these groups)
	* evaluating normality
		gen rdiff=rsta-rstu
		sum rdiff /* little difference between standardized and studentized */
		histogram rsta, freq bin(20) norm normopts(clw(medthick)) xtitle("standardised residuals", size(large) m(t+2)) ///
			ytitle(, size(large) m(r+2)) ytitle("frequency", size(large) m(r+2)) ///
			ylabel(, labsize(large) angle(horizontal)) xlabel(, labsize(large)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_histo, replace)
		qnorm rsta, xtitle("inverse normal", size(large) m(t+2)) ytitle("standardised residuals", size(large) m(r+2)) ylabel(, labsize(large) angle(horizontal)) ///
			xlabel(, labsize(large))  scheme(s1mono) plotregion(style(none)) name(fig14_norm_qnorm, replace)
		* combine the two graphs)
		graph combine fig14_norm_histo fig14_norm_qnorm, ysize(4) xsize(8) scheme(s1mono)
			graph save "figs/fig14_norm.gph", replace
			graph export "figs/fig14_norm.emf", replace
		swilk rsta
		* visually distribution looks quite good
		* but significant non-normality - excess large neg. and pos. residuals
	drop pv rsta rstu 
	graph drop _all

*************************************************************************
* Section 14.9.3 - Transformation of the outcome
	* see Section 14.9.6 (below) for code for Box-Cox transformation
*************************************************************************

*************************************************************************
* Ex 14.15 Visual evaluation of linearity 
*************************************************************************
reg bwt white college cig_2 gest
	predict pv, xb
	predict rsta, rsta
twoway (scatter rsta gest) (lowess rsta gest, clcol(black) clw(medium)), scheme(s1mono) legend(off) ///
	ytitle("standardised residuals", m(r+2)) ylabel(-4 -2 0 2 4, angle(horizontal) format(%4.2f)) ///
	xlabel(#5, format(%4.0f)) xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none))	
	graph save "figs/fig14_linear.gph", replace
	graph export "figs/fig14_linear.emf", replace
	drop pv rsta
	* effect of gestation length is clearly non-linear
	* generate lowess smoothed curve of effect of gestation length
		twoway (scatter bwt gest, msize(small)) ///
			(lowess bwt gest, clcol(black) clw(medium)) , ///
			xtitle("gestation length (weeks)", m(t+2)) ///
			ytitle("birth weight (grams)", m(r+2)) ylabel(, angle(horizontal)) ///
			legend(off) scheme(s1mono) plotregion(style(none)) 
			graph save "figs/fig14_linearity.gph", replace
			graph export "figs/fig14_linearity.emf", replace
	* adding a quadratic term and re-evaluating linearity, normality and heteroscedasticity
		reg bwt white college cig_2 gest39 gest39_sq
			predict pv, xb
			predict rsta, rsta
		twoway (scatter rsta gest) (lowess rsta gest), scheme(s1mono) legend(off) ///
			ytitle("standardized residuals", m(r+2)) ylabel(-4 -2 0 2 4, angle(horizontal) format(%4.2f)) ///
			xlabel(#5, format(%4.0f)) xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none))	
		* linearity is improved, but still not great
		* evaluating homscedasticity
		rvfplot, xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) msize(small)
		estat hettest
		egen pv_c5=cut(pv), at(0 2500 3000 3500 4000 9999)
		tab pv_c5,sum(rsta)
		* the problem of heteroscedasticity has actually become worse
	* evaluating normality
		histogram rsta, freq bin(20) norm xtitle(, size(large) m(t+2)) ytitle(, size(large) m(r+2)) ///
			ylabel(, labsize(large) angle(horizontal)) xlabel(, labsize(large)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_histo, replace)
		qnorm rsta, xtitle(, size(large) m(t+2)) ytitle(, size(large) m(r+2)) ylabel(, labsize(large) angle(horizontal)) ///
			xlabel(, labsize(large))  scheme(s1mono) plotregion(style(none)) name(fig14_norm_qnorm, replace)
		* combine the two graphs)
		graph combine fig14_norm_histo fig14_norm_qnorm, ysize(4) xsize(8) scheme(s1mono)
		swilk rsta
		* problem of excessively large negative and positive residuals remains
	* issue of functional form of effect of -gest- to be investigated further in Chapter 15
		* for Chapter 14, we will stick with the linear form
		drop pv rsta pv_c5
		graph drop _all
	
*************************************************************************
* Section 14.9.5, Example 14.16 Dealing with issue of heteroscedasticity 
	* and non-normality of resdiuals by using robust SE
*************************************************************************
	reg bwt white college cig_2 gest
		estimates store orig
	reg bwt white college cig_2 gest, robust
		estimates store robust
	estimates table orig robust, se
	* SE of -gest- is approx 50% larger
	* others - not much change
	
*************************************************************************
* Section 14.9.6 Transformations
*************************************************************************
	* run the Box Cox procedure
	boxcox bwt white college cig_2 gest
		* theta = 1.38     
* try a box-cox transformed model
		gen bwt_bc = bwt^r(est)
		reg bwt_bc white college cig_2 gest
			estimates store bc
			estat hettest
		* evaluation of predictive ability
			* r2 has dropped to 25%
		* chi-square for heterosced. has dropped from 174 to 50, but still significant
		* significance of predictors is roughly the same as previous model
	* compute residuals and diagnostic statistics
		capture drop pv rsta
		predict pv, xb
		predict rsta, rsta
	* evaluating normality
		histogram rsta, bin(20) norm xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_histo_ln, replace)
		qnorm rsta, xtitle(, m(t+2)) ytitle(, m(r+2)) ylabel(, angle(horizontal)) ///
			scheme(s1mono) plotregion(style(none)) name(fig14_norm_qnorm_ln, replace)
		* combine the two graphs
		graph combine fig14_norm_histo_ln fig14_norm_qnorm_ln, ysize(4) xsize(8) scheme(s1mono)
			graph save "figs/fig14_norm_bc.gph", replace
			graph export "figs/fig14_norm_bc.emf", replace
		swilk rsta
			drop pv rsta 
			graph drop _all
		* non-normality is still highly significant, but visually the distribution looks OK
		* main problem now is a number of very large positive residuals
		estimates table orig robust bc, se p
		* P-values all very similar
	* computing predicted values - Box-Cox model
		reg bwt_bc white i.college cig_2 gest
		margins i.college, at(gest=(38 42) white=0 cig_2=0) 
		di 70893.38^(1/1.38) 	/* 3273 */
		di 71683.33^(1/1.38) 	/* 3299 */
		di 84999.65^(1/1.38) 	/* 3733 */
		di 85789.59^(1/1.38) 	/* 3758 */
	* computing predicted values - original model
		reg bwt white i.college cig_2 gest
		margins i.college, at(gest=(38 42) white=0 cig_2=0)
		
	* Conclusion - for discussion purposes, revert to model on original scale and with usual SEs
			reg bwt white college cig_2 gest

*************************************************************************
* Ex 14.17 - 14.19 Evaluation of outliers and influential points
*************************************************************************
* refit the model with usual SEs
	reg bwt white college cig_2 gest
		estimates store final
		predict pv, xb
		predict rsta, rsta
		predict rstu, rstu
		predict lev, lev
		predict cooksd, cooksd
		predict dfit, dfit
		predict dfb_white, dfbeta(white)
		predict dfb_college, dfbeta(college)
		predict dfb_cig_2, dfbeta(cig_2)
		predict dfb_gest, dfbeta(gest)
	* examine outliers
		sum rsta , detail
			* individual outliers based on t-statistic with 1564 df and sig at alpha/n
				di 2*5000*ttail(4995, 4.43)	/* P=0.048 */
				display invttail(4995,.025/5000)
			* individuals with residuals > 4.42 or <-4.42 are outliers at two-tailed alpha =0.05 
				*in a dataset of this size (there is one at 4.428)
		* number of standardized residuals outside -2/+2 and -3/+3
			* expect 5% (n=250) outside -2,+2 and 1% (n=50) outside -3,+3
				count if rsta<-2 | rsta>2		/* n=243 */
				count if rsta<-3 | rsta>3		/* n=41 */
			* approx. the number of large residual values as expected
		format bwt pv rsta %5.2f
		format white college cig_2 gest %4.0f
		sort rsta
		list obs bwt white college cig_2 gest pv rsta in 1/5, clean noobs ab(5) nolabel
			* generally full term babies (gest>=36), but small (unexplained) birth weight
		list obs bwt white college cig_2 gest pv rsta in -5/l, clean noobs ab(5) nolabel
			* either short term (<35 wks) babies with normal weight or very large (unexplained) babies
		* no obvious reason for concern
	* examine leverage
		sum lev cooksd dfit
		format lev cooksd dfit %5.3f
		* large leverage values (threshold is 3(k+1)/n = 3*5/5000 = 0.003)
		count if lev>=0.003
				* many (n=156) high leverage values
			sort lev
			list obs bwt white college cig_2 gest pv rsta lev in -5/l, clean noobs ab(5) nolabel
				* babies from high cig.consumption, or very short gestation had high leverage
	* examine influence
		* Cooks Distance values (threshold is 4/5000 = 0.0008)
			count if cooksd>=0.0008
				* many (n=288) influential observations (based on Cooks D)
			sort cooksd
			list obs bwt white college cig_2 gest pv rsta cooksd dfit in -10/l, clean noobs ab(5) nolabel
				* most extreme are smokers or short gestation length
				* evaluate the impact of these observations on the model
					reg bwt white college cig_2 gest if cooksd<0.009
						estimates store red
					estimates table final red, se
					* only substantive change is an increase in the detrimental effect of -cig_2-
						* coef drops from -15 to -18
		* DFITS values (threshold = +/- 2*sqrt(k+1/n) = 0.063)
			count if dfit>=0.063
			count if dfit<=-0.063
				* many (n=292) influential observations (based on dfits)
			sort dfit
			list obs bwt white college cig_2 gest pv rsta cooksd dfit in 1/5,clean noobs ab(5)
			list obs bwt white college cig_2 gest pv rsta cooksd dfit in -5/l,clean noobs ab(5)
				* generally the same observations identified above
		* delta betas (threshold = 2/sqrt(n) = 0.028
			sort dfb_white
				count if dfb_white>0.028 | dfb_white<-0.028	/* n=277 */
				list obs bwt white college cig_2 gest pv rsta dfb_white in 1/10, clean noobs ab(5)
				list obs bwt white college cig_2 gest pv rsta dfb_white in -10/l, clean noobs ab(5)
				* no obvioous patter to influential observations
			sort dfb_cig_2
				count if dfb_cig_2>0.028 | dfb_cig_2<-0.028	/* n=130 */
				list obs bwt white college cig_2 gest pv rsta dfb_cig_2 in 1/10, clean noobs ab(5)
				list obs bwt white college cig_2 gest pv rsta dfb_cig_2 in -10/l, clean noobs ab(5)
				* not surprisingly, babies whose mothers smoked were influential on this coefficient

log close mer14_all

