* MER
* mer15_all.do
* Model building - all calculations
*
	version 12
	clear all
	set more off , permanent
	set memory 20m
	cd c:\mer\ch15\data		/* this line is computer dependent */
	capture log close 
	log using mer15_all, name(mer15_all) text replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Open the data and restrict the observations saved
*************************************************************************

use c:\mer\data\bw5k.dta, clear

*************************************************************************
* Ex 15.1 Cronbach's Alpha
*************************************************************************
* without standardizing ... just the simple average of the three cig measures
	alpha   cig_1 cig_2 cig_3,  item gen(cig_avg)
	gen cig_avg2=(cig_1 + cig_2 + cig_3)/3
	assert cig_avg==cig_avg2
* with standardizing (to get the avg. inter-item correlation)
	* standardizing adjusts for the fact that the 3 variables have different variances
	alpha   cig_1 cig_2 cig_3,  item gen(cigs) std
		corr cig_avg cigs
		*corr. with simple avg. is 0.995
	
*************************************************************************
* Ex 15.2 Correspondence Analysis   
*************************************************************************
	* generate categorical variables as necessary
		egen bwt_c3=cut(bwt), at(0 3000 3500 9999) icodes
			label define bwt_c3_lbl 0 "light" 1 "med." 2 "heavy"
			label val bwt_c3 bwt_c3_lbl
			* note an alternative approach to generating this variable is:
				recode bwt (3500/max  = 2 "heavy") (3000/3500 = 1 "med." ) ///
					(min/3000 = 0 "light"), gen(bwt_c3_alt)
				* codes go from big to small so that intervals include left 
				*	 endpoint but not the right
		gen white=(mrace_c3==2) if !missing(mrace_c3)
			label define white_lbl 0 "non-white" 1 "white" 
			label val white white_lbl
		gen byte college = meduc_c4 >=3 if !missing(meduc_c4)
			label define college_lbl 0 "no_coll" 1 "college"
			label val college college_lbl
		gen byte smk = cig_2 > 0 if !missing(cig_2)
			label define smk_lbl 0 "non-smoker" 1 "smoker" 
			label val smk smk_lbl
	* carry out a multiple correspondence analysis of 6 variables
		mca  bwt_c3 white college smk, method(joint) norm(principal)
		mcaplot (bwt_c3, msymb(S)) (white, msymb(x)) (college, msymb(Oh))  ///
			(smk, msymb(Th))  , overlay origin legend(off) ///
			mlabangle(45) scheme(s1mono) note("") xlab(#5) ///
			mcol(black) ylabel(#2, angle(horizontal)) ylabel(#5)
			graph save figs/fig15_mca.gph, replace
			graph export figs/fig15_mca.wmf, replace
		* non-smoking and college are associated
			tab college smk, row	/* < 4% of college=1 are smokers */
		* non-white and no_coll are somewhat associated 
			tab white college, row	/* no_coll in 59% vs 31% in non_white and white */
*************************************************************************
* Section 15.6 Evaluating the linearity of the effect of gestation length
* on birth weight
*************************************************************************
	* scatterplot
		* determining 2.5 and 97.5 percentiles of -gest-
			centile  gest, c(2.5 97.5)
				local gest_2_5=r(c_1)
				local gest_97_5=r(c_2)
			format gest %6.0f
		twoway (scatter bwt gest, m(o) msize(vsmall) xline(`gest_2_5', lpat(dash)) /// 
			xline(`gest_97_5', lpat(dash))) (lowess bwt gest, lcolor(black)), scheme(s1mono)  ///
			ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none)) legend(off)
		graph save figs/fig15_scatter.gph, replace
		graph export figs/fig15_scatter.wmf, replace

*************************************************************************
	* Section 15.6.1 smoothed scatterplots
*************************************************************************
	centile  gest, c(2.5 97.5)
		local gest_2_5=r(c_1)
		local gest_97_5=r(c_2)
		format gest %6.0f
		sort gest
	capture drop lowess8
	lowess bwt gest , nograph gen(lowess8)
		twoway line lowess8 gest, lcolor(black) ylab(#4) name(lowess8, replace) ///
			ti("Lowess smoother") subtitle("bandwidth=0.8") note("") ///
			ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none)) ylab(0(1000)4000) ///
			scheme(s1mono) xline(`gest_2_5', lp(dash)) xline(`gest_97_5', lp(dash)) 
	capture drop line8
	lowess bwt gest , nograph gen(line8) noweight
		twoway line line8 gest, lcolor(black) ylab(#4) name(line8, replace) ///
			ti("Running line smoother") subtitle("bandwidth=0.8") note("") ///
			ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none)) ylab(0(1000)4000) ///
			scheme(s1mono) xline(`gest_2_5', lp(dash)) xline(`gest_97_5', lp(dash)) 
	capture drop mean8
	lowess bwt gest , nograph gen(mean8) noweight mean
		twoway line mean8 gest, lcolor(black) ylab(#4) name(mean8, replace) ///
			ti("Running mean smoother") subtitle("bandwidth=0.8") note("") ///
			ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none)) ylab(0(1000)4000) ///
			scheme(s1mono) xline(`gest_2_5', lp(dash)) xline(`gest_97_5', lp(dash)) 
	* influence of bandwidth
		capture drop lowess2
		lowess bwt gest , bw(0.2) nograph gen(lowess2)
			twoway line lowess2 gest, lcolor(black) ylab(#4) name(lowess2, replace) ///
				ti("Lowess smoother") subtitle("bandwidth=0.2") note("") ///
				ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none)) ylab(0(1000)4000) ///
				scheme(s1mono) xline(`gest_2_5', lp(dash)) xline(`gest_97_5', lp(dash)) 
		capture drop line2
		lowess bwt gest , bw(0.2) nograph gen(line2) noweight
			twoway line line2 gest, lcolor(black) ylab(#4) name(line2, replace) ///
				ti("Running line smoother") subtitle("bandwidth=0.2") note("") ///
				ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none)) ylab(0(1000)4000) ///
				scheme(s1mono) xline(`gest_2_5', lp(dash)) xline(`gest_97_5', lp(dash)) 
		capture drop mean2
		lowess bwt gest , bw(0.2) nograph gen(mean2) noweight mean
			twoway line mean2 gest, lcolor(black) ylab(#4) name(mean2, replace) ///
				ti("Running mean smoother") subtitle("bandwidth=0.2") note("") ///
				ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none)) ylab(0(1000)4000) ///
				scheme(s1mono) xline(`gest_2_5', lp(dash)) xline(`gest_97_5', lp(dash)) 
		* combine 6 graphs
			graph combine mean8 line8 lowess8 mean2 line2 lowess2, ///
				col(3) name(smooth_comb, replace) scheme(s1mono)
			graph save "figs/fig15_smoothed.gph", replace
			graph export "figs/fig15_smoothed.wmf", replace
			graph drop _all
	* adding a confidence interval
		* determining 2.5 and 97.5 percentiles of -gest-
		centile  gest, c(2.5 97.5)
			local gest_2_5=r(c_1)			
			local gest_97_5=r(c_2)		
		sort gest
		capture drop ci8 cise8 ci8ub ci8lb
		* Note - running is a Stata add-on
		running bwt gest, ci gen(ci8) gense(cise8) nograph
			gen ci8ub=ci8+(1.96*cise8)
			gen ci8lb=ci8-(1.96*cise8)
			twoway line ci8 ci8ub ci8lb gest, lpat(solid dash dash) lcolor(black black black) ///
				lwidth(medium medium medium) ylab(#4) ///
				ti("Running line smoother") subtitle("with 95% CI") note("") ///
				ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
				xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none)) ///
				scheme(s1mono) xline(`gest_2_5', lp(dash)) xline(`gest_97_5', lp(dash)) ///
				legend(off)
			graph save "figs/fig15_ci.gph", replace
			graph export "figs/fig15_ci.wmf", replace
			graph drop _all

*************************************************************************
	* smoothed scatterplot on a logistic scale
*************************************************************************
	* generate a dichotomous version of bwt
		gen byte bwt_c2 = bwt >= 2500 if !missing(bwt)
		* determining 2.5 and 97.5 percentiles of -gest-
		centile  gest, c(2.5 97.5)
			local gest_2_5=r(c_1)
			local gest_97_5=r(c_2)
		sort gest
		capture drop lowess_logit
		lowess bwt_c2 gest , nograph gen(lowess_logit) logit
		twoway line lowess_logit gest, lcolor(black) lwidth(medium) ylab(#4) name(lowess_logit, replace) ///
					ti("Lowess smoother") subtitle("bandwidth=0.8") note("") ///
					ytitle("logit of birth weight<2500 gms", m(r+2)) ylabel(#4, angle(horizontal)) ///
					xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none))  ///
				scheme(s1mono) xline(`gest_2_5', lp(dash)) xline(`gest_97_5', lp(dash)) 
			graph save "figs/fig15_logit.gph", replace
			graph export "figs/fig15_logit.wmf", replace

*************************************************************************
	* Section 15.6.2 categorizing the predictor - indicator variables (results not in text)
*************************************************************************
		sum gest, detail
		egen gest_c4=cut(gest), at(0 38 39 40 99)
		tabstat bwt, by(gest_c4) stat( n mean min max)
		reg bwt gest
			estimates store lin
		reg bwt i.gest_c4
			estimates store cat
		estimates stats lin cat


*************************************************************************
	* Section 15.6.3 - 15.6.4 Polynomials
	* (including Examples 15.3 and 15.4)
*************************************************************************
	* polynomial models - linear
		regress bwt gest
			predict pr_lin if e(sample)
			label var pr_lin "Linear"
	* polynomials - quadratic
		gen gest_ct=gest-39		/* centering the variable before squaring it */
		gen gest_sq=(gest_ct^2)/1000  	/* helps reduce collinearity */
			/* dividing by 1000 scales the parameter estimates to keep them reasonable */
		regress bwt gest_ct gest_sq
			predict pr_quad if e(sample)
			label var pr_quad "Quadratic"
	* polynomials - cubic orthogonal
		orthpoly gest, gen(gest_orth*) degree(3)
		regress bwt gest_orth1 gest_orth2 gest_orth3
			predict pr_cubic if e(sample)
			label var pr_cubic "Cubic"
	* polynomials - fractional
		fracpoly reg bwt gest, compare
			predict pr_frac
			label var pr_frac "Fractional"
	* plot linear, quadratic, cubic and fractional polynomial relationships
		* determining 2.5 and 97.5 percentiles of -gest-
		centile  gest, c(2.5 97.5)
			local gest_2_5=r(c_1)
			local gest_97_5=r(c_2)
		sort gest
		twoway line pr_lin pr_quad pr_cubic pr_frac gest, ylab(#4) name(polynomial, replace) ///
			lpat(solid dash dot dash_dot) lwidth(medium medium medthick medium) lcolor(black black black black) ///
			ti("Polynomial functions") ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none))  scheme(s1mono) legend(off) ///
			text(4200 47 "linear", place(NE)) text(3000 47 "cubic", place(NE)) ///
			text(3300 47 "fractional", place(NE)) text(3600 47 "quadratic", place(NE)) ///
			xline(`gest_2_5', lpattern(dash)) xline(`gest_97_5', lpattern(dash)) 
			graph save "figs/fig15_poly.gph", replace
			graph export "figs/fig15_poly.wmf", replace
			graph drop _all

*************************************************************************
	* Section 15.6.5 Splines
*************************************************************************
		capture drop gest_linsp*
		mkspline gest_linsp 4=gest, pctile
			reg bwt gest_linsp*
			predict linsp, xb
		capture drop gest_cubsp*
		mkspline gest_cubsp=gest, cubic knots(36 39 42)
			reg bwt gest_cubsp*
			predict cubsp, xb
		centile  gest, c(2.5 97.5)
			local gest_2_5=r(c_1)
			local gest_97_5=r(c_2)
		sort gest
		twoway line linsp cubsp gest if gest>30, ylab(#4)  ///
			ti("Spline functions") ytitle("birth weight (grams)", m(r+2)) ylabel(#4, angle(horizontal)) ///
			lpat(solid dash ) lwidth(medium medium ) lcolor(black black) ///
			xtitle("gestation length (weeks)", m(t+2)) plotregion(style(none))  scheme(s1mono) legend(off) ///
			text(3550 45 "cubic", place(NE)) text(3325 45 "linear", place(NE)) ///
			xline(`gest_2_5', lpattern(dash)) xline(`gest_97_5', lpattern(dash))
			graph save "figs/fig15_spline.gph", replace
			graph export "figs/fig15_spline.wmf", replace
			graph drop _all

*************************************************************************
* Ex 15.5 Model Selection Procedures
*************************************************************************
	* forward selection model
		xi:stepwise, pe(0.05): regress bwt (i.mrace_c4) mage (i.meduc_c4) mar (i.frace_c3) ///
			(i.fage_c4) tbo previs cig_1 cig_2 cig_3 phyper multbrth gest male 
		estimates store fs
	* backward elimination model
		xi:stepwise, pr(0.05): regress bwt (i.mrace_c4) mage (i.meduc_c4) mar (i.frace_c3) ///
			(i.fage_c4) tbo previs cig_1 cig_2 cig_3 phyper multbrth gest male 
		estimates store be
	* backward stepwise
		xi:stepwise, pr(0.05) pe(0.0499): regress bwt (i.mrace_c4) mage (i.meduc_c4) mar (i.frace_c3) ///
			(i.fage_c4) tbo previs cig_1 cig_2 cig_3 phyper multbrth gest male 
		estimates store swb
	* forward stepwise
		xi:stepwise, pr(0.05) pe(0.0499) forward: regress bwt (i.mrace_c4) mage (i.meduc_c4) mar (i.frace_c3) ///
			(i.fage_c4) tbo previs cig_1 cig_2 cig_3 phyper multbrth gest male 
		estimates store swf
	* using -vselect- to get Mallow's CP (vselect is a Stata add-on)
			xi:vselect bwt (i.mrace_c4)   ///
			(i.fage_c4) tbo previs cig_2 multbrth gest male mage phyper , best
	
	* compare results
		estimates table fs swf be swb
		estimates stats fs swf be swb

*************************************************************************
* Ex 15.6 Regression Model Validation
*************************************************************************
	* generate a random number
		set seed 45678
		capture drop rand pv
		gen rand=uniform()
	* fit the final regression model to all of the data (based on model used for diagnostics in Ch 14)
		reg bwt white college cig_2 gest
			estimates store all
	* fit the final regression model to 1/2 of the data
		reg bwt white college cig_2 gest if rand<0.5
			estimates store half
			predict pv
	* compute the squared correlation between predicted values and observed
	* for the estimation subset and the validation subset
		display "R2 for half the data used to build the model:   " e(r2)
		corr pv bwt if rand>=0.5
		display "R2 for 2nd half of data:   " r(rho)^2
		* R2 rose very slightly from 0.274 to 0.277
	* compare the coefficients
		estimates table all half, p

*************************************************************************
* Ex 15.7 Regression Model Presentation - IQR
*************************************************************************
	* refit the final model 
		reg bwt white college cig_2 gest 
	* display the effect of being White or completing college
		di "effect of white   " _b[white]
		di "effect of white   " _b[college]
	* compute the change from 0 to 10 cigarettes
		* (10 cigarettes was the median # smoked among smokers)
		centile cig_2 if cig_2>0, centile(50)
		di "effect of change in cig_2 from 0 to 10 cigs  " _b[cig_2]*(r(c_1))
	* compute the IQR for gest
		centile gest, centile(25 75)
		di "effect of IQR change in gest   " _b[gest]*(r(c_2)-r(c_1))

log close mer15_all

