* MER
* mer16_all.do
* Logistic regression - all calculations
*
	version 12
	clear all
	set more off , permanent
	set memory 20m			/* not needed for Stata 12 or above */
	cd c:\mer\ch16\data		/* this line is computer dependent */
	capture log close 
	log using mer16_all, name(mer16_all) text replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Open the data - generate a dichotomous outcome (<2500 gm = low birth weight)
*	and recode several predictors used in previous chapters
*************************************************************************

use c:\mer\data\bw5k.dta, clear
	gen low_bw=bwt<2500 & !missing(bwt)
		label define low_bw_lbl 0 "normal" 1 "low" 
		label val low_bw low_bw_lbl
	gen white=(mrace_c3==2) & !missing(mrace_c3)
		label define white_lbl 0 "non-white" 1 "white" 
		label val white white_lbl
	gen smk=cig_2>0 &!missing(cig_2)
		label define smk_lbl 0 "non-smoker" 1 "smoker" 
		label val smk smk_lbl
save low_bw.dta, replace

*************************************************************************
* Fig 16.1 Graph of logit and inverse logit functions
*************************************************************************
	twoway (function ln(x/(1-x)), range(0.001 0.999) lcolor(black)) , scheme(s1mono) ///
		ytitle("logit", m(r+2)) ylabel(-8(2)8, angle(horizontal)) ///
		xtitle("probability", m(t+2)) xlabel(0(0.2)1, format(%4.2f)) plotregion(style(none)) ///
		yline(-4.595, lpat(dash) lwidth(medium)) yline(4.595, lpat(dash) lwidth(medium)) name(fig16_logit1, replace)
	twoway (function exp(x)/(1+exp(x)), range(-7 7) lcolor(black)) , scheme(s1mono) ///
		ytitle("probability", m(r+2)) ylabel(0(.2)1, angle(horizontal) format(%4.2f)) ///
		xtitle("logit", m(t+2)) xlabel(-8(2)8) plotregion(style(none)) ///
		xline(-4.595, lpat(dash) lwidth(medium)) xline(4.595, lpat(dash) lwidth(medium)) name(fig16_logit2, replace)
	graph combine fig16_logit1 fig16_logit2, ysize(4) xsize(8)scheme(s1mono)
		graph save "figs/fig16_logit.gph", replace
		graph export "figs/fig16_logit.emf", replace

*************************************************************************
* Fig 16.2 Graph of log likelihood function
*************************************************************************
	twoway (function ln(comb(10,3)) + 3*ln(x) + 7*ln(1-x), range(0.1 0.6)), ///
		scheme(s1mono) ytitle("log likelihood", m(r+2)) ylabel(-3(1)-1, angle(horizontal)) ///
		xtitle("prevalence", m(t+2)) xlabel(.1(.1).6, format(%4.2f)) plotregion(style(none)) ///
		xline(.3, lpat(dash) lwidth(medium)) 
	graph save "figs/fig16_lnL.gph", replace
	graph export "figs/fig16_lnL.emf", replace


*************************************************************************
* Ex 16.1 - 16.2 Multiple logistic regression and LRT
*************************************************************************
* open the low birth weight data set dataset
use low_bw.dta, clear

* fit a a variety of models
	logit low_bw 				/* null */
		estimates store null 
	logit low_bw smk			/* reduced */
		estimates store reduced			
	logit low_bw smk white i.frace_c3	/* full */
		estimates store full
	estimates stats null full reduced
* likelihood ratio tests
	lrtest full null
	lrtest full reduced
* comparing Walds and LRT for -white- and -frace-c3-
	logit low_bw smk white i.frace_c3
		testparm  white i.frace_c3 
* multiple logistic regression - 3 types of predictors
	* note changing baseline category of frace_c3 to white (=2)
	logit low_bw smk white b2.frace_c3 previs
	logistic low_bw smk white b2.frace_c3 previs		/* results as ORs */
		* testing the 2 father's race coefficients
		testparm  i.frace_c3
		* comparing Hispanics and Blacks
		lincom 1.frace_c3 - 3.frace_c3 
	

*************************************************************************
* Ex 16.3 Effects on the probability scale
*************************************************************************
	logit low_bw i.smk i.white b2.frace_c3 previs
	* compute marginal estimates
	margins smk, at(previs=(0(2)40) white=0 frace_c3==2)
	* plot marginal estimates
	marginsplot, noci recast(line) title("") scheme(s1mono) ///
		ytitle("probability of low birth weight", m(r+2)) ylabel(,angle(horizontal)) ///
		xtitle("# of prenatal visits", m(t+2)) ///
		legend(off) text(.14 12 "smoker", place(ne)) text(.08 12 "non-smoker", place(sw))
	graph save "figs/fig16_pscale.gph", replace
	graph export "figs/fig16_pscale.emf", replace

*************************************************************************
* Ex 16.4 - 16.5 Assessment of confounding and interaction
*************************************************************************
* confounding
	logit low_bw smk white b2.frace_c3 previs
		estimates store full
	logit low_bw previs
		estimates store reduced
	estimates table full reduced, se
	* no evidence of confounding
		regress previs smk white b2.frace_c3
		* white is only factor at all related to -previs- and it has a rather weak 
		* association with -low_bw- 
* interaction
	logit low_bw i.white##c.previs smk i.frace_c3
		* assessing the effect of being White at 11 prenatal visits
			lincom 1.white + 11*c.previs
		* assessing statistical significance of -previs- in Whites
			lincom previs + 1.white#c.previs
	
*************************************************************************
* Assessment of linearity of effect of -previs- (results not included in text)
*************************************************************************
* unconditional relationship
	lowess low_bw previs, logit scheme(s1mono) xtitle("# of prenatal visits", m(t+2)) ///
		ytitle("logit", m(r+2)) ylabel(,angle(horizontal)) ti("") subtitle("") note("") ///
		name(fig16_lin1, replace)
	logit low_bw smk white i.frace_c3 previs
		predict p_res, res
		twoway (scatter p_res previs) (lowess p_res previs, lcolor(black)), ///
			scheme(s1mono) xtitle("# of prenatal visits", m(t+2)) ///
			ytitle("Pearson residual", m(r+2)) ylabel(-2(2)6, angle(horizontal)) legend(off) name(fig16_lin2, replace)
	* combine the 2 graphs
	graph combine fig16_lin1 fig16_lin2, ysize(4) xsize(8)scheme(s1mono)
		graph save "figs/fig16_lin.gph", replace
		graph export "figs/fig16_lin.emf", replace
	* fractional polynomials
		xi:fracpoly logit low_bw previs smk white i.frace_c3, compare
		* need to add a quadratic term to the model
		gen previs_ct=previs-11
		gen previs_sq=previs_ct^2
		logit low_bw smk white i.frace_c3 previs_ct previs_sq
		

*************************************************************************
* Section 16.11 Generalized Linear Models
*************************************************************************
* fit model with ML logistic regression
	logit low_bw smk white b2.frace_c3 previs_ct previs_sq
	estimates store ml1
* fit model with IRLS
	glm low_bw smk white b2.frace_c3 previs_ct previs_sq, link(logit) family(binomial) irls
	estimates store irls
* fit model with ML in GLM framework
	glm low_bw smk white b2.frace_c3 previs_ct previs_sq, link(logit) family(binomial)
	estimates store ml2
* fit model with ML in GLM framework and robust SE
	glm low_bw smk white b2.frace_c3 previs_ct previs_sq, link(logit) family(binomial) vce(robust)
	estimates store robust
* compare models
	estimates table ml1 irls ml2 robust, se

*************************************************************************
* Ex 16.6 - Evaluating a Logistic Regression Model
*************************************************************************
	use low_bw.dta, clear
* all diagnostics will be based on the model with -previs- as a quadratic term
		gen previs_ct=previs-11
		gen previs_sq=previs_ct^2
	logit low_bw smk i.frace_c3 white previs_ct previs_sq
	save low_bw_dx.dta, replace
	* Note: as shown below, once -previs- as entered as a quadratic term, the interaction
		* with -white- is no longer significant and has been removed
		logit low_bw smk i.frace_c3 i.white##c.previs_ct i.white##c.previs_sq 
			testparm i.white#c.previs_ct i.white#c.previs_sq 
	
* examining the covariate patterns
	logit low_bw smk i.frace_c3 white previs_ct previs_sq
	predict cov, num
	predict pv, p
	predict res_p, res
	gen res_pi=(low_bw-pv)/(pv*(1-pv))^.5   
	sort cov
	codebook cov
	* generate a count of the number of obs. in each cov. pattern and a counter
		by cov: gen cnt=_N
		by cov: gen seq=_n
		by cov: egen n_obs=total(low_bw)
		by cov: gen n_obs_p=n_obs/_N
	format cnt seq obs %3.0f
	format n_obs_p res_p %6.3f
	list cov cnt smk frace_c3 white previs pv low_bw n_obs n_obs_p in 1/20, clean noobs
	list cov obs low_bw smk frace_c3 white previs pv res_p res_pi if cov==30, clean noobs

*************************************************************************
* Ex 16.7 - 16.10 Evaluating overall fit
*************************************************************************
	use low_bw_dx.dta, clear
	logit low_bw smk i.frace_c3 white previs_ct previs_sq
		predict cov, num
		* determining number of obs. in each covariate pattern
		preserve
		collapse (count) n=low_bw, by(cov)
		tab n
		restore
* Pearson and Deviance goodness-of-fit tests
	estat gof
	xi:logistic low_bw smk i.frace_c3 white previs_ct previs_sq
		ldev
		* note: add-on program -ldev- requires use of -logistic- and xi:
* Hosmer - Lemeshow Test
	estat gof, g(10) table
* quantifying dispersion
	* binary data
		glm low_bw smk i.frace_c3 white previs_ct previs_sq, link(logit) family(binomial)
			* estimates not valid for binary data
	* binomial data
		preserve
		collapse (count) n=low_bw (sum) pos=low_bw (mean) smk frace_c3 white previs_ct previs_sq, ///
			by(cov)
		glm pos smk white i.frace_c3 previs_ct previs_sq, link(logit) family(binomial n)
		restore
			* Pearson dispersion = 1.17
			
* r2 and other fit statistics
	logit low_bw smk i.frace_c3 white previs_ct previs_sq
	fitstat	/* fitstat is a Stata add-on */
* sensitivity and specificty of logistic model
	estat classification
	* changing the cutpoint
		estat classification, cut(0.07)
* two graph ROC
	lsens, lpattern(solid dash) scheme(s1mono) lcolor(black black) mcolor(black black) ///
		msize(small small) legend(off) ylabel(#5, angle(horizontal) format(%4.2f))  ///
		text(.2 .15 "sensitivity", place(NE)) text(.95 .5  "specificity", place(NE)) ///
		ytitle(, m(r+2)) xtitle(, m(t+2)) plotregion(style(none)) 
	graph save "figs/fig16_2groc.gph", replace
	graph export "figs/fig16_2groc.emf", replace

* standard ROC graph
	lroc , scheme(s1mono) lcolor(black black) mcolor(black black) ///
		ylabel(#5, angle(horizontal) format(%4.2f)) ///
		ytitle(, m(r+2)) xtitle(, m(t+2)) plotregion(style(none)) ///
		note("")
	graph save "figs/fig16_roc.gph", replace
	graph export "figs/fig16_roc.emf", replace


*************************************************************************
* Section 16.12.4 and Ex. 16.8 Overdispersion
*************************************************************************
* simple example
	clear
	set seed 555777
	set obs 10
	gen grp=_n
	gen lv=(grp>=6)
	expand 20
	gen y1 = rbinomial(1,.4)
	gen y2=(grp<=4)
	sum y1 y2
	collapse (mean) y1 y2, by(grp)
	sum y1 y2
	list grp y1 y2, clean noobs

* Example 16.8 - detecting overdispersion
	clear
	set seed 555777
	set obs 10
	gen grp=_n
	gen hp=(grp>=6)	/* identify 5 high prevalence groups */
	gen x=(grp==1 | grp==2 | grp==6 | grp==7)	
				/* group level variable - 2 high and 2 low prev. groups */
	gen xb= -2 + (3*hp) + x
	expand 100
 	gen y = rbinomial(1,invlogit(xb))
	* fit model with group level variable
		glm y x, fam(binomial) link(logit)
		* no evidence of overdispersion
	* collapse the data to grouped (binomial) format and re-analyse
		collapse (mean) x (count) n=y (sum) cases=y, by(grp)
		glm cases x, fam(binomial n) link(logit)
		* clear evidence of extrensive overdispersion


*************************************************************************
* Ex 16.11 - 16.12 Evaluating residuals - outliers and influential points
*************************************************************************
	use low_bw_dx.dta, clear
	logit low_bw smk i.frace_c3 white previs_ct previs_sq
* examining Pearson and Deviance residuals
	predict pv
	predict cov, num
	sort cov
	* generate a count of the number of obs. in each cov. pattern and a counter
		by cov: gen cnt=_N
		by cov: gen seq=_n
		by cov: egen pos=total(low_bw)
		by cov: gen pos_p=pos/_N
		format cnt seq pos %3.0f
		format pos_p %6.3f
	predict res_p, res
	predict res_d, dev
	predict res_stdp, rsta
	predict lev, hat
	predict db, dbeta
	predict dx2, dx2
	predict ddev, ddev
	format pv res_p res_d res_stdp lev db dx2 ddev %6.3f
	* scatterplot by covariate pattern
		twoway (scatter res_stdp cov [aweight=cnt], msymbol(Oh) mlcolor(black) mlwidth(medium)) ///
		(scatter res_stdp cov, msize(vtiny) mlabel(cov)), scheme(s1mono) ///
		ytitle("standardised Pearson residual", m(r+2)) ylabel(-2(2)6, angle(horizontal) format(%4.1f)) ///
		xtitle("covariate pattern", m(t+2)) plotregion(style(none)) legend(off)
	graph save "figs/fig16_p_res.gph", replace
	graph export "figs/fig16_p_res.emf", replace
* list extreme residuals
	sort res_stdp
	list cov pos cnt smk frace_c3 white previs res_stdp if res_stdp<= -2 & seq==1, clean noobs abb(6)
	list cov pos cnt smk frace_c3 white previs pv res_stdp if res_stdp>= 2.5 & seq==1, clean noobs abb(6)
	* refitting the model without |resid|>3
		xi:logit low_bw smk i.frace_c3 white previs_ct previs_sq
			estimates store all
			lfit, g(10)
		xi:logit low_bw smk i.frace_c3 white previs_ct previs_sq if abs(res_p)<3
			estimates store res_3
			lfit, g(10)
		estimates table all res_3, se

* identifying highest leverage points
	sort lev
	list cov cnt smk frace_c3 white previs pv pos_p res_p lev if lev>0.15 & seq==1, clean noobs abb(6)

* Cooks distance (only available after -glm-)
		preserve
		collapse (count) cnt=low_bw (sum) pos=low_bw (mean) smk frace_c3 white previs previs_ct previs_sq, ///
			by(cov)
		xi:glm pos smk white i.frace_c3 previs_ct previs_sq, link(logit) family(binomial cnt)
			predict cooksd, cooksd
			predict pv, mu
			sort cooksd
			list cov cnt smk frace_c3 white previs pv cooksd if cooksd>0.05 , clean noobs abb(6)
		restore

* evaluating delta betas
	sort db
	list cov cnt smk white frace_c3 previs pv pos_p res_p db if db>.5 & seq==1, clean noobs abb(6)
	* cov. pattern 124 has a huge influence on the coefficient(s)
	* refit the model without those observations
		xi:logit low_bw smk i.frace_c3 white previs_ct previs_sq if cov!=124
			estimates store not_cp124
				estimates table all not_cp124, se
			* eliminates almost 10% of the data, reduces effect of -smk- and -white-

* evaluating delta chisq and delta dev
	sort dx2
	list cov cnt smk white frace_c3 previs pv pos_p res_p dx2 ddev if (dx2>10 | ddev>10) & seq==1, clean noobs abb(6)

*************************************************************************
* Section 16.13 Sample size
*************************************************************************

* assume that you want to do a cohort sutdy in which the main risk factor 
* of interest is expected to raise the risk of disease from 2% to 5%
	sampsi .05 .02, power(.8)
		* you need 653 exposed and 653 non-exposed subjects (total = 1306)
* adjustment for clustering
	* assume that the risk factor is a family level factor 
	* and avg. family size = 4 people, and that the estimated ICC is 0.2
		sampclus , obs(4) rho(.2)	/* requires add-on prgm. -sampclus */
			* you need 523 families (2092 people)
			* if you just sample 2 people from each family
			sampclus , obs(2) rho(.2)
				* now you need 784 familes (392 exposed and 392 non-exposed)
				* ... or a total of 1568 people
* adjustment for multiple variables
	* assume that you think there will be 3 other factors that are likely 
	* to be confounders and that the r2 of these confounders when 
	* predicting the main risk factor is .25
		di 784 * ( 1 / (1-0.25) )
			* you now need 1046 families (523 E+ and 523 E-)

*************************************************************************
* Ex 16.13 Exact Logistic Regression
*************************************************************************
use low_bw_dx.dta, clear
	* generate a random num. to use in sampling 
	set seed 12345
	gen rand=runiform()
	* select 30 cases of phyper and 30 controls
	sort phyper rand
		gen flag=1 in -30/l
		replace flag=1 in 1/30
		keep if flag==1
		drop rand flag
		tab low_bw phyper
	* random effect logistic regression
	exlogistic low_bw  phyper white , estc coef 
	


*************************************************************************
* Ex 16.15 - 16.16 Conditiional Logistic Regression
*************************************************************************
	use c:\mer\data\salm_outbrk.dta, clear

* lack of information from groups with no within-group variation
	list match_grp casecontrol eatpork eatbeef slt_a dlr_a if match_grp==23, clean noobs

* Mantel-Haenszel statistics for binary exposures (for comparison purposes)
	cc casecontrol slt_a, by(match_grp) nohom  /* OR=3.87, P=0.002 */

* conditional logistic regression
	logistic casecontrol slt_a		/* incorrect analysis */
	clogit casecontrol slt_a, group(match_grp) or 
		estimates store simple
	clogit casecontrol i.slt_a##gender, or group(match_grp)
		*note gender dropped because no within group variance
		estimates store multi
	lrtest simple multi

* conditional logistic models - diagnostics
	clogit casecontrol slt_a, group(match_grp) or
		capture drop  _hat _dx2 _dbeta _gdx2 _gdbeta
	predict hat, hat
	predict dx2, dx2
	predict dbeta, db
	predict gdx2, gdx2
	predict gdbeta, gdb
		sort gdbeta match_grp casecontrol
		format hat dx2 dbeta gdx2 dbeta %5.3f
		list match_grp casecontrol slt_a hat dx2 dbeta gdx2 gdbeta in -11/l, clean noobs
		twoway scatter gdx2 gdbeta , mlabel(match_grp) 
		* match_grp 2 and 9 have big influence on dbeta, refit model without
			clogit casecontrol slt_a if match_grp~=2 & match_grp~=9, group(match_grp) or

log close mer16_all
