* MER
* mer17_all.do
* Ordinal and multinomial logistic regression - all calculations
*
	version 12
	clear all
	set more off , permanent
	cd c:\mer\ch17\data		/* this line is computer dependent */
	capture log close 
	log using mer17_all, name(mer17_all) text replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Open the data - generate a dichotomous outcome (<2500 gm = low birth weight)
*	and recode several predictors used in previous chapters
*	and restrict data to single birhts
*************************************************************************

use c:\mer\data\bw5k.dta, clear
	gen white=(mrace_c3==2) if !missing(mrace_c3)
		label define white_lbl 0 "non-white" 1 "white" 
		label val white white_lbl
	egen previs_c3=cut(previs), at(0 6 12 99) icodes
		label define previs_c3_lbl 0 "<6 visits" 1 "6-11 visits" 2 "12+ visits"
		label val previs_c3 previs_c3_lbl
	gen previs_c2=previs_c3>=1 if !missing(previs_c3)
		label define previs_c2_lbl 0 "1-6" 1 "7-10"
		label val previs_c2 apgar_c3_lbl
	egen apgar_c3=cut(apgar_c4), at(0 1 3 99) icodes
		label define apgar_c3_lbl 0 "1-6" 1 "7-8" 2 "9-10"
		label val apgar_c3 apgar_c3_lbl
	sum apgar_c4 apgar_c3 previs_c3 previs_c2 white gest male
	save apgar.dta, replace

*************************************************************************
* Table 17.2 Cross-tabulation of -apgar_c4- and -previs_c3-
*************************************************************************
* open the apgar dataset
	use apgar.dta, clear
	tab apgar_c4 previs_c2
	tab apgar_c4 previs_c2, col
	* multinomial
	cci 92 7 58 14
	cci 493 45 58 14
	* proportional odds
	cci 4641 287 58 14 
	cci 4549 280 150 21
	* adjacent category
	cci 92 7 58 14
	cci 493 45 92 7
	cci 4056 235 493 45 
		* estimate beta - define the constraintes
		constraint define 1 [1_6]previs_c2=0.5*[7]previs_c2
		constraint define 2 [7]previs_c2=0.5*[8]previs_c2
		constraint list
			* fit constrained model
			mlogit apgar_c4 previs_c2, base(0) constraint(1/2)
			di exp(.4094)
	* continuation ratio
	cci 92 7 58 14
	cci 493 45 150 21
	
*************************************************************************
* Ex 17.1 Simple multinomial logistic regression
*************************************************************************
* Fit a simple multinomial logistic model based on -previs_c2-
	use apgar.dta, clear
	mlogit apgar_c3 previs_c2, baseoutcome(2)
	mlogit apgar_c3 previs_c2, baseoutcome(2) rrr
	test previs_c2
	* using 4-level outcome variable with level 0 as baseline for comparison with previous section
		mlogit apgar_c4 previs_c2, baseoutcome(0) rrr

*************************************************************************
*Ex 17.2 and 17.3 Multiple multinomial logistic regression
*************************************************************************
* Fit a multiple multinomial logistic model
	label drop apgar_c3_lbl	/* -mlogtest Hausman- does not work with labelled variables */
	mlogit apgar_c3 i.previs_c3 white gest male , baseoutcome(2)
		estimates store mlogit
* Statistical significance of predictors
	* Wald tests
		testparm i.previs_c3
		test white
		test gest
		test male
	* LRT (previs_c2 only)
		mlogit apgar_c3 white gest male if _est_mlogit , baseoutcome(2)
		lrtest . mlogit
	* tests of assumption of IIA
	xi:mlogit apgar_c3 i.previs_c3 white gest male , baseoutcome(2)
		mlogtest, hausman   /* requires xi: to run */
		set seed 45678
		mlogtest, smhsiao	/* run muiltiple times because of stochastic nature of test */
		mlogtest, smhsiao
		mlogtest, smhsiao
		mlogtest, smhsiao
		mlogtest, lrcomb
* Compute predicted probabilites 
	xi:mlogit apgar_c3 i.previs_c3 white gest male , baseoutcome(2)
		predict p1_6 p7_8 p9_10, p
		format p1_6 p7_8 p9_10 %5.3f
		sort obs
		list obs apgar_c3 previs_c3 white gest male p1_6 p7_8 p9_10  ///
			if  obs== 655332 | obs==1358363 | obs==3464037 | obs==2926875 | obs==3586653 ///
			, clean noobs string(9) ab(8)


*************************************************************************
* Ex 17.4 and Fig 17.5 - Proportional odds model
*************************************************************************
* fit a proportional odds model
	xi:ologit apgar_c3 i.previs_c3 white gest male , table
		estimates store ologit
	* compute predicted values for all individuals
		capture drop p1_6 p7_8 p9_10
		predict p1_6 p7_8 p9_10, p
		predict xb, xb
		format xb p1_6 p7_8 p9_10 %5.3f
	* list the relevant variables for the first 5 observations
		sort obs
		list obs apgar_c3 previs_c3 white gest male xb p1_6 p7_8 p9_10  ///
			if  obs== 655332 | obs==1358363 | obs==3464037 | obs==2926875 | obs==3586653 ///
			, clean noobs string(9) ab(8)
* generate smoothed values of predicted probabilities
		capture drop p1_6sm p7_8sm p9_10sm 
	lowess p1_6 gest , bw(.5) gen(p1_6sm)
	lowess p7_8 gest, bw(.5) gen(p7_8sm)
	lowess p9_10 gest, bw(.5) gen(p9_10sm)
		lab var p1_6sm "Apgar 1-6"
		lab var p7_8sm "Apgar 7-8"
		lab var p9_10sm "Apgar 9-10"
	* graph the smoothed predicted values against imfat
		sort gest
		twoway line p1_6sm p7_8sm p9_10sm gest, scheme(s1mono) ///
			lcolor(black black black) lpattern(solid dash dash_dot) legend(off) ///
			ytitle("probability of Apgar score category", m(r+2)) ylabel(0(.2)1, angle(horizontal) format(%4.2f)) ///
			xtitle("gestation length (weeks)", m(t+2)) xlabel(#5,format(%4.0f)) ///
			plotregion(style(none)) text(.3 31 "7-8", place(NE)) ///
			text(.11 25 "1-6", place(NE)) text(.77 37 "9-10", place(NE)) name(fig17_ord_prb, replace)
 		graph save "figs/fig17_ord_prb.gph", replace
		graph export "figs/fig17_ord_prb.wmf", replace

*************************************************************************
* Section 17.5.2 Evaluating the proportional odds assumption
*************************************************************************
* approximate likelihood ratio test using stored estimates
	* comparison with multinomial model
		lrtest ologit mlogit, stats force
	* comparison with generalized ordinal model
		/* gologit is an add-on program */
		xi:gologit2 apgar_c3 i.previs_c3 white gest male 
		estimates store gologit
		lrtest ologit gologit, stats force
* approximate likelihood ratio test using -omodel-
		/* omodel is an add-on program */
	xi:omodel logit apgar_c3 i.previs_c3 white gest male
* Brant test
	xi:ologit apgar_c3 i.previs_c3 white gest male , table
	brant, detail

*************************************************************************
* Section 17.5.3 Dealing with non-proportional odds 
*************************************************************************
* generalized ordinal logistic regression	
	xi:gologit2 apgar_c3 i.previs_c3 white gest male 
* partial proportional odds
	xi:gologit2 apgar_c3 i.previs_c3 white gest male , autofit
	* allowing slopes to vary for -gest- and -male-
	xi:gologit2 apgar_c3 i.previs_c3 white gest male , pl(i.previs_c3 white)
		estimates store partial
		lrtest ologit partial, force stats
		lrtest partial gologit, force stats
* stereotype logistic model (not included in text)
	xi:slogit apgar_c3 i.previs_c3 white gest male
		estimates store slogit
		lrtest ologit slogit, stats force
* heterogenous choice logistic model (not included in text)
		/* oglm is an add-on program */
	xi:oglm apgar_c3 i.previs_c3 white gest male, het(gest male) store(oglm)
		lrtest ologit oglm, stats force
* overall comparison
	estimates stats mlogit ologit gologit partial slogit oglm

*************************************************************************
* Ex 17.5 Adjacent category model (constrained multinomial model)
*************************************************************************
* define the constraints
	xi i.previs_c3, prefix(xx)
	local list xxprevis_c3_1 xxprevis_c3_2 white gest male 
	local i=1
	foreach var of local list {
	constraint define `i' [1]`var'=0.5*[2]`var'
	local i=`i'+1
		}
		constraint list
* fit constrained model
	mlogit apgar_c3 xxprevis_c3_1 xxprevis_c3_2 white gest male , base(0) constraint(1/5)
		estimates store acm
		lrtest mlogit acm, stats
* overall comparison
	estimates stats mlogit ologit gologit partial slogit oglm acm
	* acm model has lowest AIC followed by partial proportional odds and sterotype logistic models 

*************************************************************************
* Section 17.7 Continuation ratio model (for demonstration of coding only)
*************************************************************************
* first create the two new outcome variables
	capture drop apg0 apg1
	gen apg1=(apgar_c3==1)
		replace apg1=. if apgar_c3==2
	gen apg2=(apgar_c3==2)
* fit an unconstrained model
   * fit the logistic models and store the log likelihoods
	logit apg1 xxprevis_c3_1 xxprevis_c3_2 white gest male
		scalar ll0=e(ll)
	logit apg2 xxprevis_c3_1 xxprevis_c3_2 white gest male
		scalar ll1=e(ll)
	display ll0 + ll1
* fit a constrained model
		/* ocratio is an add-on program */
	ocratio apgar_c3 xxprevis_c3_1 xxprevis_c3_2 white gest male
		
