* MER
* mer18_all.do
* Modelling Count Data - all calculations
*
	version 12
	clear all
	set more off , permanent
	cd c:\mer\ch18\data		/* this line is computer dependent */
	capture log close 
	log using mer18_all, name(mer18_all) text replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Open the data - and create several new variables
	* a dichotomous variable for mother's race (-white-)
	* centred versions of -mage-
	* a rescaled version of tbo
*************************************************************************

use c:\mer\data\bw5k.dta, clear
	gen white=(mrace_c3==2) if !missing(mrace_c3)
		label define white_lbl 0 "non-white" 1 "white" 
		label val white white_lbl
	gen mage_28=mage-28
		label var mage_28 "Mother's age (centred)"
	gen tbo_1=tbo-1
		label var tbo_1 "Total birth order - 1"
save previs.dta, replace
	
*************************************************************************
* Fig 18.1 Poisson distributions
	clear
	set obs 100000
	set seed 98765
	gen p_05 = rpoisson(0.5)
	gen p_10 = rpoisson(1)
	gen p_20 = rpoisson(2)
	gen p_50 = rpoisson(5)
	sum

histogram p_05, width(1) start(0) fraction discrete ///
	barwidth(0.1) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(1)4) ///
	mcolor(black) ytitle("probability of count",m(r+2)) plotregion(style(none)) ///
	ti("mean = 0.5", size(medium)) ylabel(#4, angle(horizontal)) 
	graph save "figs/fig18_pd05.gph", replace

histogram p_10, width(1) start(0) fraction discrete ///
	barwidth(0.125) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(1)5) ///
	mcolor(black) ytitle("") plotregion(style(none)) ///
	ti("mean = 1.0", size(medium)) ylabel(0 .1 .2 .3 .4, angle(horizontal))
	graph save "figs/fig18_pd10.gph", replace

histogram p_20, width(1) start(0) fraction discrete ///
	barwidth(0.2) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(1)9) ///
	mcolor(black) ytitle("probability of count",m(r+2)) plotregion(style(none)) ///
	ti("mean = 2.0", size(medium)) ylabel(#4, angle(horizontal))
	graph save "figs/fig18_pd20.gph", replace

histogram p_50, width(1) start(0) fraction discrete ///
	barwidth(0.3) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(2)16) ///
	mcolor(black) ytitle("") plotregion(style(none)) ///
	ti("mean = 5.0", size(medium)) ylabel(0 .05 .1 .15 .2, angle(horizontal))
	graph save "figs/fig18_pd50.gph", replace

* combine the four graphs
graph combine figs/fig18_pd05.gph figs/fig18_pd10.gph figs/fig18_pd20.gph figs/fig18_pd50.gph , ///
	scheme(s1mono) plotregion(style(none)) 
	graph save "figs/fig18_poisson_dist.gph", replace
	graph export "figs/fig18_poisson_dist.emf", replace

*************************************************************************
* Ex 18.1 Fitting a Poisson model - ML estimation
*************************************************************************
	use previs.dta, clear
	* Poisson model for simple count of prenatal visits (no adjustment for length of gestation)
	poisson previs white mage_28 tbo_1 i.meduc_c4
		estimates store count
	* Poisson model for rate prenatal visits (accounting for length of gestation)
	poisson previs white mage_28 tbo_1 i.meduc_c4, exp(gest)
		testparm i.meduc_c4 
		estimates store rate
	estimates table count rate
	* calculations of incidence rates (not presented in text)
		* baseline (non-white, age=28, 1st parity, gest=38, < HS diploma)
			display exp(_b[_cons])
		* baseline but with univ. degree
			display exp(_b[_cons] + _b[4.meduc_c4])
		* higher frequency individual (White, 38 years old)
			display exp(_b[_cons] + _b[white] + 10*_b[mage_28])
		* white  (white, age=28, 3rd parity, gest=38, univ. degree)
			display exp(_b[_cons] + _b[white] + 10*_b[mage_28] + _b[4.meduc_c4])
		* expected # of visits in 39 week gestation
			display 39*exp(_b[_cons] + _b[white] + 10*_b[mage_28] + _b[4.meduc_c4])

	* fitting a Poisson - GLM framework
		glm previs white mage_28 tbo_1 i.meduc_c4, exp(gest) fam(poisson) link(log)
	* expressing results as IRR
		glm previs white mage_28 tbo_1 i.meduc_c4, exp(gest) fam(poi) eform
	* Goodness of fit test
		poisson previs white mage_28 tbo_1 i.meduc_c4, exp(gest)
			estat gof

*************************************************************************
* Fig 18.2 Computing predicted values and comparing to observed
*************************************************************************
	poisson previs white mage_28 tbo_1 i.meduc_c4, exp(gest)
	* using SPOST (-prcounts-) to compute predicted probabilities
	* Note: SPOST is a user written add-on to Stata
	* -prcounts- doesn't work after -glm- so I used the -poisson- command
		capture drop pc*
		prcounts pc, plot max(30) 
		twoway line pcpreq pcobeq pcval if pcval<=20, ///
			scheme(s1mono)lcolor(black black) ///
			lpattern(solid dash) title("") legend(off) ///
			ytitle("proportion of observations", m(r+2)) ///
			xtitle("count", m(t+2)) plotregion(style(none)) ///
			text(.07 5 "predicted", place(NE)) text(.05 8  "observed", place(NE))
		graph save "figs/fig18_pred_prob.gph", replace
		graph export "figs/fig18_pred_prob.emf", replace
		capture drop pc*

*************************************************************************
* Section 18.5.3 Dispersion parameters
*************************************************************************
	glm previs white mage_28 tbo_1 i.meduc_c4, exp(gest) fam(poi)
 
*************************************************************************
* Ex 18.2 Poisson regression diagnostics
*************************************************************************
	* refit the model using -glm- in order to obtain scaled SE, and
	* to get a wider range of diagnostic prameters
	* scaled SE
		glm previs white mage_28 tbo_1 i.meduc_c4, exp(gest) fam(poi) scale(x2)
	* compute predicted values, residuals and Cook's distance
		glm previs white mage_28 tbo_1 i.meduc_c4, exp(gest) fam(poi)
		capture drop res_d res_p res_a cooks_d mu
		predict res_d, deviance standardized
		predict res_p, pearson  standardized
		predict res_a, anscombe standardized
		predict cooks_d, cooksd
		predict mu, mu
		sort res_a
		* determine # of extreme residuals
			count if res_p>3 | res_p<-3	/* n=124 (2.5%) */
		* br obs previs white mage_28 tbo_1 meduc_c4 gest mu res_d res_p res_a cooks_d
	* list the smallest and largest residuals and the largest Cook's Distances
		list  obs white mage_28 tbo_1 meduc_c4 gest previs mu in 1/3, clean noobs
		list  obs res_d res_p res_a cooks_d in 1/3, clean noobs
		list  obs white mage_28 tbo_1 meduc_c4 gest previs mu in -3/-1, clean noobs
		list  obs res_d res_p res_a cooks_d in -3/-1, clean noobs
		sort cooks_d
		list  obs white mage_28 tbo_1 meduc_c4 gest previs mu in -4/-1, clean noobs
		list  obs res_d res_p res_a cooks_d in -4/-1, clean noobs
	* diagnostic plots
		twoway scatter res_a mu, saving(r1.gph, replace) ///
			scheme(s1mono) xtitle("Predicted count") msize(small) mcolor(black) ///
			text(6.85 11.1 "726794", place(NE) size(small)) ///
			text(-5.3 12.5 "3397522", place(NE) size(small)) ///
			ytitle("Anscombe residuals", m(t+2)) plotregion(style(none)) 	
		twoway scatter res_a cooks_d, saving(r2.gph, replace) ///
			scheme(s1mono) xtitle("Cook's distance") msize(small) mcolor(black) ///
			text(6.8 .0542 "726794", place(SW) size(small)) ///
			text(4.5 0.009 "1394806", place(NE) size(small)) ///
			text(-5 0.008 "1733432", place(NE) size(small)) ///
			ytitle("Anscombe residuals", m(t+2)) plotregion(style(none)) 	
		* combine the 2 residuals graphs
			graph combine r1.gph r2.gph , xsize(6) ysize(3) scheme(s1mono) ///
			subtitle("Diagnostic plots - Poisson regression")
		graph save "figs/fig18_pois_diag_plots.gph", replace
		graph export "figs/fig18_pois_diag_plots.emf", replace
	* refit the model without 726794
		glm previs white mage_28 tbo_1 i.meduc_c4, exp(gest) fam(poi)
			estimates store all
		glm previs white mage_28 tbo_1 i.meduc_c4  if obs!=726794, exp(gest) fam(poi)
			estimates store all_1
		estimates table all all_1
			* approx. 4.6% reduction in coef. for 3.meduc_c4 (some college)
			* and 3.1% reduction in coef. for 4.meduc_c4 (college diploma)
	
*************************************************************************
* Fig 18.4 Negative binomial distributions
*************************************************************************
*generate variables with negative binomial distributions
	clear
	set obs 100000
	set seed 98765
	gen nb_2_1 = rnbinomial(1,1/3)
	gen nb_2_2 = rnbinomial(0.5,1/5)
	gen nb_5_1 = rnbinomial(1,1/6)
	gen nb_5_2 = rnbinomial(0.5,1/11)
	sum

histogram nb_2_1 if nb_2_1<=12, width(1) start(0) fraction discrete ///
	barwidth(0.1) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(1)12) ///
	mcolor(black) ytitle("proportion of count",m(r+2)) plotregion(style(none)) ///
	ti("mean = 2.0, alpha = 1", size(medium)) ylabel(0(.1).5, angle(horizontal))
	graph save "figs/fig18_nb_2_1.gph", replace

histogram nb_2_2 if nb_2_1<=12, width(1) start(0) fraction discrete ///
	barwidth(0.2) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(2)24) ///
	mcolor(black) ytitle("") plotregion(style(none)) ///
	ti("mean = 2.0, alpha = 2", size(medium))ylabel(0(.1).5, angle(horizontal))
	graph save "figs/fig18_nb_2_2.gph", replace

histogram nb_5_1 if nb_5_1<=36, width(1) start(0) fraction discrete ///
	barwidth(0.3) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(3)36) ///
	mcolor(black) ytitle("proportion of observations",m(r+2)) plotregion(style(none)) ///
	ti("mean = 5.0, alpha = 1", size(medium)) ylabel(0(.1).3, angle(horizontal))
	graph save "figs/fig18_nb_5_1.gph", replace

histogram nb_5_2 if nb_5_2<=56, width(1) start(0) fraction discrete ///
	barwidth(0.4) scheme(s1mono) xtitle("count", m(t+2)) xlabel(0(4)56) ///
	mcolor(black) ytitle("") plotregion(style(none)) ///
	ti("mean = 5.0, alpha = 2", size(medium)) ylabel(0(.1).3, angle(horizontal))
	graph save "figs/fig18_nb_5_2.gph", replace

* combine the four graphs
graph combine figs/fig18_nb_2_1.gph figs/fig18_nb_2_2.gph figs/fig18_nb_5_1.gph figs/fig18_nb_5_2.gph , ///
	scheme(s1mono) plotregion(style(none)) 
	graph save "figs/fig18_nb_dist.gph", replace
	graph export "figs/fig18_nb_dist.emf", replace

*************************************************************************
* Negative binomial regression - all calculations
*************************************************************************
	use previs.dta, clear

*************************************************************************
* Evaluating the relationship between mean and var. of reactors (not in text)
*************************************************************************
	sum previs, detail
	table meduc_c4 white, contents(mean previs sd previs)

*************************************************************************
* Ex 18.3 - 18.4 Comparing Negative binomial models fit using ML and GLM
*************************************************************************
	nbreg previs white mage_28 tbo_1 i.meduc_c4, exp(gest)
		estimates store nb_ml
	* comparing Poisson and NB coefficients
		estimates table nb_ml rate
	* comparing ML vs GLM
		glm previs white mage_28 tbo_1 i.meduc_c4, fam(nb .0188208) link(log) exp(gest)
		/* note: -glm- does not estimate "alpha" by default so to get equivalent results
		it needs to be estimated using -nbreg- and then added to the -family- option.
		Alternatively, ml estimation may be specified and Stata estimates alpha before proceeding
		glm previs white mage_28 tbo_1 i.meduc_c4, fam(nb ml) link(log) exp(gest)*/
		estimates store nb_glm
		estimates table nb_ml nb_glm, b(%5.3f) se(%5.3f)

*************************************************************************
* Ex 18.5 Model diagnostics
*************************************************************************
	glm previs white mage_28 tbo_1 i.meduc_c4, fam(nb ml) link(log) exp(gest)
	* deviance based goodness of fit 
		display chi2tail(e(df),e(deviance))
	* Pearson goodness of fit test 
		display chi2tail(e(df),e(deviance_ps))	
	
		capture drop res_d res_p res_a cooks_d mu
		predict res_d, deviance standardized
		predict res_p, pearson  standardized
		predict res_a, anscombe
		predict cooks_d, cooksd
		predict mu, mu
		sort cooks_d
		* determine # of extreme residuals
			count if res_p>3 | res_p<-3		/* n=74 (1.5%) */
	* br obs previs white mage_28 tbo_1 meduc_c4 mu res_d res_p res_a cooks_d
	list  obs white mage_28 tbo_1 meduc_c4 previs mu in -4/-1, clean noobs
	list  obs res_d res_p res_a cooks_d in -4/-1, clean noobs
	* diagnostic plots
		twoway scatter res_a mu, saving(r1.gph, replace) msize(small) mcolor(black) ///
			scheme(s1mono) xtitle("Predicted count", size(large)) ///
			text(6.75 11.2 "726794", place(NE) size(small)) ///
			text(-5.3 12.6 "3397522", place(NE) size(small)) ///
			ytitle("Anscombe residuals", m(t+2) size(large)) plotregion(style(none)) 	
		twoway scatter res_a cooks_d, saving(r2.gph, replace) msize(small) mcolor(black) ///
			scheme(s1mono) xtitle("Cook's distance",  size(large)) ///
			text(6.8 .045 "726794", place(SW) size(small)) ///
			text(4.4 0.008 "1394806", place(NE) size(small)) ///
			text(-4.9 0.007 "1733432", place(NE) size(small)) ///
			ytitle("Anscombe residuals", m(t+2) size(large)) plotregion(style(none)) 	
		* combine the 2 residuals graphs
			graph combine r1.gph r2.gph , scheme(s1mono) xsize(6) ysize(3) ///
			subtitle("Diagnostic plots - Negative binomial regression")
		graph save "figs/fig18_nb_diag_plots.gph", replace
		graph export "figs/fig18_nb_diag_plots.emf", replace
	* refit the model without 726794
		nbreg previs white mage_28 tbo_1 i.meduc_c4, exp(gest) 
			estimates store all
		nbreg previs white mage_28 tbo_1 i.meduc_c4 if obs!=726794, exp(gest) 
			estimates store all_1
		estimates table all all_1
			* approx. 4.5% reduction in coef. for 3.meduc_c4 (some college)
			* and 3.1% reduction in coef. for 4.meduc_c4 (college diploma)

*************************************************************************
* Ex 18.6 Generalized negative binomial regression
*************************************************************************
	nbreg previs white mage_28 tbo_1 i.meduc_c4, exp(gest)
		estimates store nb2
	gnbreg previs white mage_28 tbo_1 i.meduc_c4, exp(gest) lnalpha(i.meduc_c4)
		estimates store nbh
	estimates stats nb2 nbh
	di chi2tail(3, (2*(-13734.8 + 13772.9)))
	/* alpha gets smaller as level of educ. rises */
	tab meduc_c4, sum(previs)
	
*************************************************************************
* Ex 18.7 Zero inflated model
*************************************************************************
	zinb previs white mage_28 tbo_1 i.meduc_c4, exp(gest) ///
		inflate(i.meduc_c4) nolog vuong
	estimates store zinfl
	testparm i.meduc_c4, eq(inflate)
	
*************************************************************************
* Ex 18.8 Hurdle (negative binomial, logit) model
*************************************************************************
	/* requires add-on pgm. -hnblogit- */
	xi:hnblogit previs white mage_28 tbo_1 i.meduc_c4, exp(gest) 
		estimates store hrdl
	* comparison of zero-inflated and hurdle models
	zinb previs white mage_28 tbo_1 i.meduc_c4, exp(gest) ///
		inflate(white mage_28 tbo_1 i.meduc_c4) nolog 
		estimates store zinfl
	estimates stats zinfl hrdl
	estimates table zinfl hrdl

*************************************************************************
* Ex 18.9 Zero truncated model
*************************************************************************
	tnbreg tbo white i.meduc_c4 mar 
		estimates store ztnb
	* for comprison, fit an ordinary neg. bin. model
	nbreg tbo_1 white i.meduc_c4 mar 
		estimates store nb
		estimates stats ztnb nb
		estimates table ztnb nb
		
log close mer18_all


