* ch19_all.do
* Modelling Survival Data - all exercises and figures
*
	version 12
	clear all
	set more off , permanent
	cd c:\mer\ch19\data		/* this line is computer dependent */
	capture log close 
	log using mer19_all, name(mer19_all) text replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Open and declare the data to be survival data (stset)
*************************************************************************
use mi.dta, clear
	stset surv_mi, fail(died)

*************************************************************************
* Figure 19.1 - Histogram - survival from onset
*************************************************************************
* generate a histogram
	histogram surv_mi if died==1, freq start(0) width(30) scheme(s1mono) ///
		ytitle("frequency", m(r+2)) ylabel(,angle(horizontal))   ///
		xtitle("survival time (days)", m(t+2))
	graph save "figs/fig19_histo_mi.gph", replace
	graph export "figs/fig19_histo_mi.emf", replace

*************************************************************************
* Example 19.2 Actuarial and Kaplan-Meier estimates of survivor functions
* and Fig 19.5 and 19.6 graph of survivor cumulative hazard functions
* Previous bypass surgery cases only
*************************************************************************
* open the MI dataset and keep only cases that had previous bypass surgery (n=391)
	use mi.dta, clear
	keep if prcabg==1
* generate an actuarial life table (survival and hazard)
	ltable surv_mi died, interval(365) 
	ltable surv_mi died, interval(365) hazard
* generate a Kaplan-Meier estimate of the survival function
	stset surv_mi, f(died)
	* sts list    /*list is too long to be any use */
* graph the cumulative survivor function
	sts graph, gwood plotopts(lwidth(medium)) ///
		ciopts(lpattern(dash) lcolor(black) lwidth(medium)) ///
		legend(off) scheme(s1mono) title("") plotregion(style(none)) ///
		ytitle("cumulative survival probability", m(r+2)) ///
		xtitle("time (days)", m(r+2)) ylabel(,angle(horizontal))  
	graph save "figs/fig19_srv_km", replace
	graph export "figs/fig19_srv_km.emf", replace
* graph the Nelson - Aalen cumulative hazard function
	sts graph, cumhaz ytitle("cumulative hazard", m(r+2)) ylabel(,angle(horizontal)) ///
		xtitle("time (days)", m(t+2)) plotregion(style(none)) ti("") ///
		plotopts(lwidth(medium)) scheme(s1mono) legend(off) ///
		ci ciopts(lpattern(dash) lcolor(black) lwidth(medium))	
	graph save "figs/fig19_srv_ch.gph", replace
	graph export "figs/fig19_srv_ch.emf", replace


*************************************************************************
* Example 19.3 Log-rank test
*************************************************************************
* log-rank test
	sts test mar_c2
* M-H analysis
	* not feasible to compute by hand given the large number of failure times


*************************************************************************
* Example 19.4 and Fig 19.7 Non-parametric comparison of survivor functions
*************************************************************************
* generate survivor functions for the two stocking groups
	sts graph, by(mar_c2) ylabel(,angle(horizontal)) ///
		legend(off) scheme(s1mono) ytitle("cumulative survival probability") ///
		title("") plotregion(style(none)) ///
		text(.55 950 "married", place(ne)) text(.4 450 "not-married", place(ne)) ///
		plot2opts(lcolor(black) lwidth(medium)) ylabel(,glwidth(medthick) glcolor(black) glpattern(dot)) 
	graph save "figs/fig19_nonpar_tests", replace
	graph export "figs/fig19_nonpar_tests.emf", replace
* run non-parametric tests of equivalence
	sts test mar_c2
	sts test mar_c2, wilcoxon
	sts test mar_c2, cox
	sts test mar_c2, tware
	sts test mar_c2, peto
	

*************************************************************************
* Figure 19.8 Failure, Survivor and density functions
*************************************************************************
	clear
* create time scale, set hazard to 0.01 and compute S(t)
	set obs 501
	gen t=_n-1
	scalar h=0.01
	gen S=exp(-h*t)
* create intervals for step function, and S(t) - step
	gen temp=mod(t,50)
	gen temp2=_n+25
		gen S_step=S
		replace S_step=. if temp!=0
		replace S_step=S_step[temp2] if _n<=475
		replace S_step=1 in 1
* create continuous failure function F(t)
	gen F=1-S
* create step F(t)
		gen F_step=F
		replace F_step=. if temp~=0
		replace F_step=F_step[temp2] if _n<=475
		replace F_step=0 in 1
* compute derivative of F(t)
	dydx F t, gen(f)
	gen slope=F[150] + f[150]*(t-150)
		replace slope=. if t<50 | t>250
* produce the graph
	twoway (scatter S_step t, connect(stairstep) lpattern(dash) msymbol(none) ) ///
		(line F t, lpattern(dash) lcolor(black)) ///
		(scatter F_step t, connect(stairstep) lpattern(solid) msymbol(none) ) ///
		(line slope t, lcolor(black)) ///
		, scheme(s1mono) legend(off) plotregion(style(none))  ///
		ytitle("cumulative probability of survival / failure", m(r+2)) ///
		xtitle("analysis time", m(t+2)) ylabel(,angle(horizontal)) 
	graph save "figs/fig19_fct", replace
	graph export "figs/fig19_fct.emf", replace


*************************************************************************
* Figure 19.9 - Exponential Survival Function
*************************************************************************
	clear
* compute S(t)
	set obs 501
	gen t=_n-1
	gen S=exp(-0.01*t)
* produce graph
	twoway (line S t, lcolor(black)), scheme(s1mono) legend(off) ///
		ytitle("cumulative probability of survival", m(r+2)) ///
		xtitle("analysis time", m(t+2)) plotregion(style(none)) ylabel(,angle(horizontal))  
	graph save "figs/fig19_srv_exp", replace
	graph export "figs/fig19_srv_exp.emf", replace


*************************************************************************
* Figure 19.10 - Weibull Hazard Function
*************************************************************************
* create a dataset of 101 observations, time scale of 0 to 10
	clear
	range t 0 10 101
* set various hazards (p = 0.5, 1, 1.5, 2 and 3) (lambda=1)
	gen h_5=0.5*(t^(0.5-1))
	gen h1=1*(t^(1-1))
	gen h1_5=1.5*(t^(1.5-1))
	gen h2=2*(t^(2-1))
	gen h3=3*(t^(3-1))
	* limit values of large hazards
		replace h2=. if h2>5
		replace h3=. if h3>5
	* label the hazards
	label var h_5 "p=0.5"
	label var h1 "p=1"
	label var h1_5 "p=1.5"
	label var h2 "p=2"
	label var h3 "p=3"
* produce the graph
	graph twoway (line h_5 t, lcolor(black) lpattern(solid)) ///
		(line h1 t, lcolor(black) lpattern(dash) lwidth(medium)) ///
		(line h1_5 t, lcolor(black) lpattern(shortdash) lwidth(medium)) ///
		(line h2 t, lcolor(black) lpattern(solid) lwidth(medium)) ///
		(line h3 t, lcolor(black) lpattern(dash) lwidth(medium)) ///
		, scheme(s1mono) legend(off) plotregion(style(none))  ///
		ytitle("hazard", m(r+2)) xtitle("analysis time", m(t+2)) ///
		text(.3 8 "p=0.5", place(NE)) text(1.1 8 "p=1", place(NE)) ///
		text(3.4 6 "p=1.5", place(NE)) text(4 2.2 "p=2", place(NE)) ///
		text(4 0.5 "p=3", place(NE)) ylabel(,angle(horizontal)) name(width)
	graph save "figs/fig19_weib_haz", replace
	graph export "figs/fig19_weib_haz.emf", replace


*************************************************************************
* Figure 19.11 - Generate graphs showing proportional hazards
*************************************************************************
* open hypothetical dataset
	use fig19_cox_hr.dta, clear
* generate graph of Betas
	twoway line beta_0 beta_1 time, scheme(s1mono) lcolor(black black) ///
		lpattern(solid dash) lwidth(medium medium) legend(off) ///
		title("Log Hazard (Beta=0.69)") ///
		ytitle("log hazard", m(r+2) size(medium)) ///
		xtitle("time", m(t+2) size(medium)) ylabel(,angle(horizontal))   ///
		 plotregion(style(none)) saving(fig19_cox_hr1, replace)
* generate graph of Hazards
	twoway line haz_0 haz_1 time, scheme(s1mono) lcolor(black black) ///
		lpattern(solid dash) lwidth(medium medium) legend(off) ///
		title("Hazard (HR=2)") ///
		ytitle("hazard", m(r+2) size(medium)) ///
		xtitle("time", m(t+2) size(medium)) ylabel(,angle(horizontal)) ///
		 plotregion(style(none))  saving(fig19_cox_hr2, replace)
* combine the two graphs
	graph combine fig19_cox_hr2.gph fig19_cox_hr1.gph , ysize(4) xsize(8) scheme(s1mono)
	graph export "figs/fig19_cox_hr.emf", replace


*************************************************************************
* Example 19.5 Cox Proportional Hazards Model
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died)
* fit a Cox proportional Hazards model
	stcox  sex age mar_c2 card ptca, nohr
		estimates store breslow
* refit the model to obtain hazard ratios
	stcox
* evaluating methods of handling ties
	stcox  sex age mar_c2 card ptca, nohr efron
		estimates store efron
	stcox  sex age mar_c2 card ptca, nohr exactm
		estimates store exactm
	stcox  sex age mar_c2 card ptca, nohr exactp	/* quite slow */
		estimates store exactp
	estimates table breslow efron exactm exactp
		* very little difference ... because relatively few ties once past 20 days

*************************************************************************
* Figure 19.12 - K-M Survivor curves - MI data
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died)
* obtain K-M survivor curves
	sts graph, by(sex) scheme(s1mono) legend(off) ///
		title("") ytitle("probability of surviving", m(r+2)) ///
		xtitle("survival time (days after onset)", m(t+2)) ///
		 plotregion(style(none)) ylabel(,angle(horizontal))   ///
		 plot2opts(lcolor(black)) ylabel(,glwidth(medthick) glcolor(black) glpattern(dot)) ///
		text(.6 1600 "sex=0", place(NE)) text(.45 1000 "sex=1", place(NE)) 
	graph save "figs/fig19_km_prgl.gph", replace
	graph export "figs/fig19_km_prgl.emf", replace


*************************************************************************
* Figure 19.13 - Baseline Hazard 
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died)
	* rescale age so baseline person is 60 yrs old
	gen age_r60=age-60
* fit Cox model
	stcox  sex age_r60 mar_c2 card ptca, nohr basehc(hc) efron
		estimates store cox
	stcurve, hazard scheme(s1mono) lcolor(black) ylabel(,angle(horizontal)) ///
		lpattern(solid) title("") plotregion(style(none)) range(0 2500) width(20)  ///
		ytitle("smoothed estimate of baseline hazard", m(r+2)) ///
		xtitle("time (days after onset)", m(t+2)) ///
		noboundary 
	graph save "figs/fig19__bh_prgl.gph", replace
	graph export "figs/fig19__bh_prgl.emf", replace


*************************************************************************
* Example 19.7 Stratified Cox Proportional Hazrds Model
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died)
* fit Cox model - stratified by sex
	stcox  i.sex##mar_c2 age card ptca, nohr strata(sex) efron
* fit the same model with sex as a fixed effect (not stratified)
	stcox  i.sex##mar_c2 age card ptca, nohr efron

*************************************************************************
* Example 19.8 Time varying predictors in a Cox Proportional Hazrds Model
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
	list id los surv_mi _d _t _t0 in 1/3
* list the data after modification for a time varying predictor
	stsplit disch, after(time=los) at(0)
	replace disch=disch+1
	sort id disch
	list id disch _t0 _t _d in 1/6, clean
	stcox disch, efron
gen d2=1-disch
stcox d2, efron
*************************************************************************
* Example 19.9 Aalen's linear hazards model (Fig 19.14)
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* Aalen's linear hazards model
	stlh ptca , gen(cumbeta)	/* Stata add-on program */
	line cumbetaB1 surv_mi if surv_mi<2000, plotregion(style(none))  ///
		 scheme(s1mono) lcolor(black black) lpattern(solid ) ///
		ytitle("Aalen's cumulative coef.", m(r-10) size(med)) ///
		xtitle("time (days after onset)", m(t+2) size(med)) ///
		ylabel(,angle(horizontal)) 		
	graph save "figs/fig19_aalen_lin_haz.gph", replace
	graph export "figs/fig19_aalen_lin_haz.emf", replace


*************************************************************************
* Figure 19.15 Cumulative hazard plot
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age mar_c2 card ptca, nohr efron
	stphplot , by(card) scheme(s1mono) title("") legend(off) ///
		plot1opts(lcolor(black) lpattern(solid) msymbol(o) mcolor(black)) ///
		plot2opts(lcolor(black) lpattern(dash) msymbol(+) mcolor(black)) ///
		ytitle("-ln cumulative hazard", m(r+2)) ylabel(,angle(horizontal)) ///
		xtitle("ln(time)", m(t+2)) plotregion(style(none)) ///
		text(2.8 3 "card=0", place(NE)) text(1.2 1.2 "card=1", place(NE))
	graph save "figs/fig19_phplot_prgl.gph", replace
	graph export "figs/fig19_phplot_prgl.emf", replace


*************************************************************************
* Figure 19.16 - Cox vs KM plot 
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age mar_c2 card ptca, nohr efron
	stcoxkm if _t<2500, by(card)  ///
		title("") legend(off) ytitle("survival probability", m(r+2)) ///
		xtitle("time", m(t+2)) plotregion(style(none)) scheme(s1mono) ///
		text(.75 300 "KM: cardiac arrest = no", place(NE)) text(.12 4  "KM: cardiac" "arrest = yes", place(NE)) ///
		text(.59 200 "Cox:" "cardiac" "arrest = no", place(NE) just(left)) ///
		text(.4 150 "Cox: cardiac arrest = yes", place(NE)) ///
		ylabel(,angle(horizontal)) ///
			obs1opts(lcolor(gs8) lpattern(solid) msymbol(i) mcolor(gs8) msize(tiny)) ///
    		obs2opts(lcolor(gs8) lpattern(solid) msymbol(i) mcolor(gs8) msize(tiny))  ///
    		pred1opts(lcolor(black) lpattern(dash) msymbol(x) mcolor(black) msize(tiny)) ///
    		pred2opts(lcolor(black) lpattern(dash) msymbol(i) mcolor(black) msize(tiny))	
	graph save "figs/fig19_kmplot_prgl.gph", replace
	graph export "figs/fig19_kmplot_prgl.emf", replace


*************************************************************************
* Example 19.10 Time varying efects
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a model with -card- as a TVE
	stcox card, tvc(card) texp(_t) nohr efron 
	stcox card, tvc(card) texp(log(_t)) nohr efron 
	di " 0 " exp(3.437 -.57249*0) " 2 " exp(3.437 -.57249*2) " 4 " exp(3.437 -.57249*4) ///
		 " 6 " exp(3.437 -.57249*6) " 8 " exp(3.437 -.57249*8) " 10 " exp(3.437 -.57249*10)
	 di " 0 " exp(0) " 2 " exp(2) " 4 " exp(4) " 6 " exp(6) " 8 " exp(8) " 10 " exp(10)
	 * fit a model with interaction terms for both ln(_t) and ln(_t)^2
	 preserve
		stsplit, at(failures)
		gen lnt=ln(_t)
		gen lntsq=ln(_t)^2
		stcox i.card##c.lnt i.card##c.lntsq, nohr efron 
	 restore
* fit a model with age as a TVE
	* rescale age (baseline 50 and units=decades)
	gen age_r10=(age-50)/10
	stcox age_r10, tvc(age_r10)  nohr efron	/* borderline sig. P=0.03 */

*************************************************************************
* Example  19.11 and Figure 19.17 - Schoenfeld residuals
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age mar_c2 card ptca, nohr efron
* generate a smoothed graph of scaled Scheonfeld residuals vs log(time)
	estat phtest, log plot(age) yline(0, lcolor(black) lpattern(dash)) msize(tiny)  ///
		ytitle("Scaled Schoenfeld - age", m(r+2)) ylabel(,angle(horizontal)) ///
		xtitle("time (log scale)", m(t+2)) scheme(s1mono) xlab(1 10 100 1000 3000) ///
		mcolor(black) title("") plotregion(style(none)) 
	graph save "figs/fig19_ph_schoen.gph", replace
	graph export "figs/fig19_ph_schoen.emf", replace
* compute test statistics for all predictors
	estat phtest, detail 
	estat phtest, detail log

*************************************************************************
* Note - although there is clear evidence of non-proportional hazards for
* some predictors, from here on, models without TVE will be used for simplicity
*************************************************************************

*************************************************************************
* Example  19.12 - Independence of Censoring
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age mar_c2 card ptca, nohr efron
		estimates store orig
* recode censoring to assume complete positive correlation
	preserve
		replace died=1 if died==0
		stset surv_mi, f(died) id(id)
		stcox  sex age mar_c2 card ptca, nohr efron
			estimates store pos_cens
	restore
* recode censoring and dar to assume complete negative correlation
	preserve
		replace surv_mi=3000 if died==0
		replace died=1 if died==0
		stset surv_mi, f(died) id(id)
		stcox  sex age mar_c2 card ptca, nohr efron
			estimates store neg_cens
	restore
	estimates table orig pos_cens neg_cens

*************************************************************************
* Example 19.13 and Figure 19.18 Evaluating overall fit of a model / computing r2
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox sex age mar_c2 card ptca,  nohr efron
* compute modified Cox-Snell residuals
	predict cs, csnell
	* replace cs=cs+0.693 if preg==0
		label var cs "Modified Cox-Snell residuals"
   * re "stset" the data and fit the new Cox model
		stset cs, f(died)
		sts gen ch=na
		sort cs
		line ch cs cs if cs<3.5,  xlab(0 1 2 3 4) ylab(0 1 2 3 4) scheme(s1mono) ///
			ytitle("Cumulative hazard",m(r+2)) legend(off) ylabel(,angle(horizontal)) ///
			plotregion(style(none)) 
		graph save "figs/fig19_cs_fit.gph", replace
		graph export "figs/fig19_cs_fit.emf", replace
* Goodness-of-fit tests 
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age mar_c2 card ptca,  nohr mgale(mg) efron 
		/* stcoxgof is a Stata add-on */
		stcoxgof			/* Gronnesby and Borgan omnibus test */
		stcoxgof, mol(5)		/* Moreau, O'Quigley and Lellouch ominbus test */
		* stcoxgof, mom(5)		/* Moreau, O'Quigley and Mesbah test for proportional hazards */ 
			* mom test crashes due to collinearity among predictors
* Concordance
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age mar_c2 card ptca,  nohr efron
		estat concordance
* computing r2
	* open the heart attack dataset
		/* the following command is set to 100 bootstrap iterations
		so only approximate CI are computed. Running with 5000 takes considerable time */
		str2ph stcox  sex age mar_c2 card ptca,  nohr bo(100) efron	/* Stata add-on */
		str2d stcox  sex age mar_c2 card ptca,  nohr bo(100) efron	/* Stata add-on */
	* comparing with r2 from linear model of log(surv_mi)
		* not shown in book
		gen surv_mi_ln=ln(surv_mi)
		reg surv_mi_ln sex age mar_c2 card ptca if died==1


*************************************************************************
* Example 19.14 and Figure 19.19 Evaluating functional form of a predictor
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox sex age mar_c2 card ptca, nohr mgale(mg) efron
		estimates store age
	capture drop age_sm
	lowess mg age, gen(age_sm) bwidth(.7)
	sort age
	twoway line age_sm age if age<90, scheme(s1mono) xtitle("age", m(t+2)) ///
		lcolor(black) plotregion(style(none)) ylabel(,angle(horizontal)) ///
		ytitle("Lowess smoothed curve of martingale residuals",m(r+2)) legend(off) title("")
	graph save "figs/fig19_mgale.gph", replace
	graph export "figs/fig19_mgale.emf", replace
* use of fractional polynomials to evaluate functional form
	fracpoly, degree(2) compare: stcox age sex mar_c2 card ptca, nohr efron
	* log(age) is appropriate
	gen age_ln=ln(age)-4
	stcox sex age_ln mar_c2 card ptca, nohr efron
		estimates store age_ln
		estimates stats age age_ln

*************************************************************************
* Note - age will be included as ln(age) in all models from here on
*************************************************************************


*************************************************************************
* Figure 19.20 - Detecting outliers (deviance residuals)
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age_ln mar_c2 card ptca, nohr efron
		estimates store cox
		predict dev, dev
		list id sex age mar_c2 card ptca dev _t _d if abs(dev)>3.5 & !missing(dev)
	twoway (scatter dev _t, mcolor(black) msize(small)) ///
		(scatter dev _t if id==1274 | id==1151, mlab(id) msym(none) mlabpos(11)) ///
		(scatter dev _t if id==2301, mlab(id) msym(none) mlabpos(7)) ///
		(scatter dev _t if id==2923, mlab(id) msym(none) mlabpos(2)) ///
		, scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		ytitle("deviance residual",m(r+2)) plotregion(style(none))  ///
		ylabel(,angle(horizontal)) legend(off) name(overlay2, replace)
	graph save "figs/fig19_dev_res.gph", replace
	graph export "figs/fig19_dev_res.emf", replace

*************************************************************************
* Figure 19.21 - Detecting influential points - Score residuals
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age_ln mar_c2 card ptca, nohr mgale(mg) efron
		predict esr*, esr
		gsort - esr5
		br id surv_mi died esr5 sex age_ln mar_c2 card ptca 
		list id surv_mi died esr5 _t sex age_ln mar_c2 card ptca if abs(esr5)>2.5 &esr5!=., clean noobs
		* plot score residuals
	twoway (scatter esr5 _t, mcolor(black) msize(small)) ///
		(scatter esr5 _t if abs(esr5)>2.5, mlab(id) msym(none)) ///
		, scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		ytitle("score residual",m(r+2)) plotregion(style(none))  ///
		ylabel(,angle(horizontal)) legend(off) 
			graph save "figs/fig19_score_res.gph", replace
			graph export "figs/fig19_score_res.emf", replace

*************************************************************************
* Figure 19.22 Detecting influential points - Scaled score residuals
*************************************************************************
* open the MI dataset
	set matsize 5000
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit a Cox proportional hazards model
	stcox  sex age_ln mar_c2 card ptca, nohr mgale(mg) efron
		predict esr*, esr
		estimates store all
	* compute scaled score residuals
		mkmat esr1 esr2 esr3 esr4 esr5, matrix(esr)
		mat V=e(V)
		mat ssr=esr*V
		svmat ssr, names(ssr)
		label var ssr5 "ssr - ptca"
		gsort - ssr5
		list id surv_mi died ssr5 _t sex age_ln mar_c2 card ptca if abs(ssr5)>.011 & ssr5!=., clean noobs
	twoway (scatter ssr5 _t, mcolor(black) msize(small)) ///
		(scatter ssr5 _t if abs(ssr5)>0.011, mlab(id) msym(none)) ///
		, scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		ytitle("scaled score residual",m(r+2)) plotregion(style(none))  ///
		ylabel(,angle(horizontal)) legend(off) name(sclscore_ovl1, replace) 
			graph save "figs/fig19_scal_score_res.gph", replace
			graph export "figs/fig19_scal_score_res.emf", replace
	* refit model without 4 extreme residuals
		stcox  sex age_ln mar_c2 card ptca if abs(ssr5)<=.011, nohr efron 
		estimates store all_m4
		estimates table all all_m4
		* considerable difference in coefficient for -card-


*************************************************************************
* Example 19.15 - Exponential regression
*************************************************************************
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit an exponential model 
	streg sex age_ln mar_c2 card ptca, nohr dist(exp)
		estimates store expon


*************************************************************************
* Example 19.16 and Figure 19.23 - Exponential regression - piecewise constant baseline hazard
*************************************************************************
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* derive a step function estimate of the baseline hazard
	sum _t
	stsplit day, at(0 15 30 60 120(360)1200 1920 3000)
		gen day0_15= 0<=day & day<15
		gen day15_30= 15<=day & day<30
		gen day30_60= 30<=day & day<60
		gen day60_120= 60<=day & day<120
		gen day120_480= 120<=day & day<480
		gen day480_840= 480<=day & day<840
		gen day840_1200= 840<=day & day<1200
		gen day1200_1920= 1200<=day & day<1920
		gen day1920_3000= 1920<=day & day<3000
* fit an exponential model with a piecewise constant hazard
	streg  sex age_ln mar_c2 card ptca day0_15 day15_30 day30_60 day60_120 ///
		day120_480 day480_840 day840_1200 day1200_1920 day1920_3000, nohr dist(exp)
	estimates store pexpon

* compute the baseline hazard
	gen h0=exp(_b[_cons] + _b[day0_15]*day0_15 + _b[day15_30]*day15_30 + _b[day30_60]*day30_60 ///
		+ _b[day60_120]*day60_120 + _b[day120_480]*day120_480 + _b[day480_840]*day480_840 ///
		+ _b[day840_1200]*day840_1200 + _b[day1200_1920]*day1200_1920 + _b[day1920_3000]*day1920_3000)
* graph the baseline hazard (up to day 2000)
	sort day
	twoway line h0 day , connect(J) scheme(s1mono) ///
		ytitle("baseline hazard",m(r+2)) xtitle("time (days)", m(t+2)) ///
		plotregion(style(none)) ylabel(,angle(horizontal)) 
	graph save "figs/fig19_exp_reg_pc.gph", replace
	graph export "figs/fig19_exp_reg_pc.emf", replace

*************************************************************************
* Example 19.17  Weibull model 
*************************************************************************
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit a Weibull model
	streg sex age_ln mar_c2 card ptca, nohr dist(weib)
		estimates store weib
* compare models
	estimates table cox expon pexpon weib, p
* fit a Gompertz model (data not shown in text)
	streg sex age_ln mar_c2 card ptca, nohr dist(gompertz)
		estimates store gomp


*************************************************************************
* Fig 19.24 - Hazard function and survival time distributions - log logitic model
*************************************************************************
* set up the initial dataset of 200 records
	clear
	set obs 2000
	set seed 12345
	set more off
	gen id=_n
* generate values for hazard over 100 days
	gen t=id if id<=100
	gen x1=0
	gen day=.
	gen fail=0
	gen failday=.
	gen hz=.
* compute the baseline log-logistic hazard (three functions)
	* formulae derived from Collett 2nd Ed (p 176, 195-200)
 		* k=4, th=-11.98    	k=4, th=-11.98 --> median t is 20
  		 	scalar k=4	
			scalar th=-11.98
			gen hz_4=(exp(th)*k*(t^(k-1))) / (1+exp(th)*(t^k))
		* k=2, th=-5.99    	k=2, th=-5.99 --> median t is 20
  		 	scalar k=2	
			scalar th=-5.99
			gen hz_2=(exp(th)*k*(t^(k-1))) / (1+exp(th)*(t^k))
		* k=1, th=-3    	k=1, th=-3 --> median t is 20
  		 	scalar k=1	
			scalar th=-3
			gen hz_1=(exp(th)*k*(t^(k-1))) / (1+exp(th)*(t^k))
		* k=0.5, th=-1.5    	k=0.5, th=-1.5 --> median t is 20
  		 	scalar k=0.5	
			scalar th=-1.5
			gen hz_05=(exp(th)*k*(t^(k-1))) / (1+exp(th)*(t^k))
		line hz_4 hz_2 hz_05 t, scheme(s1mono) xtitle("time (days)", m(t+2)) ///
			lcolor(black black black) lpattern(solid dash dot) ///
			clwidth(medium medium thick) legend(off) ///
			ytitle("hazard",m(r+2)) plotregion(style(none)) ///
			text(.1 40 "gamma=0.25", place(NE)) text(.05 30 "gamma=0.5", place(NE)) ///
			text(.01 40 "gamma=2", place(NE)) ylabel(,angle(horizontal))
		graph save "C:\mer\ch19\data\fig19_log_logist_a.gph", replace
* simulate the data, one day at a time
	forvalues i=1/100 {
	gen rand = runiform()
	scalar HZ = hz_4[`i']
	quietly replace failday = `i' if rand<HZ & fail==0
	quietly replace fail = !missing(failday) // no need to be specific about updates
	drop rand
}
* censor all remaining observations at day 101 and graph distn. of survival times
	replace failday=101 if fail==0
      stset failday, f(fail) id(id)
	histogram failday, width(2) start(0) frequency ///
		scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		mcolor(black) ytitle("frequency",m(r+2)) plotregion(style(none)) ///
		text(100 50 "gamma=0.25", place(NE)) ylabel(,angle(horizontal))
	graph save "C:\mer\ch19\data\fig19_log_logist_b.gph", replace
* combine the two graphs
	graph combine fig19_log_logist_a.gph fig19_log_logist_b.gph, ///
		scheme(s1mono) plotregion(style(none)) 
	graph export "figs/fig19_log_logist.emf", replace


*************************************************************************
* Example 19.18 - Log-logistic model 
*************************************************************************
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit a Weibull model
	streg sex age_ln mar_c2 card ptca, dist(llogistic)
		estimates store loglog
* computing median survival time
	di exp(8.559)
* is there an initial rise in hazard?
	ltable surv_mi died if surv_mi<=10, hazard
	
*************************************************************************
* Example 19.19 - Comparison of parametric models
*************************************************************************
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit all distributions to generate time ratios
	streg sex age_ln mar_c2 card ptca, nohr dist(exp) tr
		estimates store expontr
	streg sex age_ln mar_c2 card ptca, nohr dist(weib) tr
		estimates store weibtr
	streg sex age_ln mar_c2 card ptca, dist(loglog)
		estimates store loglogtr
	streg sex age_ln mar_c2 card ptca, dist(lnormal)
		estimates store lnormtr
	streg sex age_ln mar_c2 card ptca, dist(gamma)
		estimates store gammatr
	streg sex age_ln mar_c2 card ptca, nohr dist(gompertz)
		estimates store gomptr
* compare fits
	estimate stats expontr weibtr loglogtr lnormtr gammatr gomptr
* compare coefficients
	estimates table expontr weibtr loglogtr lnormtr gammatr gomptr
		
*************************************************************************
* Fig 19.25 - Effects of individual frailties
*************************************************************************
* set up the initial dataset of 2000 records
	clear
	set obs 2000
	set seed 12345
	set more off
	gen id=_n
* set up some variables
	gen t=id if id<=100
	gen x1=0
	gen day=.
	gen fail=0
	gen failday=.
*  generate gamma frailties (xg) - mu=1 sd=1 
 		gen xg=rgamma(1,1)
			histogram xg, start(0) width(.5) name(gamma, replace)
* compute the baseline constant hazard (0.1) and add frailty effect
	gen hz=0.05 * xg
* simulate the data, one day at a time
forvalues i = 1/100 {
   gen rand = runiform()
   quietly replace failday = `i' if rand<hz & fail==0
   quietly replace fail = !missing(failday)
   drop rand
}
* censor all remaining observations at day 101
	replace failday=101 if fail==0
      stset failday, f(fail)
* fit a Weibull model (no predictors)
	streg , dist(weibull) nohr
	stcurve, hazard ///
		scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		mcolor(black) ytitle("hazard",m(r+2)) plotregion(style(none)) ///
		ylabel(,angle(horizontal)) title("Empirical hazard") name(emphaz, replace)
	graph export "figs/fig19_ind_frailty.emf", replace
* fit a Weibull model with a gamma frailty (not in text)
	streg , dist(weibull) frailty(gamma)
	stcurve, hazard alpha1  ///
		scheme(s1mono) xtitle("time (days)", m(t+2)) ///
		mcolor(black) ytitle("hazard",m(r+2)) plotregion(style(none)) ///
		ylabel(,angle(horizontal)) title("Conditional hazard") name(condhaz, replace)

*************************************************************************
* Example 19.20  Individual frailty - MI data
*************************************************************************
* open the MI dataset and keep only cases that had previous bypass surgery (n=391)
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit a Weibull model (no time varing covariates)
	* note convergence tolerance relaxed to achieve convergence
	streg sex age_ln mar_c2 card ptca, nohr dist(weib) fr(gamma) ltol(0.01) tol(0.01) nrtol(0.01)


*************************************************************************
* Example 19.21 - Weibull model with shared gamma frailty 
*************************************************************************
* open the MI dataset and keep only cases that had previous bypass surgery (n=391)
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit a Weibull model 
	streg sex age_ln mar_c2 card ptca, nohr dist(weib) frailty(gamma) shared(hosp)
		estimates store weibshfr
	estimates table weib weibshfr
	
*************************************************************************
* Example 19.22 - Cox model with shared gamma frailty 
*************************************************************************
* open the MI dataset and keep only cases that had previous bypass surgery (n=391)
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* fit a Cox model 
	stcox sex age_ln mar_c2 card ptca, nohr frailty(gamma) shared(hosp) efron
		estimates store coxshfr
	estimates table weib weibshfr age_ln coxshfr


*************************************************************************
* Example 19.23 - Cox Model (shared frailty)  - fit by Poisson regression
*************************************************************************
* open the MI dataset and keep only cases that had previous bypass surgery (n=391)
	use mi.dta, clear
	gen age_ln=ln(age)-4
	stset surv_mi, f(died) id(id)
* Cox model - no frailty
	stcox sex age_ln mar_c2 card ptca, nohr efron
		estimates store coxnofr
* Cox models with shared frailty (gamma distn)
	stcox sex age_ln mar_c2 card ptca, nohr efron shared(hosp)
		estimates store coxgamfr
* gernerate events dataset
	stsplit, at(failures) riskset(RS)
	replace died =0 if died==.
	sort id surv_mi
	by id: gen len=cond(_n>1, surv_mi-surv_mi[_n-1], surv_mi)
	gen lnlen=ln(len)
* ordinary Cox model via Poisson
	* Poisson - fixed effects for risk intervals
		xtpoisson died sex age_ln mar_c2 card ptca, fe i(RS) offset(lnlen)
			estimates store Pfenofr
	* Poisson - replace fixed effects with smoothed high-order function of time
		orthpoly surv_mi, gen(t1-t6) degree(6)
		poisson died sex age_ln mar_c2 card ptca t1-t6, offset(lnlen)
			estimates store Psmnofr
* random effects model (shared frailty)
	* Poisson - gamma frailty with smoothed high-order function of time
		xtpoisson died sex age_ln mar_c2 card ptca t1-t6, re i(hosp) offset(lnlen)
			estimates store Psmgamfr
	* log-normal frailty (not in text - takes quite a while to run)
		/* xtpoisson died sex age_ln mar_c2 card ptca t1-t6, re i(hosp) offset(lnlen) normal
			estimates store Psmnormfr */
* comparing models
	estimates stats coxnofr Pfenofr Psmnofr coxgamfr Psmgamfr 
		* note: lnL of null models not the same - can't compare fits
	estimates table coxnofr Pfenofr Psmnofr coxgamfr Psmgamfr , b(%6.3f) se(%6.3f) p(%6.3f)

*************************************************************************
* Example 19.24 - Multiple failure time data
*************************************************************************
* open the multiple event MI dataset
	use mi_mult.dta, clear
		sort id agemi
		* generate a variable that identifies the last record for each person
		by id: gen exit=(_N==_n)
* Anderson-Gill model
	* generate number of previous cases
		quietly by id: gen prev_mi=(_n-1)
		stset agemi, f(mi) id(id) exit(exit=1)
		br
	* fit a Cox model with robust SE
		stcox sex white prev_mi, nohr robust efron
			estimates store AGcoxrbst
	* fit a Weibull model with robust SE
		streg sex white prev_mi, dist(weibull) nohr robust 
			estimates store AGweibrbst
	* fit a Weibull model with a gamma frailty
		* note convergence tolerance relaxed to achieve convergence
		streg sex white prev_mi, dist(weibull) nohr shared(id) ltol(0.001) tol(0.001) nrtol(0.001)
			estimates store AGweibshrd
	* create some variables for printing
		gen t0_ag=_t0
		gen t_ag=_t
* Marginal risk set model (Wei, Lin and Weissfeld) (not included in text)
	* not recommended by Hosmer et al
	* generate identifier for risk set
		gen risk_set=prev_mi+1
		stset agemi, f(mi) 
		br
		stcox sex white , nohr efron cluster(id) strata(risk_set)
			estimates store WLWcoxrbst		
	* create some variables for printing
		gen t0_wlw=_t0
		gen t_wlw=_t
* Conditional risk set model (Prentice, Williams, Peterson)
	* time from entry
		stset agemi, f(mi) id(id) exit(exit==1)
		br
	* fit a Cox model with robust SE
		stcox sex white, nohr robust strata(risk_set) efron
			estimates store PWP0coxrbst
	* fit a Weibull model with robust SE	
		streg sex white, nohr robust strata(risk_set) dist(weibull)
			estimates store PWP0weibrbst
	* create some variables for printing
		gen t0_pwp0=_t0
		gen t_pwp0=_t
* Conditional risk set model (Prentice, Williams, Peterson)
	* time from previous event
	* generate time zero = 0 and time = length of interval
		replace _t=_t-_t0
		replace _t0=0
		br
	* fit a Cox model with robust SE
		stcox sex white, nohr robust strata(risk_set) efron
			estimates store PWP1coxrbst
	* fit a Weibull model with robust SE	
		streg sex white, nohr robust strata(risk_set) dist(weibull)
			estimates store PWP1weibrbst
	* create some variables for printing
		gen t0_pwp1=_t0
		gen t_pwp1=_t
* compare models and create tables of estimates
	estimates stats AGcoxrbst AGweibrbst AGweibshrd WLWcoxrbst PWP0coxrbst PWP0weibrbst ///
		PWP1coxrbst PWP1weibrbst
	estimates table AGcoxrbst AGweibrbst AGweibshrd  ///
		PWP1coxrbst PWP1weibrbst, b(%6.3f) se(%6.3f)
* listing data for the example
	format risk_set t0_ag t_ag t0_wlw t_wlw t0_pwp0 t_pwp0 t0_pwp1 t_pwp1  %4.0f
	list id agemi mi sex white prev_mi risk_set t0_ag t_ag  ///
		t0_pwp1 t_pwp1 if id==19 | id==20, abb(4) clean noobs
	

*************************************************************************
* Example 19.25 & 19.26 - Discrete time survival analysis & Compl. Loglog
*************************************************************************
* open the MI dataset
	use mi.dta, clear
	gen age_ln=ln(age)-4
* divide data into intervals with starting points 0 10 20 30 45 60 90 120 180 360 720 1440 3000
	* cutpoints included in upper, censored observations excluded
		gen surv_mi_d=surv_mi+0.001
			* add a small quantity to -surv_mi which puts observed falling on the 
			* cut points into the category after the cut point- 
		stset surv_mi_d, f(died) id(id)
		stsplit per, at(0 10 20 30 45 60 90 120 180 360 720 1440 3000)
			* drop last period for censored observations (ie cens. obs. excluded)
				bysort id (_t): drop if _n==_N & _n>1 & died==0
					tab per _d, row
		* generate material for Table 19.4
			* table with card=0
			preserve
				collapse (count) n=_d (sum) d=_d (min) t0=_t0 (max) t=_t if card==0, by(per) 
				gen h=100*(d/n)/(t-t0)
				list t0 t n d h, clean noobs
			restore
			* table with card=1
			preserve
				collapse (count) n=_d (sum) d=_d (min) t0=_t0 (max) t=_t if card==1, by(per)
				gen h=100*(d/n)/(t-t0)
				list t0 t n d h, clean noobs
			restore
	* fit a logistic model to the data
		logit _d sex age_ln mar_c2 card ptca i.per
		* include -card- by time interval interactions
			logit _d sex age_ln mar_c2 ptca i.card##i.per
				testparm  i.card#i.per
				estimates store DTlogit
		* compare with complementary log-log regression
			cloglog _d sex age_ln mar_c2 ptca i.card##i.per
				testparm i.card#i.per
				estimates store DTloglog
	estimates stats DTlogit DTloglog
	estimates table DTlogit DTloglog

*************************************************************************
* Figure 19.27 - Complementary log-log function
*************************************************************************
twoway (function y = logit(x), range(0 1) lpattern(dash) lcolor(black)) ///
   (function y = cloglog(x), range(0 1) lpattern(solid) lcolor(black)) ///
		, scheme(s1mono) legend(off) plotregion(style(none))  ///
		ytitle("logit / cloglog", m(r+2)) ///
		xtitle("probability", m(t+2)) ylabel(,angle(horizontal)) ///
		text(2 .75 "logit", place(NE)) text(-1 .5  "complementary log-log", place(NE))
	graph export "figs/fig19_cloglog.emf", replace

*************************************************************************
* Example 19.27 Sample size calculation	Note: -artsurv- is a Stata add-on
*************************************************************************
	* Scenario a
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(.333 .333 ) recrt(0 0, 1, 0 ) ///
			distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
	* Scenario b
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(.333 .333 ) lg(1 2 3 ) ///
			ldf( .05 .15 .3 .4, 1 2 3 4; .05 .15 .3 .4, 1 2 3 4; .05 .15 .3 .4, 1 2 3 4 ) ///
			recrt(0 0, 1, 0 ) distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
	* Scenario c
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(.333 .333 ) lg(1 2 3 ) ///
			ldf( .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4 ) ///
			recrt(0 0, 1, 0 ) distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
	* Scenario d
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(.333 .333 ) wg(1 2 ) ///
			wdf(.2, 1; .1, 1) crover(3 3 ) lg(1 2 3 ) ldf( .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4 ) ///
			recrt(0 0, 1, 0 ) distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
	* Scenario e
		artsurv, method(l) nperiod(4) ngroups(3) fp(0) edf0(.7 .5 .4 .35, 1 2 3 4) ///
			hratio(.75, .5) alpha(0.05) power(0.8) aratios(3 2 ) wg(1 2 ) wdf(.2, 1; .1, 1) ///
			crover(3 3 ) lg(1 2 3 ) ldf( .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4; .1 .3 .6 .8, 1 2 3 4 ) ///
			recrt(0 0, 1, 0 ) distant(0) detail(0) onesided(0) ni(0) tunit(1) trend(0)
