* mer20_all.do
* Introduction to clustered data
*
	version 12
	clear all
	set more off , permanent
	cd c:\mer\ch20\data		/* this line is computer dependent */
	capture log close 
	log using mer20_all, name(mer20_all) text replace 

*************************************************************************
* Disclaimer
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
*/

*************************************************************************
* Open and declare the data to be survival data (stset)
*************************************************************************
use c:\mer\data\brazil.dta, clear

*************************************************************************
* Example 20.1 - Hierarchical structure of reu_cfs data
*************************************************************************
   * determine number of units at each level
		codebook mun comm fam id
   * aggregate to household level
		collapse (count) nhouse=id, by(mun comm fam)
   * determine distribution at household level
		sum nhouse
   * aggregate to community level
		collapse (count) ncomm=nhouse, by(mun comm)
   * determine distribution at community level
		sum ncomm
   * aggregate to municipality level
		collapse (count) nmun=ncomm, by(mun)
   * determine distribution at municipality level
		sum nmun

*************************************************************************
* Example 20.2 - clustering in a continuous data model
*************************************************************************
* open the simulated data with -X- as a clinic level variable
	use c:\mer\data\simcont_clustclin.dta, clear
	* fit an ordinary linear regression
		regress bmi X
	* fit a linear mixed model with clin as the cluster
		xtmixed bmi X || clin:
	* collapse the data to the clin level and fit an ordinary reg.
		collapse (mean) bmi X, by(clin)
		regress bmi X
* open the simulated data with -X- as an individual level variable
	use c:\mer\data\simcont_clustind.dta, clear
	* fit an ordinary linear regression
		regress bmi X
	* fit a linear mixed model with clin as the cluster
		xtmixed bmi X || clin:

*************************************************************************
* Example 20.3 - clustering in a binary data model
*************************************************************************
* open the dataset -X- as a clin level variable
	use c:\mer\data\simbin_clustclin.dta, clear
	* fit an ordinary logistic regression
		logit Y X
	* fit a random effects model with clin as the cluster
		xtlogit Y X, i(clin)
* open the dataset -X- as an individual level variable
	use c:\mer\data\simbin_clustind.dta, clear
	* fit an ordinary logistic regression
		logit Y X
	* fit a a random effects model with clin as the cluster
		xtlogit Y X, i(clin)

*************************************************************************
* Figure 20.5 - Simulation study on effects of clustering
*************************************************************************
* open the dataset with 1000 simulation results
	use c:\mer\data\simbin_1000.dta, clear
	* graph the coef. for X and its SE over various levels of ICC of X 
	twoway (line xbias_b1 icc, sort lcolor(black) lpattern(solid)) ///
		(line xbias_se1 icc, lcolor(black) lpattern(dash)), ///
		ytitle("bias", m(r+2) size(medlarge)) ylabel(#6, angle(horizontal)) ///
		xtitle("ICC of X (%)", m(t+2) size(medlarge)) ///
		legend(off) name(simbin_1, replace) scheme(s1mono) ///
		text(.6 65 "SE(beta)", place(NE) size(medlarge)) text(.87 50  "beta", place(NE) size(medlarge)) 
* open the dataset with the raw data from the 1000 simulations
	use c:\mer\data\simbin_1000_all.dta, clear
	replace icc=100*icc
	* graph the SD of the estimates from ordinary and random effects log. regn.
		preserve
		collapse (mean) o_mu=X1o_b1b r_mu=X1r_b1b (sd) o_sd=X1o_b1b r_sd=X1r_b1b, by(icc)
		twoway (line o_sd icc, sort lcolor(black) lpattern(solid)) ///
			(line r_sd icc, lcolor(black) lpattern(dash)), ///
			ytitle("SD of estimates of beta", m(r+2) size(medlarge)) ///
			ylabel(#6, angle(horizontal)) xtitle("ICC of X (%)", m(t+2) size(medlarge)) ///
			legend(off) name(simbin_2, replace) scheme(s1mono) ///
			text(.09 28 "ordinary" "logistic regression", place(NE) size(medlarge)) ///
			text(.05 50 "random effects" "logistic regression", place(NE) size(medlarge))
		restore
* combine the two graphs
	graph combine simbin_1 simbin_2, scheme(s1mono)  ysize(4) xsize(8) 
	graph save "figs/fig20_bias.gph", replace
	graph export "figs/fig20_bias.emf", replace

*************************************************************************
* Figure 20.6 - Clustering and confounding
*************************************************************************
* open the dataset with 1000 simulation results
	use c:\mer\data\simbin_1000.dta, clear
	* graph the coef. for X and its SE over various levels of ICC of Z 
	twoway (line zbias_b1 icc, sort lcolor(black) lpattern(solid)) ///
		(line zbias_se1 icc, lcolor(black) lpattern(dash)), ///
		ytitle("bias", m(r+2)) ylabel(#6, angle(horizontal)) xtitle("ICC of Z (%)", m(t+2)) ///
		legend(off) name(Z_B1_dich, replace) scheme(s1mono) ///
		text(1.37 20 "Beta", place(NE)) text(.97 50  "SE(Beta)", place(NE)) 
	graph save "figs/fig20_conf.gph", replace
	graph export "figs/fig20_conf.emf", replace

*************************************************************************
* Example 20.4 - Analyses for binary data
*************************************************************************
* open the simulated binary data with -X- as an individual-level variable
	use c:\mer\data\simbin_clustind.dta, clear
	* fit a fixed effects logistic regression model
		xi:logit Y X 
		xi:logit Y X i.clin
	* carry out a stratified analysis
		cc Y X, by(clin)
	* manual calculation of SE on log-scale
		di (r(ub_or)-r(lb_or))/2/(2*1.96)
	* logistic regression with robust SE
		logit Y X, vce(cluster clin)
        * GEE analysis with exchangeble correlation structure
                set matsize 400
                xtgee Y X, fam(bin) link(logit) i(clin) corr(exch) robust
	* survey methods with clustering at clinics
		svyset clin
		svy: logit Y X

* open the simulated binary data with -X- as a clinic-level variable
	use simbin_clustclin.dta, clear
	* aggregate to clin level
		collapse X (sum) pos=Y (count)n=Y, by(clin)
	* fit a generalised linear model with overdispersion
		glm pos X, fam(bin n) link(logit) scale(x2)
	* Williams method (williams command from J Hilbe)
		williams pos n X
	use simbin_clustclin.dta, clear
	* logistic regression with robust SE
		logit Y X, vce(cluster clin)
        * GEE analysis with exchangeble correlation structure
                set matsize 400
                xtgee Y X, fam(bin) link(logit) i(clin) corr(exch) robust
	* survey methods with clustering at clinics
		svyset clin
		svy: logit Y X


*************************************************************************
* Example 20.5 - Analyses for continuous data
*************************************************************************
* open the simulated continuous data with -X- as an individual-level variable
	use simcont_clustind.dta, clear
	* fit a fixed effects linear regression model
		xi: regress bmi X i.clin
	* linear regression with robust SE
		regress bmi X, vce(cluster clin)
        * GEE analysis with exchangeble correlation structure
                set matsize 400
                xtgee bmi X, i(clin) corr(exch) robust
	* survey methods with clustering at clinics
		svyset clin
		svy: regress bmi X

* open the simulated continuous data with -X- as a clinic-level variable
	use simcont_clustclin.dta, clear
	* linear regression with robust SE
		regress bmi X, vce(cluster clin)
        * GEE analysis with exchangeble correlation structure
                set matsize 400
                xtgee bmi X, i(clin) corr(exch) robust
	* survey methods with clustering at clinics
		svyset clin
		svy: regress bmi X


*************************************************************************
* Example 20.6 - Survey analysis for complex survey data
*************************************************************************
* open the Brazil sampling dataset (contains sampling weights)
	use "C:\mer\data\brazil_smpl.dta", clear
		* there were a few households in which the # sampled exceeded the number reported to be in the household
		* drop these records (required for the finite pop. correction later in this example)
			bysort fam: gen f2=_N
			drop if fam_n<f2
	* survey methods: household clustering (only)
		svyset fam, clear 
		svy: logit diarr age cistern
	* same results with robust variance
		logit diarr age cistern, vce(cluster fam)
	* survey methods: household clustering and sampling weights
		svyset fam [pw=wt], clear
		svy: logit diarr age cistern
	* same results with robust variance
		logit diarr age cistern [pw=wt], vce(cluster fam)
	* survey methods: household clustering, sampling weights and fixed municipality effects
		svyset fam [pw=wt], clear
		svy: logit diarr age cistern i.mun
	* same results with robust variance
		logit diarr age cistern i.mun [pw=wt], vce(cluster fam)
	* survey methods: household clustering, sampling weights and stratification by municipality
		svyset fam [pw=wt], strata(mun) clear
		svy: logit diarr age cistern
	* survey methods: herd clustering, sampling weights, stratification by municipality
		* and fpc at municipality and household levels
		svyset fam [pw=wt], strata(mun) fpc(m_pop_families) || id , fpc(fam_n)
		svy: logit diarr age cistern 
