* ch21.do
	* command for all examples and figures in Chapter 21
	* and a few analyses that are not included in the chapter
*
	clear all
	version 12
	set more off 
	cd c:\mer\ch21\data		/* this line is computer dependent */
	capture log close ch21_all
	log using ch21_all, name(ch21_all) replace 

*************************************************************************
* Example 21.1 - Variance components and random effects
*************************************************************************
* open the bp dataset
	use bp, clear
* estimate parameters in 'null' model
	xtmixed dbp || centre: if visit==3, var reml
* calculation of variance proportion or ICC using Eq 21.4
	di 3.430885/(3.430885+81.39656)  
* calculation of approximate ICC using Eq. 21.14
	anova dbp centre if visit==3
	di (110.203613-81.6147768)/(110.203613+(287/29-1)*81.6147768)

*************************************************************************
* Example 21.2 - Mixed model estimates for 2-level blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* centre the initial dbp (dbp1) by subtracting the median
	capture drop dbp1c
	gen dbp1c=dbp1-102
	label var dbp1c "cent.dbp1: dbp1-102"
* estimate parameters with all predictors
	xtmixed dbp i.tx dbp1c || centre: if visit==3, var reml

*************************************************************************
* Example 21.3 - Intra-class correlations in a 4-level mixed model
*************************************************************************
* not based on datasets included

*************************************************************************
* Fig 21.1 - Interaction graph
*************************************************************************
* note: centres 1,3,15,11,36 
twoway (function y=37.43 + .558*x, range(90 120) lcolor(black)) (function y=38.53 + .558*x, range(90 120) lcolor(black)) ///
	(function y=37.07 + .558*x, range(90 120) lcolor(black)) (function y=36.67 + .558*x, range(90 120) lcolor(black)) ///
	(function y=35.56 + .558*x, range(90 120) lcolor(black)), scheme(s1mono)  ///
	ytitle("blood pressure (y)", m(r+2)) ylabel(#2, angle(horizontal)) ///
	xtitle("initial blood pressure (x)", m(t+2)) plotregion(style(none)) legend(off) ///
	title("additive model (no interaction)") 
	graph save "figs/fig21_intn_1.gph", replace
	graph export "figs/fig21_intn_1.wmf", replace

twoway (function y=74.08 + .200*x, range(90 120) lcolor(black)) (function y=-33.83 + 1.257*x, range(90 120) lcolor(black)) ///
	(function y=8.59 + 0.837*x, range(90 120) lcolor(black)) (function y=49.06 + .450*x, range(90 120) lcolor(black)) ///
	(function y=32.15 + .593*x, range(90 120) lcolor(black)), scheme(s1mono)  ///
	ytitle("blood pressure (y)", m(r+2)) ylabel(#2, angle(horizontal)) ///
	xtitle("initial blood pressure (x)", m(t+2)) plotregion(style(none)) legend(off) ///
	title("interaction - group and initial blood pressure") 
	graph save "figs/fig21_intn_2.gph", replace
	graph export "figs/fig21_intn_2.wmf", replace

* combine the two graphs
	graph combine "figs/fig21_intn_1.gph" "figs/fig21_intn_2.gph", ///
	ysize(4) xsize(8) scheme(s1mono)
	graph save "figs/fig21_intn_comb.gph", replace
	graph export "figs/fig21_intn_comb.wmf", replace

*************************************************************************
* Example 21.4 - Random slopes of dbp1c for blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* estimate random slope parameters for dbp1c
	xtmixed dbp i.tx dbp1c || centre: dbp1c if visit==3, cov(un) var reml

*************************************************************************
* Figure 21.2 - Variance function of random slopes model
*************************************************************************
* generate pairs of points (dbp1,var(y)) based on Eq 21.11
	clear
	set obs 29
	gen dbp1=_n+91
	gen dbp1c=dbp1-102
	gen vary=1.772541+dbp1c^2*0.2853688+2*dbp1c*(-0.1384331)+67.72989
* plot points connected
	twoway (connected vary dbp1, sort msymbol(none) lwidth(thick)), ytitle(var(Y)) ///
        ylabel(50(50)200) xtitle(initial blood pressure) xlabel(90(10)120) scheme(s1mono)
	graph save "figs/fig21_varfunc", replace
	graph export "figs/fig21_varfunc.wmf", replace

*************************************************************************
* Example 21.5 - Random slopes of tx for blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* generate indicator (dummy) variables for tx
	tabulate tx, gen(tx)
* estimate random slope parameters for tx
	xtmixed dbp i.tx dbp1c || centre: tx2 tx3 if visit==3, cov(un) var reml
* estimate factorial random effects for tx (i.e. a 3-level model)
	xtmixed dbp i.tx dbp1c || centre: || centre: R.tx if visit==3, var reml

*************************************************************************
* Figure 21.3 - Contextual effects
*************************************************************************
use cntx.dta, clear
twoway (scatter mid1 x) (function y=0.5852 + .0557*x, range(1 9) lpat(dash) lcol(black)) ///
	(function y=0.8 - 3*.0557 + .0557*x, range(1 5) lcolor(black)) ///
	(function y=0.7 - 4*.0557 + .0557*x, range(2 6)lcolor(black)) ///
	(function y=1.0 - 5*.0557 + .0557*x, range(3 7)lcolor(black)) ///
	(function y=0.75 - 6*.0557 + .0557*x, range(4 8)lcolor(black))  ///
	(function y=1.1 - 7*.0557 + .0557*x, range(5 9)lcolor(black))  ///
	(function y=1.0 - 8*.0557 + .0557*x, range(6 10)lcolor(black))  ///
	, scheme(s1mono) 	ytitle("y", m(r+2)) ylabel(#2, angle(horizontal)) ///
	xtitle("x", m(t+2)) plotregion(style(none)) legend(off) ///
	title("no contextual effect") 
	graph save "figs/fig21_cntx_1.gph", replace
	graph export "figs/fig21_cntx_1.wmf", replace
twoway (scatter mid2 x) (function y=1.1981 - .0557*x, range(1 9) lpat(dash) lcol(black)) ///
	(function y=1.0 - 3*.0557 + .0557*x, range(1 5) lcolor(black)) ///
	(function y=1.1 - 4*.0557 + .0557*x, range(2 6)lcolor(black)) ///
	(function y=0.75 - 5*.0557 + .0557*x, range(3 7)lcolor(black)) ///
	(function y=1.0 - 6*.0557 + .0557*x, range(4 8)lcolor(black))  ///
	(function y=0.7 - 7*.0557 + .0557*x, range(5 9)lcolor(black))  ///
	(function y=0.8 - 8*.0557 + .0557*x, range(6 10)lcolor(black))  ///
	, scheme(s1mono) 	ytitle("y", m(r+2)) ylabel(#2, angle(horizontal)) ///
	xtitle("x", m(t+2)) plotregion(style(none)) legend(off) ///
	title("contextual effect") 
	graph save "figs/fig21_cntx_2.gph", replace
	graph export "figs/fig21_cntx_2.wmf", replace
* combine the two graphs
	graph combine "figs/fig21_cntx_1.gph" "figs/fig21_cntx_2.gph", ///
	 ysize(4) xsize(8) scheme(s1mono)
	graph save "figs/fig21_cntx_comb.gph", replace
	graph export "figs/fig21_cntx_comb.wmf", replace

*************************************************************************
* Example 21.6 - Contextual effects for blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* compute centre means of predictor (dbp1c)
	bysort centre: egen hdbp1c=mean(dbp1c) if visit==3
* compute group-mean centered version of dbp1c
	gen zdbp1c=dbp1c-hdbp1c
* estimate random intercept model
	xtmixed dbp i.tx dbp1c || centre: if visit==3, var reml
* estimate contextual model (two versions)
	xtmixed dbp i.tx dbp1c hdbp1c || centre: if visit==3, var reml
	xtmixed dbp i.tx zdbp1c hdbp1c || centre: if visit==3, var reml
* estimate random slopes model
	xtmixed dbp i.tx dbp1c || centre: dbp1c if visit==3, cov(un) var reml
* estimate contextual and random slopes model (two versions)
	xtmixed dbp i.tx dbp1c hdbp1c || centre: dbp1c if visit==3, cov(un) var reml
	xtmixed dbp i.tx zdbp1c hdbp1c || centre: dbp1c if visit==3, cov(un) var reml
* extra: group-mean centered random slopes (not the same model!)
	xtmixed dbp i.tx zdbp1c hdbp1c || centre: zdbp1c if visit==3, cov(un) var reml

*************************************************************************
* Section 21.5.1: Approximation formula for ICC
*************************************************************************
* open the bp dataset
	use bp, clear
* calculation of approximate ICC using Eq. 21.14
	anova dbp centre if visit==3
	di (110.203613-81.6147768)/(110.203613+(287/29-1)*81.6147768)

*************************************************************************
* Example 21.7 - Inference for fixed effects for blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* estimate parameters with all predictors (same as Example 21.2)
	xtmixed dbp i.tx dbp1c || centre: if visit==3, var reml
	testparm i.tx
	lincom 2.tx-3.tx

*************************************************************************
* Example 21.8 - Inference for random effects for blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* estimate model without random effects and store results
	xtmixed dbp i.tx dbp1c if visit==3, var reml
	estimates store onelevel
* estimate models of Example 21.2 and store results
	xtmixed dbp i.tx dbp1c || centre: if visit==3, var reml
	estimates store randint
* likelihood-ratio test for random effects
	lrtest onelevel randint, stats
* manual calculation of P-value
	di chi2tail(1,1.08)/2
* estimate models of Examples 21.4-5 and compute likelihood-ratio tests
* random slope of dbp1c
	xtmixed dbp i.tx dbp1c || centre: dbp1c if visit==3, cov(un) var reml
	estimates store rslopedbp1c
	lrtest randint rslopedbp1c, stats
	di chi2tail(2,9.24)/2
* random slopes of tx
	tabulate tx, gen(tx)
	xtmixed dbp i.tx dbp1c || centre: tx2 tx3 if visit==3, cov(un) var reml
	estimates store rslopetx
	lrtest randint rslopetx, stats
	di chi2tail(5,1.91)/2
* random tx by group interaction
	xtmixed dbp i.tx dbp1c || centre: || centre: R.tx if visit==3, var reml
	estimates store rslopetxf
	lrtest randint rslopetxf, stats
	di chi2tail(1,1.30)/2

*************************************************************************
* Example 21.9 - Residuals and diagnostics for blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* for simplicity of operations on residuals, reduce data to visit=3
	keep if visit==3   
* reestimate model from Example 21.2
	xtmixed dbp i.tx dbp1c || centre: if visit==3, var reml
	estimates store examp21_2
* patient-level raw residuals and predictions (fixed effects only and both fixed and random effects)
	predict resid_subj, residuals
	predict predf_subj, xb
	predict pred_subj, fitted
	* note: subject-level standardised residuals from xtmixed are just scaled by sigma (not really standardised) 
* raw centre-level predictions/residuals
	predict resid_centre, reffects level(centre)
* centre-level predicted values based on fixed effects
	bysort centre: egen pred_centre=mean(predf_subj)

* reanalyse with gllamm to obtain standardised residuals (after ML estimation)
	xi: gllamm dbp i.tx dbp1c if visit==3, i(centre) adapt
* standardised subject-level residuals
	gllapred sresid_subj, pearson
* standardized centre-level residuals
	gllapred res_, ustd 
	rename res_m1 sresid_centre
	drop res_s1
* centre-level Cook's distance (only for top level)
	gllapred cook_centre, cooksd

* inspect residuals etc at centre level
	bysort centre: gen within_gr=_n
	bysort centre: egen ncentre=count(patient)
	br centre ncentre resid_centre sresid_centre cook_centre pred_centre if within_gr==1
	list centre ncentre resid_centre sresid_centre cook_centre pred_centre if within_gr==1
* inspect residuals etc at subject level (not in text)
	br centre patient resid_subj sresid_subj predf_subj pred_subj
* normal normality test of standardised residuals at centre level
	swilk sresid_centre if within_gr==1
*************************************************************************
* Figure 21.4 - group-level residuals 
*************************************************************************
	qnorm sresid_centre if within_gr==1, ytitle(Group-level standardised residual, m(r+2)) ///
		ylabel(,angle(horizontal)) xtitle(inverse normal distribution, m(t+2)) scheme(s1mono) 
	graph save "figs/fig21_res1.gph", replace
	graph export "figs/fig21_res1.wmf", replace
	twoway (scatter sresid_centre pred_centre) if within_gr==1, ///
		ytitle(Group-level standardised residual, m(r+2)) ylabel(,angle(horiz)) ///
		xtitle(Group-level fixed effects prediction, m(t+2)) scheme(s1mono)
	graph save "figs/fig21_res2.gph", replace
	graph export "figs/fig21_res2.wmf", replace
* combine the two graphs
	graph combine "figs/fig21_res1.gph" "figs/fig21_res2.gph", ysize(4) xsize(8) scheme(s1mono)
	graph save "figs/fig21_res_comb.gph", replace
	graph export "figs/fig21_res_comb.wmf", replace

* normal plot, normality test and residual plots at subject level (not in text)
	qnorm sresid_subj
	swilk sresid_subj
	scatter sresid_subj pred_subj
	scatter sresid_subj predf_subj
* subject-level residuals assessed against predictors (not in text)
	bysort tx: summarize sresid_subj
	scatter sresid_subj dbp1
* assessing influential groups (not shown in text)\
	xtmixed dbp i.tx dbp1c || centre: if visit==3 & centre~=1, var reml
	estimates store excl_1
	xtmixed dbp i.tx dbp1c || centre: if visit==3 & centre~=7, var reml
	estimates store excl_7
	xtmixed dbp i.tx dbp1c || centre: if visit==3 & centre~=24, var reml
	estimates store excl_24
	xtmixed dbp i.tx dbp1c || centre: if visit==3 & centre~=31, var reml
	estimates store excl_31
	estimates table examp21_2 excl_1 excl_7 excl_24 excl_31

*************************************************************************
* Example 21.10: Box-Cox transformation for blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* calculate nobs and meanlnscc
	egen nobs=count(dbp) if visit==3
	gen lndbp=ln(dbp)
	egen meanlndbp=mean(lndbp) if visit==3
* set up a file to hold the results for different lambda
	tempname memhold
	postfile memhold lambda lnL pl using temp.dta, replace
	gen y=.
* loop over lambda values
	foreach lambda of numlist 2 1.5 1.25 1 0.75 0.5 0.25 0 -0.5 -1 {
		di `lambda'
		replace y=(dbp^(`lambda')-1)/(`lambda')
		replace y=lndbp if (`lambda')==0
		xtmixed y i.tx dbp1c || centre: if visit==3, var
		scalar lnL=e(ll)
		scalar pl=lnL+n*(`lambda'-1)*meanlndbp
		post memhold (`lambda') (lnL) (pl)
		}
* stop the process of posting results to a file
	postclose memhold
* open the file that captured the results
	use temp.dta, clear
	list lambda lnL pl
*************************************************************************
* Figure 21.5 Profile-likelihood function
*************************************************************************
* plot the profile-likelihood function against lambda
	scatter pl lambda, ytitle(profile log-likelihood, m(r+2)) ylabel(,angle(horiz)) ///
		xtitle(lambda, m(t+2)) scheme(s1mono)
	graph save "figs/fig21_bc.gph", replace
	graph export "figs/fig21_bc.wmf", replace

*************************************************************************
* Section 21.5.7: Hausman test for Example 21.2
*************************************************************************
* open the bp dataset
	use bp, clear
* fixed effects version of model in Example 21.2, fit using the xtreg procedure
	xtreg dbp i.tx dbp1c if visit==3, fe i(centre)
	estimates store fixed
* random effects model of Example 21.2, fit using the xtreg procedure
	xtreg dbp i.tx dbp1c, re i(centre)
	estimates store random 
* compute Hausman test
	hausman fixed random

*************************************************************************
* Section 21.5.8: Robust standard errors for Example 21.2
*************************************************************************
* open the bp dataset
	use bp, clear
* estimation by gllamm (xtmixed does not allow for robust standard errors)
	xi: gllamm dbp i.tx dbp1c if visit==3, i(centre) adapt
	gllamm, robust
	gllamm, cluster(centre) /* same result */

*************************************************************************
* Example 21.11: Heteroscedastic variance for blood pressure data
*************************************************************************
* open the bp dataset
	use bp, clear
* estimation by gllamm (xtmixed does not allow for heteroscedasticity)
	xi: gllamm dbp i.tx dbp1c if visit==3, i(centre) adapt
* heterogeneous treatment variance model
	tab tx, gen(tx)
	eq het2: tx1 tx2 tx3
	xi: gllamm dbp i.tx dbp1c if visit==3, i(centre) adapt s(het2)
* compute variance and its SE (formula from delta method)
	di "var "  exp(2.1538425)^2 " SE " 2*(exp(2.1538425)^2)*.07291123
	di "var "  exp(2.1161503)^2 " SE " 2*(exp(2.1161503)^2)*.07697061
	di "var "  exp(2.1644145)^2 " SE " 2*(exp(2.1644145)^2)*.07479038
* heterogeneous -dbp1c- variance model: quadratic function of -dbp1c-
	gen cons=1
	gen dbp1c_sq=dbp1c^2/10
	eq het: cons dbp1c dbp1c_sq
	xi: gllamm dbp i.tx dbp1c if visit==3, i(centre) adapt s(het)
* compute variance at dbp1=102 and its SE (formula from delta method)
	di "var "  exp(2.0741674)^2 " SE " 2*(exp(2.0741674)^2)*.05174568
* random slopes model (not in text with ML estimation)
	eq int: cons
	eq slope: dbp1c
	xi: gllamm dbp i.tx dbp1c if visit==3, i(centre) adapt nrf(2) eqs(int slope)
* heterogeneous variance and random slopes model (not in text)
	xi: gllamm dbp i.tx dbp1c if visit==3, i(centre) adapt s(het) nrf(2) eqs(int slope)
