* ch22_all.do
* Mixed Models for Discrete Data - all calculations
*
	clear all
	version 12
	set more off 
	cd c:\mer\ch22\data		/* this line is computer dependent */
	capture log close ch22_all
	log using ch22_all, name(ch22_all) replace 

**************************************************************************************
* Example 22.1 - Random effects logistic regression model for 2-level diarrhoea data
**************************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate family-level diarrhoea outcome and auxiliary variables
	bysort fam: egen famdiarr=max(diarr)
	bysort fam: gen within_f=_n
* determine the unconditional association between famdiarr and cistern
	cc famdiarr cistern if within_f==1
* logistic regression with random community effects
	xtlogit famdiarr cistern if within_f==1, i(comm)
	xtlogit famdiarr cistern if within_f==1, i(comm) or
* same analysis with xtmelogit, in order to display variance
	xtmelogit famdiarr cistern || comm: if within_f==1, var intp(12)
	gllamm famdiarr cistern if within_f==1, fam(binom) link(logit) i(comm) trace adapt nip(12)

*************************************************************************
* Example 22.3 - Random effects Poisson for 2-level diarrhoea data
*************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate family-level diarrhoea count and auxiliary variables
	bysort fam: egen famcases=sum(diarr)
	bysort fam: gen within_f=_n
* fit a random effects model - normal random effects - xtpoisson
	xtpoisson famcases cistern water_tx if within_f==1, exp(fam_n) i(comm) normal
* same analysis with xtmepoisson, in order to display variance
	xtmepoisson famcases cistern water_tx || comm: if within_f==1, exp(fam_n) var intp(12)
* same analysis with gllamm
	gen logn=ln(fam_n)
	xi:gllamm famcases cistern water_tx if within_f==1, fam(pois) link(log) off(logn) i(comm) adapt nip(12)
	* compute ICC for families with/without cisterns and water_tx
	matrix define est = e(b)
	scalar sigma=est[1,4]
	gen bx=est[1,3]+logn
	gen cist0tx0lev1=exp(bx+sigma^2/2)
	gen cist0tx0lev2=exp(2*bx+2*sigma^2)-exp(2*bx+sigma^2)
	gen cist0tx0=cist0tx0lev2/(cist0tx0lev2+cist0tx0lev1)
	label var cist0tx0 "no cistern or water tx"		
	replace bx=est[1,1]+est[1,2]+est[1,3]+logn
	gen cist1tx1lev1=exp(bx+sigma^2/2)
	gen cist1tx1lev2=exp(2*bx+2*sigma^2)-exp(2*bx+sigma^2)
	gen cist1tx1=cist1tx1lev2/(cist1tx1lev2+cist1tx1lev1)
	label var cist1tx1 "cistern and water tx"		
*************************************************************************
* Fig 22.1 - Intra-class correlation graph
*************************************************************************
	twoway (scatter cist0tx0 fam_n, msymbol(square_hollow) mcolor(black)) ///
		(scatter cist1tx1 fam_n, sort msymbol(circle) mcolor(black)), ///
		ytitle(ICC (community), m(r+2)) ylabel(,angle(horix)) xtitle(family size, m(t+2)) ///
		legend(position(11) ring(0)) scheme(s1mono)
	graph save "figs/fig22_icc.gph", replace
	graph export "figs/fig22_icc.wmf", replace

********************************************************************************
* Example 22.4 - Generalised linear mixed modles (random effects logistic and 
*                probit regression) for 4-level diarrhoea data
********************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate predictor variables
	capture drop age5
	gen age5=age<=5 if age~=.
	label var age5 "age<=5"
	capture drop smsize
	gen smsize=(m_pop_families-2000)/1000
	label var smsize "stand.m_pop_fam: (m_pop_families-2000)/1000"
* 4-level logistic regression with random effects
	xtmelogit diarr cistern water age5 smsize || mun: || comm: || fam:, var intp(12 12 12)
* compute ICCs
	scalar pi=2*acos(0)
	di (.5902032+.0568085+2.197066)/(.5902032+.0568085+2.197066+pi^2/3)
	di (.5902032+.0568085)/(.5902032+.0568085+2.197066+pi^2/3)
	di .5902032/(.5902032+.0568085+2.197066+pi^2/3)
* same model fit with gllamm
	gllamm diarr cistern water age5 smsize, fam(bin) link(logit) adapt i(fam comm mun) dots trace nip(12)
* fit a random effects probit using gllamm
	gllamm diarr cistern water age5 smsize, fam(bin) link(probit) adapt i(fam comm mun) dots trace nip(12)
* compute ICCs
	di (.17380681+.01655912+.69051845)/(.17380681+.01655912+.69051845+1)
	di (.17380681+.01655912) /(.17380681+.01655912+.69051845+1)
	di  .17380681 /(.17380681+.01655912+.69051845+1)

*************************************************************************
* Example 22.5 - Random effects count models for diarrhoea data
*************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate family-level diarrhoea count and auxiliary variables
	bysort fam: egen famcases=sum(diarr)
	bysort fam: gen within_f=_n
	rename water_treat_chlor water_tx
* 2-level Poisson with normally distributed random effects
	xtmepoisson famcases cistern water_tx || comm: if within_f==1, exp(fam_n) var intp(12)
	xtmepoisson famcases cistern water_tx || mun: if within_f==1, exp(fam_n) var intp(12)
* 3-level Poisson with normally distributed random effects
	xtmepoisson famcases cistern water_tx || mun: || comm: if within_f==1, exp(fam_n) var intp(12)
* 2-level Poisson with log-gamma distributed random effects
	xtpoisson famcases cistern water_tx if within_f==1, exp(fam_n) i(comm)
	xtpoisson famcases cistern water_tx if within_f==1, exp(fam_n) i(mun)
* 2-level negative binomial with beta distributed random effects
	xtnbreg famcases cistern water_tx if within_f==1, exp(fam_n) i(comm)
	xtnbreg famcases cistern water_tx if within_f==1, exp(fam_n) i(mun)
* negative binomial model with normal distributed random effects
	* not available in Stata, fit using SAS (proc nlmixed or glimmix)
* extra analyses with zero-inflated distributions (no random effects)
	zip famcases cistern water_tx if within_f==1, exp(fam_n) inflate(cistern water_tx)
	zinb famcases cistern water_tx if within_f==1, exp(fam_n) inflate(cistern water_tx)
	
**************************************************************************
* Example 22.6 - Random effects proportional odds model for diarrhoea data
**************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smplx, clear
* generate family-level diarrhoea outcome and auxiliary variables
	bysort fam: egen mean_di_cnt=mean(di_cnt)
	egen famsev=cut(mean_di_cnt), at(0,0.01,0.51,3) icodes
	bysort fam: gen within_f=_n
	rename water_treat_chlor water_tx
* random effects model fit using gllamm
	gllamm famsev cistern water_tx if within_f==1, i(comm mun) link(ologit) adapt nip(12)
* assessment of proportional odds assumption without random effects (not shown in text)
	ologit famsev cistern water_tx if within_f==1
	brant, detail
* comparison with logistic regressions (not shown in text)
	bysort fam: egen famdiarr=max(diarr)
	xtmelogit famdiarr cistern water_tx || mun: || comm: if within_f==1, var intp(12)
	gen famsev12=(famsev==2) & famsev~=.
	xtmelogit famsev12 cistern water_tx || mun: || comm: if within_f==1, var intp(12)

*************************************************************************
* Example 22.7 - Checking maximum-likelihood estimation of a GLMM
*************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate predictor variables
	capture drop age5
	gen age5=age<=5 if age~=.
	label var age5 "age<=5"
	capture drop smsize
	gen smsize=(m_pop_families-2000)/1000
	label var smsize "stand.m_pop_fam: (m_pop_families-2000)/1000"
* 4-level logistic regression with random effects
	xtmelogit diarr cistern water age5 smsize || mun: || comm: || fam:, var intp(1 1 1)
	xtmelogit diarr cistern water age5 smsize || mun: || comm: || fam:, var intp(3 3 3)
	xtmelogit diarr cistern water age5 smsize || mun: || comm: || fam:, var intp(7 7 7)
	xtmelogit diarr cistern water age5 smsize || mun: || comm: || fam:, var intp(12 12 12)
	xtmelogit diarr cistern water age5 smsize || mun: || comm: || fam:, var intp(20 20 20)

*************************************************************************
* Example 22.8 - Quasi-likelihood estimation of a GLMM
*************************************************************************
* all analyses carried out in MLwiN(no Stata equivalent)
	
*************************************************************************
* Example 22.9 - Statistical inference in a GLMM
*************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate predictor variables
	capture drop age5
	gen age5=age<=5 if age~=.
	label var age5 "age<=5"
	capture drop smsize
	gen smsize=(m_pop_families-2000)/1000
	label var smsize "stand.m_pop_fam: (m_pop_families-2000)/1000"
* refit full 4-level random effects logistic models
	xtmelogit diarr cistern water age5 smsize || mun: || comm: || fam:, var intp(12 12 12)
	estimates store full
* fit reduced 3-level random effects logistic models
	xtmelogit diarr cistern water age5 smsize || comm: || fam:, var intp(12 12)
	xtmelogit diarr cistern water age5 smsize || mun: || fam:, var intp(12 12)
	xtmelogit diarr cistern water age5 smsize || mun: || comm:, var intp(12 12)
* fit 4-level model without smsize
	xtmelogit diarr cistern water age5 || mun: || comm: || fam:, var intp(12 12 12)
	lrtest full, stats
	
****************************************************************************
* Example 22.10 - Prediction in a 2-level random-effects logistic regression
****************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate family-level diarrhoea outcome and auxiliary variables
	bysort fam: egen famdiarr=max(diarr)
	bysort fam: gen within_f=_n
* restrict data to one observation per family and to relevant variables
	keep if within_f==1
	keep mun comm fam famdiarr cistern
* simple logistic regression with community random effects
* code from Skrondal & Rabe-Hesketh (2009)
   	gllamm famdiarr cistern, fam(binom) link(logit) i(comm) trace adapt nip(12)
	gen zero1=0
	gllapred p1, mu us(zero) 
	gllapred p2, mu marg
	predict u, u
	gllapred p1u, mu us(um)
	gllapred p3, mu
* attempt to compute from xtmelogit
	capture drop p1 p2 p3 p1u
	xtmelogit famdiarr cistern || comm:, var intp(12)
	predict p1, mu fixedonly
	predict xb, xb 
	gen p2=invlogit(xb/sqrt(1+0.346*0.592))
	predict u, reffects
	gen p1u=invlogit(xb+u)
	predict p3, mu
* note that p3=p1u (which it should not be!)

*************************************************************************
* Example 22.11 - Residuals from a 3-level GLMM (xtmelogit)
* note: in this example, the residuals from xtmelogit seem flawed
*************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate predictor variables
	capture drop age5
	gen age5=age<=5 if age~=.
	label var age5 "age<=5"
	capture drop smsize
	gen smsize=(m_pop_families-2000)/1000
	label var smsize "stand.m_pop_fam: (m_pop_families-2000)/1000"
* reestimate model from Example 22.4
	xtmelogit diarr cistern water age5 smsize || mun: || comm: || fam:, var intp(12 12 12)
* individual-level raw residuals and predictions (fixed effects only)
	predict xb, xb
	predict ref_fam, ref level(fam)
	predict rese_fam, rese level(fam)
	gen stdres_fam=ref_fam/rese_fam
	predict ref_com, ref level(comm)
	predict rese_com, rese level(comm)
	gen stdres_com=ref_com/rese_com
	predict ref_mun, ref level(mun)
	predict rese_mun, rese level(mun)
	gen stdres_mun=ref_mun/rese_mun
* level counters (to enable suitable restrictions for plots etc)
	bysort mun: gen within_m=_n
	bysort mun comm: gen within_c=_n
	bysort mun comm fam: gen within_f=_n
* normal probability plot of family level residuals
	qnorm stdres_fam if within_f==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(family level std. residuals,m(r+2)) xtitle(,m(t+2))
* normal probability plot of community level residuals
	qnorm stdres_com if within_c==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(community level std. residuals,m(r+2)) xtitle(,m(t+2))	
* normal probability plot of herd level residuals
	qnorm stdres_mun if within_m==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(municipality level std. residuals,m(r+2)) xtitle(,m(t+2))
* compute family-level predicted values (linear predictor) based on fixed effects
	bysort fam: egen pred_fam=mean(xb)
* residual plot at family level
	scatter stdres_fam pred_fam if within_f==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(family level std. residuals,m(r+2)) xtitle(family level linear prediction,m(t+2)) 
* compute community-level predicted values (linear predictor) based on fixed effects
	bysort comm: egen pred_com=mean(xb)
* residual plot at community level
	scatter stdres_com pred_com if within_c==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(community level std. residuals,m(r+2)) xtitle(community level linear prediction,m(t+2)) 
* compute municipality-level predicted values (linear predictor) based on fixed effects
	bysort mun: egen pred_mun=mean(xb)
* residual plot at municipality level
	scatter stdres_mun pred_mun if within_m==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(municipality level std. residuals,m(r+2)) xtitle(municipality level linear prediction,m(t+2)) 
			
*************************************************************************
* Example 22.11 - Residuals from a 3-level GLMM (gllamm)
* note: generally speaking, residuals from gllamm are preferable
*************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate predictor variables
	capture drop age5
	gen age5=age<=5 if age~=.
	label var age5 "age<=5"
	capture drop smsize
	gen smsize=(m_pop_families-2000)/1000
	label var smsize "stand.m_pop_fam: (m_pop_families-2000)/1000"
* reanalyse model from Example 22.4 with gllamm to obtain diagnostic standardised residuals
	gllamm diarr cistern water age5 smsize, fam(bin) link(logit) adapt i(fam comm mun) dots trace
* standardised residuals at family, community and municipality levels
	gllapred res_, ustd
	drop res_s1 res_s2 res_s3
	gllapred xb, xb
* level counters (to enable suitable restrictions for plots etc)
	bysort mun: gen within_m=_n
	bysort mun comm: gen within_c=_n
	bysort mun comm fam: gen within_f=_n
*************************************************************************
* Fig 22.2 Normal probability plot of family level residuals
*************************************************************************
	qnorm res_m1 if within_f==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(family level std. residuals,m(r+2)) xtitle(inverse normal,m(t+2))
	graph save "figs/fig22_normfam.gph", replace
	graph export "figs/fig22_normfam.wmf", replace

* normal probability plot of community level residuals
	qnorm res_m2 if within_c==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(community level std. residuals,m(r+2)) xtitle(,m(t+2))
	graph save "figs/fig22_normcom.gph", replace
	graph export "figs/fig22_normcom.wmf", replace
*************************************************************************
* Fig 22.3 Normal and residual plots at municipality level
*************************************************************************
	qnorm res_m3 if within_m==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(municipality level std. residuals,m(r+2)) xtitle(,m(t+2))
	graph save "figs/fig22_normmun.gph", replace
	graph export "figs/fig22_normmun.wmf", replace

* compute family-level predicted values (linear predictor) based on fixed effects
	bysort fam: egen pred_fam=mean(xb)
* residual plot at family level
	scatter res_m1 pred_fam if within_f==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(family level std. residuals,m(r+2)) xtitle(family level linear prediction,m(t+2)) 
	graph save "figs/fig22_resfam.gph", replace
	graph export "figs/fig22_resfam.wmf", replace
* compute community-level predicted values (linear predictor) based on fixed effects
	bysort comm: egen pred_com=mean(xb)
* residual plot at community level
	scatter res_m2 pred_com if within_c==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(community level std. residuals,m(r+2)) xtitle(community level linear prediction,m(t+2)) 
	graph save "figs/fig22_rescom.gph", replace
	graph export "figs/fig22_rescom.wmf", replace
* compute municipality-level predicted values (linear predictor) based on fixed effects
	bysort mun: egen pred_mun=mean(xb)
* residual plot at municipality level
	scatter res_m3 pred_mun if within_m==1, scheme(s1mono) ylabel(,angle(horiz)) ///
			ytitle(municipality level std. residuals,m(r+2)) xtitle(municipality level linear prediction,m(t+2)) 
	graph save "figs/fig22_resmun.gph", replace
	graph export "figs/fig22_resmun.wmf", replace
* Fig 22.3 combine the two graphs
	graph combine figs/fig22_normmun.gph figs/fig22_resmun.gph , ysize(3) xsize(8) scheme(s1mono)
	graph save "figs/fig22_res_comb.gph", replace
	graph export "figs/fig22_res_comb.wmf", replace

*************************************************************************
* Example 22.12 - Analyses for family-level diarrhoea data
*************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate family-level diarrhoea outcome and auxiliary variables
	bysort fam: egen famdiarr=max(diarr)
	bysort fam: gen within_f=_n
* 3-level analysis with xtmelogit (not shown)
	xtmelogit famdiarr cistern || mun: || comm: if within_f==1, var intp(12)
* 2-level analysis with xtmelogit (municipality level)
	xtmelogit famdiarr cistern || mun: if within_f==1, var intp(12)
* 2-level analysis with xtmelogit (community level, same as Example 22.1)
	xtmelogit famdiarr cistern || mun: if within_f==1, var intp(12)
* comparison of approaches to account for clustering at municipality level
* ordinary logistic regression
	logit famdiarr cistern if within_f==1
* robust variance (clustered at municipality)
	logit famdiarr cistern if within_f==1, vce(cluster mun)
* fixed farm effects
	logit famdiarr cistern i.mun if within_f==1
* stratification by M-H method
	cs famdiarr cistern if within_f==1, by(mun) or
* GLMM with maximum likelihood estimation (same as above)
	xtmelogit famdiarr cistern || mun: if within_f==1, var intp(12)
* GEE with exchangeable correlation structure (municipality)
	xtgee famdiarr cistern if within_f==1, fam(bin) i(mun) robust 
* Bayesian analysis carried out in MLwiN
* burn-in: 10,000; sample: 100,000; standard uniform priors
