* mer23_all.do
* Repeated Measures Data - command for most examples and figures in Chapter 23

	clear all
	version 12
	set more off 
	cd c:\mer\ch23\data		/* this line is computer dependent */
	capture log close ch23_all
	log using ch23_all, name(ch23_all) replace 

*************************************************************************
* Example 23.1 Graphs and descriptive statistics for blood pressure data
*************************************************************************
* open the blood pressure dataset
	use bp, clear	
	capture drop dbp1c
		gen dbp1c=dbp1-102
		label var dbp1c "cent.dbp1: dbp1-102"
* construct wide format sub-dataset with complete series only (bpwide.dta)
	drop dbp1c cf
	reshape wide dbp, i(patient) j(visit)
	order centre patient tx dbp1 dbp3 dbp4 dbp5 dbp6
	keep if dbp1~=. & dbp3~=. & dbp4~=. & dbp5~=. & dbp6~=.
	save bpwide, replace
* table of means
	bysort tx: sum dbp1-dbp6
* correlations and covariances
	corr dbp1-dbp6
	corr dbp1-dbp6, cov
*************************************************************************
* Fig 23.1 - Profile and Mean plots
*************************************************************************
* profile plot
	reshape long dbp, i(patient) j(visit)
	twoway (connected dbp visit if patient==80, msymbol(none) lcolor(black) lpattern(longdash)) ///
      (connected dbp visit if patient==81, msymbol(none) lcolor(black) lpattern(shortdash)) ///
      (connected dbp visit if patient==274, msymbol(none) lcolor(black) lpattern(solid)) ///
      (connected dbp visit if patient==84, msymbol(none) lcolor(black) lpattern(shortdash)) ///
      (connected dbp visit if patient==181, msymbol(none) lcolor(black) lpattern(longdash)) ///
      (connected dbp visit if patient==182, msymbol(none) lcolor(black) lpattern(shortdash)) /// 
      (connected dbp visit if patient==183, msymbol(none) lcolor(black) lpattern(solid)) ///
      (connected dbp visit if patient==185, msymbol(none) lcolor(black) lpattern(shortdash)) ///
      (connected dbp visit if patient==186, msymbol(none) lcolor(black) lpattern(longdash))  ///
      (connected dbp visit if patient==187, msymbol(none) lcolor(black) lpattern(solid)) /// 
      (connected dbp visit if patient==189, msymbol(none) lcolor(black) lpattern(longdash)) ///
      (connected dbp visit if patient==190, msymbol(none) lcolor(black) lpattern(shortdash)) ///
      (connected dbp visit if patient==192, msymbol(none) lcolor(black) lpattern(longdash)) ///
      (connected dbp visit if patient==262, msymbol(none) lcolor(black) lpattern(solid)) /// 
      (connected dbp visit if patient==263, msymbol(none) lcolor(black) lpattern(solid)), ///
	  ytitle(diastolic blood pressure, m(r+2)) xtitle(visit number, m(t+2)) ///
	  plotregion(style(none)) legend(off) ylabel(, angle(horizontal) format(%4.0f)) ///
	  scheme(s1mono)
	graph save "figs/fig23_profplot.gph", replace
	graph export "figs/fig23_profplot.emf", replace
	* note - the above command assumes that there is a "figs" directory under your working directory
* mean plot for tx
	bysort tx visit: gen within=_n
	bysort tx visit: egen meandbp=mean(dbp)
	twoway (connected meandbp visit if within==1 & tx==1, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected meandbp visit if within==1 & tx==2, msymbol(none) lcolor(black) lpattern(longdash)) ///
	  (connected meandbp visit if within==1 & tx==3, msymbol(none) lcolor(black) lpattern(shortdash)), ///
	  ytitle(mean diastolic blood pressure, m(r+2)) xtitle(visit number, m(t+2)) legend(off) ///
	  plotregion(style(none)) ylabel(, angle(horizontal) format(%4.0f)) ///
	  scheme(s1mono)
	graph save "figs/fig23_meanplot.gph", replace
	graph export "figs/fig23_meanplot.emf", replace
* combine the two graphs
	graph combine "figs/fig23_profplot.gph" "figs/fig23_meanplot.gph" , ysize(4) xsize(8) scheme(s1mono)
	graph save "figs/fig23_1_comb.gph", replace
	graph export "figs/fig23_1_comb.emf", replace


*************************************************************************
* Example 23.2 Incomplete series in blood pressure data
*************************************************************************
* open the blood pressure dataset
	use bp, clear
* tabulate incomplete series profiles
	reshape wide dbp, i(patient) j(visit)
	gen profile=1000*(dbp3~=.)+100*(dbp4~=.)+10*(dbp5~=.)+(dbp6~=.)
	tab profile tx
*************************************************************************
* Fig 23.2 - Profile plot with incomplete profiles
*************************************************************************
* profile plot
	reshape long dbp, i(patient) j(visit)
	twoway (connected dbp visit if patient==12, msymbol(none) lcolor(black) lpattern(longdash)) ///
      (connected dbp visit if patient==22, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected dbp visit if patient==59, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected dbp visit if patient==69, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected dbp visit if patient==79, msymbol(none) lcolor(black) lpattern(solid)) ///
      (connected dbp visit if patient==91, msymbol(none) lcolor(black) lpattern(longdash)) ///
	  (connected dbp visit if patient==121, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected dbp visit if patient==148, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected dbp visit if patient==151, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected dbp visit if patient==164, msymbol(none) lcolor(black) lpattern(solid)) ///
      (connected dbp visit if patient==209, msymbol(none) lcolor(black) lpattern(longdash))  ///
      (connected dbp visit if patient==211, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected dbp visit if patient==215, msymbol(none) lcolor(black) lpattern(longdash)) ///
	  (connected dbp visit if patient==235, msymbol(none) lcolor(black) lpattern(solid)) ///
      (connected dbp visit if patient==236, msymbol(none) lcolor(black) lpattern(solid)) ///
	  (connected dbp visit if patient==238, msymbol(none) lcolor(black) lpattern(longdash)) ///
	  (connected dbp visit if patient==254, msymbol(none) lcolor(black) lpattern(shortdash)) /// 
	  (connected dbp visit if patient==289, msymbol(none) lcolor(black) lpattern(shortdash)) ///
	  (connected dbp visit if patient==318, msymbol(none) lcolor(black) lpattern(longdash)) ///
	  (connected dbp visit if patient==321, msymbol(none) lcolor(black) lpattern(shortdash)), ///
	  ytitle(diastolic blood pressure, m(r+2)) xtitle(visit number, m(t+2)) ///
	  plotregion(style(none)) legend(off) ylabel(, angle(horizontal) format(%4.0f)) ///
	  scheme(s1mono)
	graph save "figs/fig23_incprofplot.gph", replace
	graph export "figs/fig23_incprofplot.emf", replace
	  
      
*************************************************************************
* Example 23.3 Univariate methods for blood pressure data
*************************************************************************
* open wide format blood pressure sub-dataset
	use bpwide, clear
	gen dbp1c=dbp1-102
* analysis at separate time points
	anova dbp1 tx
	anova dbp3 tx c.dbp1c
	regress /* to see parameter estimates */
	anova dbp4 tx c.dbp1c
	regress
	anova dbp5 tx c.dbp1c
	regress
	anova dbp6 tx c.dbp1c
	regress
* summary statistic: initial gain
	gen gain13=dbp3-dbp1
	anova gain13 tx 
	regress
* summary statistic: slope for visits 3-6
	gen runid=_n
	reshape long dbp, i(patient) j(visit)
	gen slope=.
	* loop through the patients and compute slope for visits 3-6
	set more off
	forvalues xx=1/256 {
		* di `xx'
		quietly reg dbp visit if runid==`xx' & visit>=3
		replace slope=_b[visit] if runid==`xx'
		}
	* identify 1 observation per fish
	gen rec1=.
	bysort runid: replace rec1=(_n==1)
	* ANOVA of slopes
	anova slope i.tx if rec1==1
	regress

	
*************************************************************************
* Example 23.4 ANOVA and MANOVA analysis of blood pressure data
*************************************************************************
* open wide format blood pressure sub-dataset
	use bpwide, clear
	gen dbp1c=dbp1-102
* MANOVA (all visits)
	manova dbp1 dbp3 dbp4 dbp5 dbp6 = tx
	* note: calculation of tests involving time carried out in SAS, proc glm
	* calculation of estimated variance-covariance matrix
	matrix define e = e(E)
	di e[1,1]/253
	di e[2,2]/253
	di e[3,3]/253
	di e[4,4]/253
	di e[5,5]/253
	di e[1,2]/sqrt(e[1,1]*e[2,2])
	di e[1,3]/sqrt(e[1,1]*e[3,3])
	di e[1,4]/sqrt(e[1,1]*e[4,4])
	di e[1,5]/sqrt(e[1,1]*e[5,5])
	di e[2,3]/sqrt(e[2,2]*e[3,3])
	di e[2,4]/sqrt(e[2,2]*e[4,4])
	di e[2,5]/sqrt(e[2,2]*e[5,5])
	di e[3,4]/sqrt(e[3,3]*e[4,4])
	di e[3,5]/sqrt(e[3,3]*e[5,5])
	di e[4,5]/sqrt(e[4,4]*e[5,5])
* MANOVA (visits 3-6)
	manova dbp3 dbp4 dbp5 dbp6 = tx
 	* note: calculation of tests involving time carried out in SAS, proc glm
	* calculation of estimated variance-covariance matrix
	matrix define e = e(E)
	di e[1,1]/253
	di e[2,2]/253
	di e[3,3]/253
	di e[4,4]/253
	di e[1,2]/sqrt(e[1,1]*e[2,2])
	di e[1,3]/sqrt(e[1,1]*e[3,3])
	di e[1,4]/sqrt(e[1,1]*e[4,4])
	di e[2,3]/sqrt(e[2,2]*e[3,3])
	di e[2,4]/sqrt(e[2,2]*e[4,4])
	di e[3,4]/sqrt(e[3,3]*e[4,4])

* convert to long format for remaining analyses
	reshape long dbp, i(patient) j(visit)
* random intercept model analysed by ANOVA method
	set matsize 800
	anova dbp tx / patient|tx visit visit#tx
* random intercept model analysed by maximum likelihood method
	xtmixed dbp tx##visit || patient:, var reml
	testparm tx#visit /* reference chi-sq distribution instead of F distribution */
	* tests for tx and visit not available from interaction model
* repeated measures ANOVA
	anova dbp tx / patient|tx visit visit#tx, repeated(visit)
* mixed model with unstructured covariance
	xtmixed dbp tx##visit || patient:, noconstant residuals(uns, t(visit)) var reml
	testparm tx#visit /* reference chi-sq distribution instead of F distribution */

* rerun above steps for visits 3-6
* random intercept model analysed by ANOVA method
	set matsize 800
	anova dbp tx / patient|tx visit visit#tx if visit>1
* random intercept model analysed by maximum likelihood method
	xtmixed dbp tx##visit || patient: if visit>1, var reml
	testparm tx#visit /* reference chi-sq distribution instead of F distribution */
	* tests for tx and visit not available from interaction model
* repeated measures ANOVA
	anova dbp tx / patient|tx visit visit#tx if visit>1, repeated(visit)
* mixed model with unstructured covariance
	xtmixed dbp tx##visit || patient: if visit>1, noconstant residuals(uns, t(visit)) var reml
	testparm tx#visit /* reference chi-sq distribution instead of F distribution */

	
*************************************************************************************
* Example 23.5 Linear mixed models with correlation structure for blood pressure data
*************************************************************************************
* open the full blood pressure dataset
	use bp, clear
	gen week=2*(visit-2)+4*(visit==6) /* week post-treatment */
* 3-level random intercept models with different fixed effects (not shown)
	xtmixed dbp tx##visit dbp1c || centre: || patient:, var reml
	xtmixed dbp tx##visit##c.dbp1c || centre: || patient:, var reml
	xtmixed dbp tx##visit##c.dbp1c##c.dbp1c || centre: || patient:, var reml
* 3-level random intercept models with additive fixed effects
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml
* correlation structure: ar(1)
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml noconstant residuals(ar 1, t(visit))
* error correlation structure: ar(1)
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml residuals(ar 1, t(visit))
* correlation structure: arma(1,1) - fit using SAS proc mixed
* correlation structure: toeplitz
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml noconstant residuals(toep, t(visit))
* correlation structure: unstructured (not shown)
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml noconstant residuals(un, t(visit))
* correlation structure: exponential = reparametrization of ar(1) allowing for non-equidistant times
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml noconstant residuals(expo, t(week))
* error correlation structure: ar(2) (not shown)
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml residuals(ar 2, t(visit))
* error correlation structure: ma(1) (not shown)
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml residuals(ma 1, t(visit))

	
*************************************************************************
* Example 23.6 Trend model for blood pressure data
*************************************************************************
* open the full blood pressure dataset
	use bp, clear
	gen visitc3=visit-3
* random slope model with linear effect of visit at patient level only
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient: visitc3, cov(uns) var reml
* random slope model with linear effect of visit at patient and centre levels (not shown)
	xtmixed dbp i.tx i.visit dbp1c || centre: visitc3, cov(uns) || patient: visitc3, cov(uns) var reml
* random slope model with ar(1) errors and linear effect of visit at patient level only (not shown)
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient: visit, cov(uns) var reml residuals(ar 1, t(visit))
* random slope model with ar(1) errors and linear effect of visit at centre level only (not shown)
	xtmixed dbp i.tx i.visit dbp1c || centre: visitc3, cov(uns) || patient:, var reml residuals(ar 1, t(visit))
* random slope model with ar(1) errors and linear effect of visit at patient and centre levels
	xtmixed dbp i.tx i.visit dbp1c || centre: visitc3, cov(uns) || patient: visitc3, cov(uns) var reml residuals(ar 1, t(visit))

	
*************************************************************************
* Example 23.7 Generalised linear mixed models for high blood pressure data
*************************************************************************
* open the full blood pressure dataset
	use bp, clear
* generate binary outcome
	gen highdbp= dbp>=90
	replace highdbp=. if dbp==.
* descriptive statistics for highscc
	table highdbp
	summarize highdbp
	bysort patient: egen within_pat_mean=mean(highdbp)
	bysort patient: egen within_pat_sd=sd(highdbp)
	bysort patient: gen within_n=_n
	table within_pat_mean within_pat_sd if within_n==1, row col
* random intercept model
	xtmelogit highdbp i.tx i.visit dbp1c || centre: || patient:, var
* transitional model
	gen laghighdbp=highdbp[_n-1]
	gen lagvisit=visit[_n-1]
	replace laghighdbp=. if within_n==1 & lagvisit~=visit-1
	xtmelogit highdbp i.tx i.visit dbp1c laghighdbp || centre: || patient:, var
* rerun random intercept model on reduced dataset to see if that affects the estimates	
	xtmelogit highdbp i.tx i.visit dbp1c || centre: || patient: if laghighdbp~=., var
* trend model
	gen visitc3=visit-3
	xtmelogit highdbp i.tx i.visit dbp1c || centre: || patient: visitc3, cov(uns) var


*************************************************************************
* Example 23.8 GEE analyses for diarrhoea data
*************************************************************************
* open the Brazilian diarrhoea dataset dataset
	use brazil_smpl, clear
* generate predictor variables
	capture drop age5
	gen age5=age<=5 if age~=.
	label var age5 "age<=5"
	capture drop smsize
	gen smsize=(m_pop_families-2000)/1000
	label var smsize "stand.m_pop_fam: (m_pop_families-2000)/1000"
* family-level model
* generate family-level diarrhoea outcome and auxiliary variables
	bysort fam: egen famdiarr=max(diarr)
	bysort fam: gen within_f=_n
* GEE with exchangeable correlation structure (municipality)
	xtgee famdiarr cistern if within_f==1, fam(bin) i(mun) robust 
	xtcorr, compact
* GEE with exchangeable correlation structure (community)
	xtgee famdiarr cistern if within_f==1, fam(bin) i(comm) robust 
	xtcorr, compact
* random effects models with robust standard errors using gllamm
* municipality random effects
	gllamm famdiarr cistern if within_f==1, fam(bin) link(logit) i(mun) adapt robust
* community random effects
	gllamm famdiarr cistern if within_f==1, fam(bin) link(logit) i(comm) adapt robust
* ALR analysis run in SAS proc genmod

	
*************************************************************************
* Example 23.9 GEE for continuous outcome in blood pressure data
*************************************************************************
* open the full blood pressure dataset
	use bp, clear
* declare panel data structure for GEE
	xtset patient visit
* default GEE estimations (not shown, due to variable number of observations)	
* GEE estimation with compound symmetry working correlation structure
	xtgee dbp i.tx i.visit dbp1c, fam(gaus) link(ident) corr(exch) robust
* GEE estimation with ar(1) working correlation structure
	xtgee dbp i.tx i.visit dbp1c, fam(gaus) link(ident) corr(ar 1) robust
* GEE estimation with stationary working correlation structure
	xtgee dbp i.tx i.visit dbp1c, fam(gaus) link(ident) corr(sta 3) robust
* GEE estimation with unstructured working correlation structure
	xtgee dbp i.tx i.visit dbp1c, fam(gaus) link(ident) corr(uns) robust
* GEE estimation with compound symmetry working correlation structure at centre level
	xtgee dbp i.tx i.visit dbp1c, fam(gaus) link(ident) i(centre) corr(exch) robust
	
* exclusion of patients with one observation only
	bysort patient: egen nobs=count(dbp)
	xtset patient visit
* GEE estimation and QIC with compound symmetry working correlation structure
	xtgee dbp i.tx i.visit dbp1c if nobs>1, fam(gaus) link(ident) corr(exch) robust
	xi: qic dbp i.tx i.visit dbp1c if nobs>1, fam(gaus) link(ident) corr(exch) robust
	xtcorr, compact
* GEE estimation with ar(1) working correlation structure - note force option
	xi: qic dbp i.tx i.visit dbp1c if nobs>1, fam(gaus) link(ident) corr(ar 1) robust force
	xtcorr, compact
* GEE estimation with unstructured working correlation structure - note force option
	xi: qic dbp i.tx i.visit dbp1c if nobs>1, fam(gaus) link(ident) corr(uns) robust force
	xtcorr, compact
* GEE estimation with compound symmetry working correlation structure at centre level and QIC
	xi: qic dbp i.tx i.visit dbp1c if nobs>1, fam(gaus) link(ident) i(centre) corr(exch) robust
	xtcorr, compact

	
*************************************************************************
* Example 23.10 GEE and ALR estimation for high blood pressure data
*************************************************************************
* open the full blood pressure dataset
	use bp, clear
* generate binary outcome
	gen highdbp= dbp>=90
	replace highdbp=. if dbp==.
* exclusion of patients with one observation only
	bysort patient: egen nobs=count(dbp)
* ordinary logistic regression
	logit highdbp i.tx i.visit dbp1c if nobs>1
* GEE estimation with within-patient compound symmetry correlation structure (not shown)
	xtset patient visit
	xtgee highdbp i.tx i.visit dbp1c if nobs>1, fam(bin) link(logit) corr(exch) robust
	xtcorr, compact
* GEE estimation with within-patient autoregressive correlation structure
	xtgee highdbp i.tx i.visit dbp1c if nobs>1, fam(bin) link(logit) corr(ar1) robust force
	xtcorr, compact
* GEE estimation with within-centre compound symmetry correlation structure
	xtgee highdbp i.tx i.visit dbp1c if nobs>1, fam(bin) link(logit) i(centre) corr(exch) robust
	xtcorr, compact
* ALR analysis run in SAS proc genmod
