* mer24_all.do
* Bayesian Analysis - most analyses in this chapter were acrried out using MLwiN

	clear all
	version 12
	set more off 
	cd c:\ver2\ch23\data		/* this line is computer dependent */
	capture log close ch24_all
	log using ch24_all, name(ch24_all) replace 


*************************************************************************
* Example 24.1 - Bayesian analysis of proportions
   * Fig 24.1 - Posterior distributions
*************************************************************************
	clear
	set obs 999
		gen p=_n/1000
		label var p probability
		gen beta66=(p^5)*((1-p)^5)*exp(lngamma(12)-lngamma(6)-lngamma(6))
		gen beta93=(p^8)*((1-p)^2)*exp(lngamma(12)-lngamma(9)-lngamma(3))
		label var beta66 "5 individuals"
		label var beta93 "8 individuals"
	twoway (line beta66 p, lcolor(black) lpattern(solid)) (line beta93 p, lcolor(black) lpattern(dash)) ///
		, ytitle(Posterior density) ylabel(0 1 2 3) xtitle(Proportion p) xlabel(0 0.2 0.4 0.6 0.8 1.0) scheme(s1mono) ///
		legend(on position(11) ring(0))
	graph save "figs/fig24_postplot.gph", replace
	graph export "figs/fig24_postplot.emf", replace

	* Frequentist estimates and exact binomial confidence intervals
		cii 10 5
		cii 10 8

*************************************************************************
* Fig 24.2 Convergence of a Markov chain
*************************************************************************
	clear
	set obs 21
		scalar p01=0.8
		scalar p11=0.3
		gen p=0
		replace p=p01*(1-p[_n-1])+p11*p[_n-1] in 2/21
		gen t=_n-1
	twoway (connected p t, lcolor(black)), ytitle("probability of state 1", m(r+2)) ///
		ylabel(0(0.2)1, angle(horizontal) format(%4.1f)) xtitle("time", m(t+2)) ///
		xlabel(0(5)20, format(%4.0f)) plotregion(style(none)) scheme(s1mono)
	graph save "figs/fig24_converge.gph", replace
	graph export "figs/fig24_converge.emf", replace


*************************************************************************************
* Example 24.5 Linear mixed model  for blood pressure data
*************************************************************************************
* open the full blood pressure dataset
	use bp, clear
* 3-level random intercept models with additive fixed effects
	xtmixed dbp i.tx i.visit dbp1c || centre: || patient:, var reml
* all other analyses carried out in MLwiN
