
* ch25_all.do
* Visualization of spatial data
	* Note: some of the analyses in this Chapter were perfored using a variety of other software programs

	clear all
	version 12
	set more off 
	cd c:\mer\ch25\data		/* this line is computer dependent */
	capture log close ch25_all
	log using ch25_all, name(ch25_all) replace 

*************************************************************************
* Ex 25.1 Point map - cases vs no cases
*************************************************************************
use "district-d.dta", clear
spmap using "district-c.dta", id(id) legenda(on) leglabel(Districts) ///
    point(data("commune-d.dta") xcoord(x_coord) ycoord(y_coord) psize(relative) ///
    shape(s d) fcolor(gray black) ocolor(white) size(*.65) legenda(on) by(infected) ///
    legtitle(Infection status) legcount) legend(size(*1.2) region(ls(solid) fc(white))) ///
    plotregion(margin(medium) color(gs16)) graphregion(fcolor(gs16) lcolor(black))  ///
    xlab(, grid) xtitle(Easting (km)) ytitle(Northing (km)) freestyle ///
    scalebar(units(100) label(km)) ylabel(, angle(horizontal)) scheme(s1mono)
graph save "figs/fig25_point_all", replace
graph export "figs/fig25_point_all.wmf", replace

*************************************************************************
* Ex 25.1 (cont.) Point maps - cases vs no cases by year
*************************************************************************
use "district-d.dta", clear
* 2003 - 2004
spmap using "district-c.dta", id(id) oc(gs8) ///
    polygon(data(prov-c.dta) os(medium) legenda(on) leglabel(Provinces))   ///
    point(data("commune-d.dta") xcoord(x_coord) ycoord(y_coord) psize(relative) ///
     shape(D) fcolor(gs5) ocolor(black) size(*.5) legenda(on) by(inf_2003_4) ///
     legtitle(Infection 2003/04) legcount select(keep if inf_2003_4==1)) legenda(on) ///
     leglabel(Districts) legend(size(*.8) region(ls(solid) fc(white))) ///
     plotregion(margin(medium) color(gs16)) graphregion(fcolor(gs16) lcolor(black)) ///
     xlab(, grid) freestyle xtitle(Easting (km)) ytitle(Northing (km)) ///
    scalebar(units(100) label(km)) ylabel(, angle(horizontal)) scheme(s1mono)
graph save "figs/fig25_point_2003", replace
graph export "figs/fig25_point_2003.wmf", replace

* 2004 - 2005
spmap using "district-c.dta", id(id) oc(gs8) ///
    polygon(data(prov-c.dta) os(medium) legenda(on) leglabel(Provinces))   ///
    point(data("commune-d.dta") xcoord(x_coord) ycoord(y_coord) psize(relative) ///
     shape(D) fcolor(gs5) ocolor(black) size(*.5) legenda(on) by(inf_2004_5) ///
     legtitle(Infection 2004/05) legcount select(keep if inf_2004_5==1)) ///
     legenda(on) leglabel(Districts) legend(size(*.8) region(ls(solid) fc(white))) ///
     plotregion(margin(medium) color(gs16)) graphregion(fcolor(gs16) lcolor(black)) ///
     xlab(, grid) freestyle xtitle(Easting (km)) ytitle(Northing (km)) ///
    scalebar(units(100) label(km)) ylabel(, angle(horizontal)) scheme(s1mono)
graph save "figs/fig25_point_2004", replace
graph export "figs/fig25_point_2004.wmf", replace

* 2005 - 2006
spmap using "district-c.dta", id(id) oc(gs8) ///
    polygon(data(prov-c.dta) os(medium) legenda(on) leglabel(Provinces))   ///
    point(data("commune-d.dta") xcoord(x_coord) ycoord(y_coord) psize(relative) ///
     shape(D) fcolor(gs5) ocolor(black) size(*.5) legenda(on) by(inf_2005_6) ///
     legtitle(Infection 2005/06) legcount select(keep if inf_2005_6==1)) ///
     legenda(on) leglabel(Districts) legend(size(*.8) region(ls(solid) fc(white))) ///
     plotregion(margin(medium) color(gs16)) graphregion(fcolor(gs16) lcolor(black)) ///
     xlab(, grid) freestyle xtitle(Easting (km)) ytitle(Northing (km)) ///
    scalebar(units(100) label(km)) ylabel(, angle(horizontal)) scheme(s1mono)
graph save "figs/fig25_point_2005", replace
graph export "figs/fig25_point_2005.wmf", replace

* combine the 4 graphs
    graph combine "figs/fig25_point_all" "figs/fig25_point_2003" ///
        "figs/fig25_point_2004" "figs/fig25_point_2005" , ///
        imargin(r+5) scheme(s1mono)
    graph save "figs/fig25_point_comb", replace
    graph export "figs/fig25_point_comb.wmf", replace

*************************************************************************
* Ex 25.2 Proportional dot maps
*************************************************************************
use district-d.dta, clear
*chicken density
spmap using "district-c.dta", id(id) fcolor(gs16) ocolor(gs10)   legstyle(1)              ///
    point(data("district-d.dta") xcoord(x_coord) ycoord(y_coord) psize(relative)  ///
    proportional(chickdens) ocolor(black) fcolor(Greys2) legenda(on) by(chickdens)   ///
    legshow(1 10 25 50 89) size(*1)) ///
    legend(position(2) cols(1) title(Birds per km. square, size(vsmall)) size(*1)) ///
    polygon(data(region-c.dta) ocolor(gs8))
graph save "figs/fig25_dots_chick", replace
graph export "figs/fig25_dots_chick.wmf", replace

* duck density
spmap using "district-c.dta", id(id) fcolor(gs16) ocolor(gs10)                 ///
    point(data("district-d.dta") xcoord(x_coord) ycoord(y_coord) ps(relative)  ///
    proportional( duckdens) ocolor(black) fcolor(Greys2) legenda(on) by(duckdens)   ///
    legshow(1 10 25 40 61) size(*1)) polygon(data(region-c.dta) ocolor(gs8)) ///
    legend(position(2) cols(1) title(Birds per km. square, size(vsmall)) size(*1))
graph save "figs/fig25_dots_duck", replace
graph export "figs/fig25_dots_duck.wmf", replace

* human density
spmap using "district-c.dta", id(id) fcolor(gs16) ocolor(gs10)                 ///
    point(data("district-d.dta") xcoord(x_coord) ycoord(y_coord) ps(relative)  ///
    proportional(humandens) ocolor(black) fcolor(Greys2) legenda(on) by(humandens)   ///
    legshow(1 25 50 75 89) size(*1)) polygon(data(region-c.dta) ocolor(gs8)) ///
    legend(position(2) cols(1) title(Population per km. square, size(vsmall)) size(*1))

graph save "figs/fig25_dots_human", replace
graph export "figs/fig25_dots_human.wmf", replace

* combine the 3 graphs
    graph combine "figs/fig25_dots_chick" "figs/fig25_dots_duck" ///
        "figs/fig25_dots_human" , imargin(r+5) scheme(s1mono)
    graph save "figs/fig25_dots_comb", replace
    graph export "figs/fig25_dots_comb.wmf", replace


*************************************************************************
* Ex. 25.3 Choropleth prevalence maps
*************************************************************************
use "district-d.dta", clear
* equal interval
spmap prev_all using "district-c.dta", id(id) clm(eqint) cln(4) fcolor(Greys2)   ///
        ocolor(white ..) osize(medthin ..) legcount legstyle(2) legend(ring(0)         ///
        position(7))   legend(size(*.6) region(ls(solid) fc(white)))         ///
        plotregion(margin(medium) color(gs16)) graphregion(fcolor(gs16)       ///
        lcolor(black))  polygon(data("prov-c.dta") os(medium))              ///
        xlab(, grid) freestyle xtitle(Easting (km)) ytitle(Northing (km)) ///
        scalebar(units(100) label(km)) ///
        ylabel(, angle(horizontal)) scheme(s1mono)
graph save "figs/fig25_prev_equal", replace
graph export "figs/fig25_prev_equal.wmf", replace

*quantile method
spmap prev_all using "district-c.dta", id(id) clm(quantile) fcolor(Greys2)   ///
        ocolor(white ..) osize(medthin ..) legcount legstyle(2) legend(ring(0)       ///
        position(7))   legend(size(*.6) region(ls(solid) fc(white)))         ///
        plotregion(margin(medium) color(gs16)) graphregion(fcolor(gs16)       ///
        lcolor(black)) polygon(data("prov-c.dta") os(medium))      ///
        xlab(, grid) freestyle xtitle(Easting (km)) ytitle(Northing (km)) ///
        scalebar(units(100) label(km))  ///
    ylabel(, angle(horizontal)) scheme(s1mono)
graph save "figs/fig25_prev_quant", replace
graph export "figs/fig25_prev_quant.wmf", replace

* kmeans
* kmeans - remember if spmap is installed the kmeans options in the ado file need to be
* modified where is says "_cl_1_" replace by "_clus_1_"
spmap prev_all using "district-c.dta", id(id) clm(kmeans)  fcolor(Greys2)   ///
        ocolor(white ..) osize(medthin ..) legcount legstyle(2) legend(ring(0)         ///
        position(7))   legend(size(*.6) region(ls(solid) fc(white)) )         ///
        plotregion(margin(medium) color(gs16)) graphregion(fcolor(gs16)       ///
        lcolor(black)) polygon(data("prov-c.dta") os(medium))                 ///
        xlab(, grid) freestyle xtitle(Easting (km)) ytitle(Northing (km)) ///
        scalebar(units(100) label(km))  ///
        ylabel(, angle(horizontal)) scheme(s1mono) ///
        legend(label(2 "0 - 14 (61)"))
graph save "figs/fig25_prev_kmean", replace
graph export "figs/fig25_prev_kmean.wmf", replace

* combine the 3 graphs
    graph combine "figs/fig25_prev_equal" "figs/fig25_prev_quant" ///
        "figs/fig25_prev_kmean", imargin(r+5) scheme(s1mono)
    graph save "figs/fig25_prev_comb", replace
    graph export "figs/fig25_prev_comb.wmf", replace


*************************************************************************
* Ex 25.4 cartogram choropleth prevalence maps
*************************************************************************
*k-means choropleth map
use "district-d.dta", clear
spmap prev_all using "district-c.dta", id(id) clm(kmeans) fcolor(Greys2)   ///
        ocolor(white ..) osize(medthin ..) legcount legstyle(2) legend(ring(0)         ///
        position(7))   legend(size(*1.2) region(ls(solid) fc(white)) )         ///
        plotregion(margin(large) color(gs16)) graphregion(fcolor(gs16)) ///
         polygon(data("prov-c.dta") os(medium)) ///
        legend(label(2 "0 - 14 (61)")) title("Choropleth map")
graph save "figs/fig25_chlor_kmean", replace
graph export "figs/fig25_chlor_kmean.wmf", replace

*cartogram using shape file created in scape toad
use "district-d.dta", clear
spmap prev_all using "district_cart-c.dta", id(id) clm(kmeans) fcolor(Greys2)   ///
        ocolor(white ..) osize(medthin ...) legcount legstyle(2) legend(ring(0)         ///
        position(7))   legend(size(*1.2) region(ls(solid) fc(white)) )         ///
        plotregion(margin(medium) color(gs16) ilwidth(thick)) graphregion(fcolor(gs16))       ///
        polygon(data("district_cart-c.dta") os(medium)) ///
        legend(label(2 "0 - 14 (61)")) title("Cartogram")
graph save "figs/fig25_chlor_shape", replace
graph export "figs/fig25_chlor_shape.wmf", replace

* combine the 2 graphs
    graph combine "figs/fig25_chlor_kmean" "figs/fig25_chlor_shape" ///
        , imargin(r+5) scheme(s1mono) ysize(3) xsize(8)
    graph save "figs/fig25_chlor_comb", replace
    graph export "figs/fig25_chlor_comb.wmf", replace

*************************************************************************
*Fig 25.5 PEI elevation - proportional dot map
*************************************************************************
use pei-d, clear
spmap using "pei-c.dta", id(id) oc(gs8)  legstyle(0)   ///
   point(data("peipts.dta") xcoord(point_x) ycoord(point_y) ps(relative)  ///
    proportional(z) size(*.8) ocolor(black) fcolor(Greys2) legenda(on) by(z_cat)  ///
    legshow(1 2 3 4 5)) ///
    legend(position(2) size(*1) cols(1) title("Meters above sea level", size(small)))
graph save "figs/fig25_pei_dots", replace
graph export "figs/fig25_pei_dots.wmf", replace
