* mer28_all.do
* Meta analysis - all calculations
*
	version 12
	clear all
	set more off 
	cd c:\mer\ch28\data		/* this line is computer dependent */
	capture log close 
	log using mer28_all, replace text

*************************************************************************
* Disclaimers
*************************************************************************
/* 
The analyses shown in this program are carried out for providing examples for the text. 
	They do not, in any way, represent a complete or appropriate analysis of the dataset
For results of a more complete analysis, see:
	Bucher O, et al, Prev Vet Med. 2012;103(1):1-15.
	
Many of the commands used in this file are Stata "add-on" programs which need to be
installed prior to use.
	(including: meta, metan, metareg, metabias, metafunnel, metatrim)
*/

*************************************************************************
* Open the data 
*************************************************************************

use "C:\mer\data\salm_ma.dta", clear
	/* file is limited to non-lab (pilot and commercial) studies 
		- for continuous outcomes, use all records
		- for RR studies, limit to chilling interventions */
	sort refid id 
	br refid id md_ln if !missing(md_ln)
	br refid id rr_ln if !missing(rr_ln)  & intrv==5

*************************************************************************
*Ex 28.3 - 28.4  Fixed and random effects meta-analyses, forest plot
*************************************************************************
	* Salmonella concentration
		metan  md_ln mdse_ln, fixed nograph label(namevar=id)
		metan  md_ln mdse_ln, random nograph label(namevar=id)
	* Salmonella isolation
		metan rr_ln rrse_ln if intrv==5, fixed nograph label(namevar=id)
		metan rr_ln rrse_ln if intrv==5, random nograph label(namevar=id)
* Forest plot - Salmonella concentration
	metan md_ln mdse_ln, random  lcols(author id) xlabel(-6, -5, -4, -3, -2, -1, 0, 1) force ///
		scheme(s1mono) xtitle("Mean difference in (log)Salmonella counts", size(small)) nohet
		graph save "figs/fig28_forrest.gph", replace
		graph export "figs/fig28_forrest.emf", replace


*************************************************************************
* Ex 28.5 Stratified analysis
*************************************************************************
	metan  md_ln mdse_ln, random nograph
	metan  md_ln mdse_ln, random by(serotype) nograph

*************************************************************************
* Section 28.7.3 Heterogeneity - Graphical assessment and meta-regression
*************************************************************************
* Fig 28.3 Galbraith Plot
	metan  md_ln mdse_ln, nograph
		scalar es=r(ES)
		capture drop y x l1 l2 l3
		gen y=md_ln/mdse_ln
		gen x=1/mdse_ln
		gen l1=x*es
		gen l2=l1+2
		gen l3=l1-2
	twoway (scatter y x,) (scatter y x if refid==218, msize(tiny) mlabel(refid) mlabpos(9) mlabgap(*2)) ///
		(line l1 x, lcolor(black) lwidth(medium)) ///
		(line l2 x, lcolor(black) lwidth(medium) ) ///
		(line l3 x, lcolor(black) lwidth(medium) ), scheme(s1mono) ///
		legend(off) yti("Z = T / SE", m(r+2)) xti("1/SE", m(r+2)) ylabel(, angle(horizontal)) ///
		xlabel(#5)  
		graph save "figs/fig28_glbrth.gph", replace
		graph export "figs/fig28_glbrth.emf", replace
	* removing outlier
	metan md_ln mdse_ln if refid!=218, nograph
	
* Fig 28.4 Scatter plot assessment of heterogeneity
	metan  md_ln mdse_ln, nograph
	format conc_c7 %6.0f 
	twoway (scatter md_ln conc_c7 [aweight=_WT], ///
			msymbol(circle_hollow) msize(small) mcolor(black) mlwidth(medium)) ///
		(scatter md_ln conc_c7 if refid==218, msize(vtiny) mlabel(refid) mlabgap(*8)), ///
			scheme(s1mono) legend(off) xti("chemical concentration (%)", m(t+2)) yti("mean difference in (log) concentration", m(r+2)) ///
			ylabel( , angle(horizontal)) xlabel(0 "none" 1 "<=0.009" 2 "0.01 - 1" 3 "1 - 3" 4 "4 - 9" 5 ">9" , angle(45))
		graph save "figs/fig28_het_scatter.gph", replace
		graph export "figs/fig28_het_scatter.emf", replace

*************************************************************************
* Ex 28.6 Meta-regression
*************************************************************************
* fit null model for comparison
	metareg rr_ln , wsse(rrse_ln) bsest(reml)
* add predictors
	egen pub_date_c4=cut(pub_date), at(1970 1980 1990 2000 9999)
	xi:metareg rr_ln i.pub_date_c4 i.stdy_dsgn i.stdy_pop i.intrv i.disinf_c6  , ///
		wsse(rrse_ln) bsest(reml)
		* study design and publication data forced in
		testparm _Ipub_d*
		testparm _Istdy_d*
		testparm _Istdy_p*	/* note - only 4 studies "uncontam. birds" group */
		testparm _Iintr*
		testparm _Idisi*

*************************************************************************
* Fig 28.5 L'Abbe plot
*************************************************************************
	metan rr_ln rrse_ln if intrv==5, random nograph eform 
		scalar es=r(ES)
	capture drop l1
	gen l1=risk_cntrl*es
	twoway (scatter risk_tx risk_cntrl [aweight=num], msymb(Oh) msize(small) mcolor(black) mlwidth(medium)) ///
		(line l1 risk_cntrl, lwidth(medium) lcolor(black)) ///
		(scatter risk_tx risk_cntrl if risk_tx>20 & risk_cntrl<50, msize(vtiny) mlabel(refid) mlabpos(10)) ///
		(scatter risk_tx risk_cntrl if risk_tx>20 & risk_cntrl>50, msize(vtiny) mlabel(refid) mlabpos(8)) , ///
		legend(off) scheme(s1mono) xti("risk in control group", m(t+2)) xlab(#5) ///
		yti("risk in treated group", m(r+2)) ylab(#5, angle(horizontal))
		graph save "figs/fig28_labbe.gph", replace
		graph export "figs/fig28_labbe.emf", replace
	* adding underlying risk to meta-regression
	xi:metareg rr_ln i.pub_date_c4 i.stdy_dsgn i.stdy_pop i.intrv i.disinf_c6 risk_cntrl , ///
		wsse(rrse_ln) bsest(reml)
		
*************************************************************************
* Section 28.8 Publication bias
*************************************************************************
	metan md_ln mdse_ln, random label(namevar=id)
	* Fig 28.7 funnel plot and Begg's and Egger's tests
		metabias md_ln mdse_ln, begg
		metabias md_ln mdse_ln, egger
		metafunnel md_ln mdse_ln, ytitle("SE of treatment effect", m(r+2)) ///
			ylabel(, angle(horizontal)) xtitle(, m(t+2)) plotregion(style(none)) ///
			scheme(s1mono) text(.1 -4.65 "29")
		graph save "figs/fig28_funnel.gph", replace
		graph export "figs/fig28_funnel.emf", replace
	* Ex 28.8 perform Duval and Tweedie's "Trim and Fill" procedure
		metatrim md_ln mdse_ln , re funnel t2("xyz") save(trim_fill, replace)
			use trim_fill, clear
			* filled points at -4.07, .242 ; -3.91, .981; -3.70, .241)
		metafunnel filled fillse , ytitle("SE of treatment effect", m(r+2)) ///
			ylabel(, angle(horizontal)) xtitle(treatment effect, m(t+2)) plotregion(style(none)) ///
			text(.242 -4.07 "+", size(huge)) text(.981 -3.91 "+", size(huge) ) text(.241 -3.70 "+", size(huge) ) ///
			text(.1 -4.65 "29") scheme(s1mono) name(trimfill2, replace)
				* outlying point in top left is refid=218, id=29
			graph save "figs/fig28_trimfill.gph", replace
			graph export "figs/fig28_trimfill.emf", replace
	use "C:\mer\data\salm_ma.dta", clear

*************************************************************************
* Ex 28.9 Influential studies
*************************************************************************
metan md_ln mdse_ln, random nograph lcols(id)
* eliminate most influential study
metan md_ln mdse_ln if id!=29, random nograph lcols(id)


